@@ -4,8 +4,8 @@ connection to the graphlab backend server.
 """
 
 """ The module usage metric tracking object """
-from graphlab_util.config import DEFAULT_CONFIG as default_local_conf
-from graphlab_util.metric_tracker import MetricTracker
+from graphlab_util.config import DEFAULT_CONFIG as _default_local_conf
+from graphlab_util.metric_tracker import MetricTracker as _MetricTracker
 
 """ The global client object """
 __CLIENT__ = None
@@ -14,7 +14,7 @@ __CLIENT__ = None
 __SERVER__ = None
 
 """ The module usage metric tracking object """
-__USAGE_METRICS__ = MetricTracker(mode=default_local_conf.mode)
+__USAGE_METRICS__ = _MetricTracker(mode=_default_local_conf.mode)
 
 
 def _get_metric_tracker():
