@@ -1,6 +1,6 @@
 """@package graphlab.connect.aws.ec2
 
-This module makes it easy to have a GraphLab server running in EC2.
+This module makes it easy to have a GraphLab server running on an EC2 instance.
 """
 
 import ConfigParser
@@ -212,9 +212,27 @@ def _get_region_from_config(config_path = (os.path.join(os.path.expanduser("~"),
 
 def get_credentials():
     """
-    Returns the AWS credential pairs stored in the environment variables:
-    AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY.
+    Returns the values stored in the AWS credential environment variables.
+    Returns the value stored in the AWS_ACCESS_KEY_ID environment variable and
+    the value stored in the AWS_SECRET_ACCESS_KEY environment variable.
+
+    Returns
+    -------
+    out : tuple [string]
+        The first string of the tuple is the value of the AWS_ACCESS_KEY_ID
+        environment variable. The second string of the tuple is the value of the
+        AWS_SECRET_ACCESS_KEY environment variable.
+
+    See Also
+    --------
+    set_credentials
+
+    Examples
+    --------
+    >>> graphlab.aws.get_credentials()
+    ('RBZH792CTQPP7T435BGQ', '7x2hMqplWsLpU/qQCN6xAPKcmWo46TlPJXYTvKcv')
     """
+
     if (not 'AWS_ACCESS_KEY_ID' in os.environ):
         raise KeyError('No access key found. Please set the environment variable AWS_ACCESS_KEY_ID, or using graphlab.aws.set_credential()')
     if (not 'AWS_SECRET_ACCESS_KEY' in os.environ):
@@ -225,38 +243,59 @@ def get_credentials():
 def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None, auth_token=None,
                security_group=None, tags=None, use_secure_connection=True):
     """
-    Launches an EC2 instance of the specified type. It is expected that AWS credentials are already set
-    as environment variables (see :py:func:`~.graphlab.aws.set_aws_credential()`), or configuration is provided in the Boto config file.
+    Launches an EC2 instance. All subsequent GraphLab operations will be
+    executed on that host. Prior to calling this function, AWS credentials must
+    be set as environment variables (see
+    :py:func:`~graphlab.aws.set_credentials`).
 
     Parameters
     ----------
     instance_type : string, optional
-        The EC2 instance type to launch, default is m3.xlarge. We support all instance types
-        except: 't2.micro', 't2.small', 't2.medium' and 'm3.medium'. For a list of instance_types,
-        please refer to `here <http://aws.amazon.com/ec2/instance-types/#Instance_Types>`_.
+        The EC2 instance type to launch. We support `all instance types
+        <http://aws.amazon.com/ec2/instance-types/#Instance_Types>`_ except:
+        't2.micro', 't2.small', 't2.medium' and 'm3.medium'.
 
     region : string, optional
-        The AWS region in which to launch your instance. Default is 'us-west-2'.
+        The `AWS region
+        <http://docs.aws.amazon.com/general/latest/gr/rande.html#ec2_region>`_
+        in which to launch your instance. Default is 'us-west-2'.
 
-    CIDR_rule : string or list of strings, optional
-        The Classless Inter-Domain Routing rule(s) to use for the instance. Useful for restricting the IP
-        Address Range for a client. Default is no restriction. If you specify CIDR_rule(s), you must also
-        specify a security group to use.
+    CIDR_rule : string or list[string], optional
+        The Classless Inter-Domain Routing rule(s) to use for the instance.
+        Useful for restricting the IP Address Range for a client. Default is no
+        restriction. If you specify CIDR_rule(s), you must also specify a
+        security group to use.
 
     auth_token : string, optional
-        The Authentication Token to be used by the instance. By default a randomly generated token is used.
+        The Authentication Token to be used by the instance. By default a
+        randomly generated token is used.
 
     security_group : string, optional
         The name of the security group for the EC2 instance to use.
 
-    tags : dict, optional
-        A dictionary containing the name/value tag pairs to be assigned to the instance. If you want
-        to create only a tag name, the value for that tag should be the empty string (i.e. ''). 
-        In addition to these specified tags, a 'GraphLab' tag will also be assigned. 
+    tags : dict, optional    
+        A dictionary containing the name/value tag pairs to be assigned to the
+        instance. If you want to create only a tag name, the value for that tag
+        should be the empty string (i.e. ''). In addition to these specified
+        tags, a 'GraphLab' tag will also be assigned.
 
-    use_secure_connection : boolean, optional
-       Determine whether the communication, between your computer and the EC2 instance, should be encrypted. Default is True.
+    use_secure_connection : bool, optional
+       Determine whether the communication, between your computer and the EC2
+       instance, should be encrypted.
+
+    See Also
+    --------
+    terminate_EC2
+
+    Examples
+    --------
+    >>> # Launch a general purpose xlarge host in Oregon.
+    >>> graphlab.aws.launch_EC2()
+
+    >>> # Launch an xlarge compute optimized host in the US East Region.
+    >>> graphlab.aws.launch_EC2(instance_type='c3.xlarge', region='us-east-1')
     """
+
     # Check existing connection
     if glconnect.is_connected():
         __LOGGER__.warning('You have GraphLab objects instantiated on the current server. If you want to \n'
@@ -298,16 +337,27 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
 
 def set_credentials(access_key_id, secret_access_key):
     """
-    Set the AWS credential environment variables:
-    AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY.
+    Sets the AWS credential environment variables.
+
+    Helper function to set the following environment variables: AWS_ACCESS_KEY_ID and
+    AWS_SECRET_ACCESS_KEY. You are also free to set these environment variables directly.
 
     Parameters
     ----------
-    access_key_id : string
-        Value of AWS_ACCESS_KEY_ID
+    access_key_id : str
+        Value for the AWS_ACCESS_KEY_ID environment variable.
+
+    secret_access_key : str
+        Value for the AWS_SECRET_ACCESS_KEY environment variable.
+
+    See Also
+    --------
+    get_credentials
+
+    Examples
+    --------
+    >>> graphlab.aws.set_credentials('RBZH792CTQPP7T435BGQ', '7x2hMqplWsLpU/qQCN6xAPKcmWo46TlPJXYTvKcv')
 
-    secret_access_key : string
-        Value of AWS_SECRET_ACCESS_KEY
     """
     os.environ['AWS_ACCESS_KEY_ID'] = str(access_key_id)
     os.environ['AWS_SECRET_ACCESS_KEY'] = str(secret_access_key)
@@ -316,13 +366,36 @@ def set_credentials(access_key_id, secret_access_key):
 @__catch_and_log__
 def list_instances(region='us-west-2'):
     """
-    Returns GraphLab EC2 instance that are currently running.
-
+    Instance IDs for all active GraphLab EC2 hosts. Returns instance IDs for all
+    active GraphLab EC2 hosts (for the specified region), not just the instance
+    ID associated with the current python session.
+    
     Parameters
     ----------
-    region : string, optional
-        The AWS region in which to list instances. Default is 'us-west-2'.
+    region : string, optional    
+        The `AWS region
+        <http://docs.aws.amazon.com/general/latest/gr/rande.html#ec2_region>`_
+        in which to list instances. Default is 'us-west-2'.
+
+    Returns
+    -------
+    out : list[string]    
+        A list of AWS instance IDs corresponding to active instances in the
+        specified region.
+
+    Examples
+    --------
+    .. sourcecode:: python
+
+        # No instances running in 'us-west-2'.
+        >>> graphlab.aws.list_instances()
+        []
+
+        # Two insances running in 'us-east-1'.
+        >>> graphlab.aws.list_instances(region = 'us-east-1')
+        [u'i-8df559a7', u'i-ed2aa9c1']
     """
+
     __LOGGER__.info("Listing instances in %s." % region)
     aws_connection = boto.ec2.connect_to_region(region)
     recent_ids = [i.res_id for i in aws_connection.get_all_tags(filters={'tag:GraphLab': ''})]
@@ -333,8 +406,19 @@ def list_instances(region='us-west-2'):
 @__catch_and_log__
 def status():
     """
-    Returns the status of any running EC2 instance associated with the current client.
+    Returns the status of any running EC2 instance associated with the current
+    python session.
+
+    Examples
+    --------
+    >>> # With an EC2 instance running.
+    >>> graphlab.aws.status()    
+    u'running'
+
+    >>> # Without an EC2 instance running.
+    >>> graphlab.aws.status()
     """
+
     server = glconnect.get_server()
     if server is None or not isinstance(server, _EC2):
         return 'No instance running.'
@@ -350,7 +434,11 @@ def status():
 
 def terminate_EC2():
     """
-    Brings down the current EC2 instance.
+    Terminates the EC2 instance associated with the current python session.
+
+    Examples
+    --------
+    >>> graphlab.aws.terminate_EC2() 
     """
     server = glconnect.get_server()
     if server is not None and isinstance(server, _EC2):
