@@ -13,6 +13,8 @@ import subprocess
 import commands
 import sys
 import time
+import glob as _glob
+from distutils.util import get_platform as _get_platform
 
 
 class GraphLabServer(object):
@@ -139,10 +141,17 @@ class LocalServer(GraphLabServer):
             arglist.append("--secret_key=%s" % self.secret_key)
 
         env = os.environ.copy()
+        classpath = _get_hadoop_class_path()
         if ("CLASSPATH" in env):
-            env["CLASSPATH"] = _get_hadoop_class_path() + env["CLASSPATH"]
+            env["CLASSPATH"] = env['CLASSPATH'] + (":" + classpath if classpath != '' else '')
         else:
-            env["CLASSPATH"] = _get_hadoop_class_path()
+            env["CLASSPATH"] = classpath
+
+        libjvm_path = _get_libjvm_path()
+        if ('LD_LIBRARY_PATH' in env):
+            env['LD_LIBRARY_PATH'] = env['LD_LIBRARY_PATH'] + (":" + libjvm_path if libjvm_path != '' else '')
+        else:
+            env['LD_LIBRARY_PATH'] = libjvm_path
 
         # Start a local server as a child process.
         if self.server_addr == 'default':
@@ -298,14 +307,69 @@ def _get_server_addr_protocol(addr):
     else:
         return addr.split("://")[0]
 
-
 def _get_hadoop_class_path():
     # Try get the classpath directly from executing hadoop
-    status, output = commands.getstatusoutput('hadoop classpath')
-    if status == 0:
-        return output
-    status, output = commands.getstatusoutput('${HADOOP_HOME}/bin/hadoop classpath')
-    if status == 0:
-        return output
+    try:
+        status, output = commands.getstatusoutput('hadoop classpath')
+        if status != 0:
+            status, output = commands.getstatusoutput('${HADOOP_HOME}/bin/hadoop classpath')
+
+        if status == 0:
+            return __get_expanded_classpath(output)
+    except Exception as e:
+        logging.getLogger(__name__).warning("Exception trying to retrieve Hadoop classpath: %s" % e)
+
     logging.getLogger(__name__).debug("Hadoop not found. HDFS url is not supported. Please make hadoop available from PATH or set the environment variable HADOOP_HOME.")
     return ""
+
+def __get_expanded_classpath(classpath):
+    """
+    Take a classpath of the form:
+      /etc/hadoop/conf:/etc/hadoop/conf:/etc/hadoop/conf:/usr/lib/hadoop/lib/*:/usr/lib/hadoop/.//*: ...
+
+    and return it expanded to all the JARs (and nothing else):
+      /usr/lib/hadoop/lib/netty-3.6.2.Final.jar:/usr/lib/hadoop/lib/jaxb-api-2.2.2.jar: ...
+
+    mentioned in the path
+    """
+    if classpath is None or classpath == '':
+        return ''
+
+    #  so this set comprehension takes paths that end with * to be globbed to find the jars, and then 
+    #  recombined back into a colon separated list of jar paths, removing dupes and using full file paths
+    jars = ':'.join({ ":".join([os.path.abspath(jarpath) for jarpath in _glob.glob(path + '.jar')]) 
+        for path in classpath.split(':') if path.endswith('*')})
+    logging.getLogger(__name__).debug('classpath being used: %s' % jars)
+    return jars
+
+def _get_libjvm_path():
+    """
+    Return path of libjvm.so for LD_LIBRARY_PATH variable.
+    This is only required for Linux platform, and by looking in a bunch of 'known' paths.
+    """
+    cur_platform = _get_platform()
+
+    if cur_platform.startswith("macosx"):
+        return ''
+    else:
+        potential_paths = [ 
+                '/usr/lib/jvm/default-java/jre/lib/amd64/server',               # ubuntu / debian distros
+                '/usr/lib/jvm/java/jre/lib/amd64/server',                       # rhel6
+                '/usr/lib/jvm/jre/lib/amd64/server',                            # centos6
+                '/usr/lib64/jvm/jre/lib/amd64/server',                          # opensuse 13
+                '/usr/local/lib/jvm/default-java/jre/lib/amd64/server',         # alt ubuntu / debian distros
+                '/usr/local/lib/jvm/java/jre/lib/amd64/server',                 # alt rhel6
+                '/usr/local/lib/jvm/jre/lib/amd64/server',                      # alt centos6  
+                '/usr/local/lib64/jvm/jre/lib/amd64/server',                    # alt opensuse 13
+                '/usr/local/lib/jvm/java-7-openjdk-amd64/jre/lib/amd64/server', # alt ubuntu / debian distros
+                '/usr/lib/jvm/java-7-openjdk-amd64/jre/lib/amd64/server',       # alt ubuntu / debian distros
+                '/usr/local/lib/jvm/java-6-openjdk-amd64/jre/lib/amd64/server', # alt ubuntu / debian distros
+                '/usr/lib/jvm/java-6-openjdk-amd64/jre/lib/amd64/server',       # alt ubuntu / debian distros
+            ]
+        for path in potential_paths:
+            if os.path.isfile(os.path.join(path, 'libjvm.so')):
+                logging.getLogger(__name__).debug('libjvm.so path used: %s' % path)
+                return path
+        return ''
+
+                
