@@ -4,6 +4,7 @@ import graphlab.connect.main as glconnect
 from graphlab.cython.cy_sketch import UnitySketchProxy
 from graphlab.cython.context import debug_trace as cython_context
 from graphlab.data_structures.sarray import SArray
+from graphlab.data_structures.sframe import SFrame
 from math import sqrt
 
 __all__ = ['Sketch']
@@ -17,11 +18,13 @@ class Sketch(object):
 
     To construct a Sketch object, the following methods are equivalent:
 
+    >>> my_sarray = graphlab.SArray([1,2,3,4,5])
     >>> sketch = graphlab.Sketch(my_sarray)
     >>> sketch = my_sarray.sketch_summary()
 
     Typically, the SArray is a column of an SFrame:
 
+    >>> my_sframe =  graphlab.SFrame({'column1': [1,2,3]})
     >>> sketch = graphlab.Sketch(my_sframe['column1'])
     >>> sketch = my_sframe['column1'].sketch_summary()
 
@@ -56,42 +59,36 @@ class Sketch(object):
      - frequent items (:func:`~graphlab.Sketch.frequent_items`)
      - frequency count of any value (:func:`~graphlab.Sketch.frequency_count`)
 
-    For SArray of type list, there is a sketch summary for all values inside the
-    list element. Sketch summary flattens all list values and do a sketch summery
-    over flattened values. Each value in list is casted to string for sketch summary.
-    The summary can be retrieved by calling:
-     - element_summary(:func:`~graphlab.Sketch.element_summary`) -- sketch summary of all elements
+    For SArray of type list or array, there is a sub sketch for all sub elements.
+    The sub sketch flattens all list/array values and then computes sketch
+    summary over flattened values. Element sub sketch may be retrieved through:
+     - element_summary(:func:`~graphlab.Sketch.element_summary`)
 
-    For SArray of type array(vector), there is a sketch summary for all values inside
-    vector element. Sketch summary flattens all vector values and do a sketch summery
-    over flattened values. The summary can be retrieved by calling:
-     - element_summary(:func:`~graphlab.Sketch.element_summary`) -- sketch summary of all elements
-
-    For SArray of type dict, additional sketch summary over the keys and values are
-    provided. They can be retrieved by calling:
-     - dict_key_summary(:func:`~graphlab.Sketch.dict_key_summary`) -- sketch summary of all keys in the dictionary
-     - dict_value_summary(:func:`~graphlab.Sketch.dict_value_summary`) -- sketch summary of all values in the dictionary
+    For SArray of type dict, there are sub sketches for both dict key and value.
+    The sub sketch may be retrieved through:
+     - dict_key_summary(:func:`~graphlab.Sketch.dict_key_summary`)
+     - dict_value_summary(:func:`~graphlab.Sketch.dict_value_summary`)
 
     For SArray of type dict, user can also pass in a list of dictionary keys to
-    sketch_summary function, this would cause one sub sketch for each of the key.
+    sketch_summary function, this would generate one sub sketch for each key.
     For example:
 
+         >>> sa = graphlab.SArray([{'a':1, 'b':2}, {'a':3}])
          >>> sketch = sa.sketch_summary(sub_sketch_keys=["a", "b"])
 
     Then the sub summary may be retrieved by:
 
          >>> sketch.element_sub_sketch()
 
-    Or:
+    or to get subset keys:
 
-         >>> sketch.element_sub_sketch(["key1", "key2"])
-
-    for subset of keys
+         >>> sketch.element_sub_sketch(["a"])
 
     Similarly, for SArray of type vector(array), user can also pass in a list of
     integers which is the index into the vector to get sub sketch
     For example:
 
+         >>> sa = graphlab.SArray([[100,200,300,400,500], [100,200,300], [400,500]])
          >>> sketch = sa.sketch_summary(sub_sketch_keys=[1,3,5])
 
     Then the sub summary may be retrieved by:
@@ -110,7 +107,7 @@ class Sketch(object):
     Parameters
     ----------
     array : SArray
-        Array to sketch.
+        Array to generate sketch summary.
 
     background : boolean
       If True, the sketch construction will return immediately and the
@@ -120,12 +117,15 @@ class Sketch(object):
 
     References
     ----------
-    - `Wikipedia. Streaming algorithms. <http://en.wikipedia.org/wiki/Streaming_algorithm>`_
-    - Charikar, et al. (2002) `Finding frequent items in data streams. <https://www.cs.rutgers.edu/~farach/pubs/FrequentStream.pdf>`_
-    - Cormode, G. and Muthukrishnan, S. (2004) `An Improved Data Stream Summary: The Count-Min Sketch and its Applications. <http://dimacs.rutgers.edu/~graham/pubs/papers/cm-latin.pdf>`_
+    - Wikipedia. `Streaming algorithms. <http://en.wikipedia.org/wiki/Streaming_algorithm>`_
+    - Charikar, et al. (2002) `Finding frequent items in data streams.
+      <https://www.cs.rutgers.edu/~farach/pubs/FrequentStream.pdf>`_
+    - Cormode, G. and Muthukrishnan, S. (2004) `An Improved Data Stream Summary:
+      The Count-Min Sketch and its Applications.
+      <http://dimacs.rutgers.edu/~graham/pubs/papers/cm-latin.pdf>`_
     """
 
-    def __init__(self, array=None, background=False, sub_sketch_keys=None, _proxy=None):
+    def __init__(self, array=None, background=False, sub_sketch_keys=[], _proxy=None):
         """__init__(array)
         Construct a new Sketch from an SArray.
 
@@ -134,15 +134,15 @@ class Sketch(object):
         array : SArray
             Array to sketch.
 
-        background: boolean | optional
+        background : boolean, optional
             If true, run the sketch in background. The the state of the sketch
             may be queried by calling (:func:`~graphlab.Sketch.sketch_ready`)
             default is False
 
-        sub_sketch_keys: list of string | list of int
-            The list of sub sketch to calculate, for SArray of dictionary type, the
-            key needs to be a string, for SArray of vector(array) type, the key needs
-            to be positive integer
+        sub_sketch_keys : list
+            The list of sub sketch to calculate, for SArray of dictionary type.
+            key needs to be a string, for SArray of vector(array) type, the key
+            needs to be positive integer
         """
         _mt._get_metric_tracker().track('sketch.init')
         if (_proxy):
@@ -158,67 +158,70 @@ class Sketch(object):
       """
       Emits a brief summary of all the statistics as a string.
       """
-      s = "Exact Values\n"
-      s += "------------\n";
-      s += "Length: " + str(self.size()) + "\n"
-      s += "# Missing Values: " + str(self.num_undefined()) + "\n"
-      try:
-        # these may fail for non-numeric columns
-        s += "Min: " + str(self.min()) + "\n"
-        s += "Max: " + str(self.max()) + "\n"
-        s += "Mean: " + str(self.mean()) + "\n"
-        s += "Variance: " + str(self.var()) + "\n"
-        s += "Standard Deviation: " + str(self.std()) + "\n"
-      except:
-        pass
+      fields = [
+        ['size',           'Length' ,       'Yes'],
+        ['min',            'Min' ,          'Yes'],
+        ['max',            'Max' ,          'Yes'],
+        ['mean',           'Mean' ,         'Yes'],
+        ['sum',            'Sum' ,          'Yes'],
+        ['var',            'Variance' ,     'Yes'],
+        ['std',            'Standard Deviation' , 'Yes'],
+        ['num_undefined', '# Missing Values' , 'Yes',],
+        ['num_unique',     '# unique values',  'No' ]
+      ]
+
+      s = '\n'
+      result = []
+      for field in fields:
+        try:
+          method_to_call = getattr(self, field[0])
+          result.append([field[1], str(method_to_call()), field[2]])
+        except:
+          pass
+      sf = SArray(result).unpack(column_name_prefix = "")
+      sf.rename({'0': 'item', '1':'value', '2': 'is exact'})
+      s += sf.__str__(footer=False)
       s += "\n"
-      s += "Approximate Values\n"
-      s += "------------------\n";
-      s += "# Unique Values: " + str(self.num_unique()) + "\n"
+
+      s += "\nMost frequent items:\n"
       frequent = self.frequent_items()
       sorted_freq = sorted(frequent.iteritems(), key=operator.itemgetter(1), reverse=True)
-      sorted_freq = sorted_freq[:10]
-      s += "Most frequent items (item:count) :\n"
-      s += "\t"
       if len(sorted_freq) == 0:
-          s += " -- All elements appear with less than 0.01% frequency -- "
+          s += " -- All elements appear with less than 0.01% frequency -- \n"
       else:
-          for elem in sorted_freq:
-              s += str(elem[0]) + ":" + str(elem[1]) + "\t"
+        sorted_freq = sorted_freq[:10]
+        sf = SFrame()
+        sf.add_column(SArray(['count']), 'value')
+        for elem in sorted_freq:
+          sf.add_column(SArray([elem[1]]), str(elem[0]))
+        s += sf.__str__(footer=False) + "\n"
       s += "\n"
 
       try:
-        # test if we can query quantiles
+        # print quantiles
         t = self.quantile(0)
         s += "Quantiles: \n"
-        s += "\tMin\t1%\t5%\t25%\t50%\t75%\t95%\t99%\tMax\n"
-        s += "\t"
+        sf = SFrame()
         for q in [0.0,0.01,0.05,0.25,0.5,0.75,0.95,0.99,1.00]:
-          s += str(self.quantile(q)) + "\t"
-        s += "\n"
+          sf.add_column(SArray([self.quantile(q)]), str(int(q * 100)) + '%')
+        s += sf.__str__(footer=False) + "\n"
       except:
         pass
 
       try:
         t_k = self.dict_key_summary()
         t_v = self.dict_value_summary()
-        s += "\n\n================================\n";
-        s += "Dictionary element key summary\n"
-        s += "================================\n";
+        s += "\n******** Dictionary Element Key Summary ********\n"
         s += t_k.__repr__()
-        s += "\n\n================================\n";
-        s += "Dictionary element value summary\n"
-        s += "================================\n";
-        s += t_v.__repr__()
+        s += "\n******** Dictionary Element Value Summary ********\n"
+        s += t_v.__repr__() + '\n'
       except:
         pass
 
       try:
         t_k = self.element_summary()
-        s += "\n\n================================\n";
-        s += "Element summary\n"
-        s += "============================\n";
-        s += t_k.__repr__()
+        s += "\n******** Element Summary ********\n"
+        s += t_k.__repr__() + '\n'
       except:
         pass
 
@@ -389,6 +392,8 @@ class Sketch(object):
         also appear in the result. The item counts are estimated using
         the CountSketch.
 
+        Missing values are not taken into account when copmuting frequent items.
+
         If this function returns no elements, it means that all elements appear
         with less than 0.01% occurrence.
 
@@ -425,7 +430,7 @@ class Sketch(object):
 
         Returns
         -------
-        out : float | string
+        out : float | str
             An estimate of the value at a quantile.
         """
         _mt._get_metric_tracker().track('sketch.quantile.%g' % quantile_val)
@@ -482,31 +487,38 @@ class Sketch(object):
     def element_length_summary(self):
         """
         Returns the sketch summary for the element length. This is only valid for
-        a sketch constructed SArray of type list/array/dict, raises Runtime exception
-        otherwise.
+        a sketch constructed SArray of type list/array/dict, raises Runtime
+        exception otherwise.
 
         Examples
         --------
-        >>> sa = gl.SArray([[j for j in range(i)] for i in range(1,1000)])
+        >>> sa = graphlab.SArray([[j for j in range(i)] for i in range(1,1000)])
         >>> sa.sketch_summary().element_length_summary()
-        Exact Values
-        ------------
-        Length: 999
-        # Missing Values: 0
-        Min: 1.0
-        Max: 999.0
-        Mean: 500.0
-        Variance: 83166.6666667
-        Standard Deviation: 288.386314978
-        <BLANKLINE>
-        Approximate Values
-        ------------------
-        # Unique Values: 992
-        Most frequent items (item:count) :
-                1:1   2:1   3:1   4:1   5:1   6:1   7:1   8:1   9:1   10:1
+        +--------------------+---------------+----------+
+        |        item        |     value     | is exact |
+        +--------------------+---------------+----------+
+        |       Length       |      999      |   Yes    |
+        |        Min         |      1.0      |   Yes    |
+        |        Max         |     999.0     |   Yes    |
+        |        Mean        |     500.0     |   Yes    |
+        |        Sum         |    499500.0   |   Yes    |
+        |      Variance      | 83166.6666667 |   Yes    |
+        | Standard Deviation | 288.386314978 |   Yes    |
+        |  # Missing Values  |       0       |   Yes    |
+        |  # unique values   |      992      |    No    |
+        +--------------------+---------------+----------+
+        Most frequent items:
+        +-------+---+---+---+---+---+---+---+---+---+----+
+        | value | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 |
+        +-------+---+---+---+---+---+---+---+---+---+----+
+        | count | 1 | 1 | 1 | 1 | 1 | 1 | 1 | 1 | 1 | 1  |
+        +-------+---+---+---+---+---+---+---+---+---+----+
         Quantiles:
-                Min    1%     5%     25%    50%    75%     95%     99%     Max
-                1.0    10.0   50.0   250.0  500.0  750.0   950.0   990.0   999.0
+        +-----+------+------+-------+-------+-------+-------+-------+-------+
+        |  0% |  1%  |  5%  |  25%  |  50%  |  75%  |  95%  |  99%  |  100% |
+        +-----+------+------+-------+-------+-------+-------+-------+-------+
+        | 1.0 | 10.0 | 50.0 | 250.0 | 500.0 | 750.0 | 950.0 | 990.0 | 999.0 |
+        +-----+------+------+-------+-------+-------+-------+-------+-------+
 
         Returns
         -------
@@ -520,24 +532,27 @@ class Sketch(object):
 
     def dict_key_summary(self):
         """
-        Returns the sketch summary for all dictionary keys. This is only valid for
-        sketch object from an SArray of dict type. Dictionary keys are converted
-        to strings and then do the sketch summary.
+        Returns the sketch summary for all dictionary keys. This is only valid
+        for sketch object from an SArray of dict type. Dictionary keys are
+        converted to strings and then do the sketch summary.
 
         Examples
         --------
-        >>> sa = gl.SArray([{'I':1, 'love': 2}, {'nature':3, 'beauty':4}])
+        >>> sa = graphlab.SArray([{'I':1, 'love': 2}, {'nature':3, 'beauty':4}])
         >>> sa.sketch_summary().dict_key_summary()
-        Exact Values
-        ------------
-        Length: 4
-        # Missing Values: 0
-        <BLANKLINE>
-        Approximate Values
-        ------------------
-        # Unique Values: 4
-        Most frequent items (item:count) :
-                I:1     love:1  beauty:1      nature:1
+        +------------------+-------+----------+
+        |       item       | value | is exact |
+        +------------------+-------+----------+
+        |      Length      |   4   |   Yes    |
+        | # Missing Values |   0   |   Yes    |
+        | # unique values  |   4   |    No    |
+        +------------------+-------+----------+
+        Most frequent items:
+        +-------+---+------+--------+--------+
+        | value | I | love | beauty | nature |
+        +-------+---+------+--------+--------+
+        | count | 1 |  1   |   1    |   1    |
+        +-------+---+------+--------+--------+
 
         """
         _mt._get_metric_tracker().track('sketch.dict_key_summary')
@@ -546,36 +561,41 @@ class Sketch(object):
 
     def dict_value_summary(self):
         """
-        Returns the sketch summary for all dictionary values. This is only valid for
-        sketch object from an SArray of dict type.
+        Returns the sketch summary for all dictionary values. This is only valid
+        for sketch object from an SArray of dict type.
 
-        Type of value summary is inferred from first set of values. If the values
-        are all numeric, then the sketch summary will calculate numeric characteritics
-        of the values. Otherwise, the dict values are converted to string for sketch
-        summary. Missing values are counted accordingly too.
+        Type of value summary is inferred from first set of values.
 
         Examples
         --------
-        >>> sa = gl.SArray([{'I':1, 'love': 2}, {'nature':3, 'beauty':4}])
+
+        >>> sa = graphlab.SArray([{'I':1, 'love': 2}, {'nature':3, 'beauty':4}])
         >>> sa.sketch_summary().dict_value_summary()
-        Exact Values
-        ------------
-        Length: 4
-        # Missing Values: 0
-        Min: 1.0
-        Max: 4.0
-        Mean: 2.5
-        Variance: 1.25
-        Standard Deviation: 1.11803398875
-        <BLANKLINE>
-        Approximate Values
-        ------------------
-        # Unique Values: 4
-        Most frequent items (item:count) :
-                1.0:1   2.0:1   3.0:1   4.0:1
+        +--------------------+---------------+----------+
+        |        item        |     value     | is exact |
+        +--------------------+---------------+----------+
+        |       Length       |       4       |   Yes    |
+        |        Min         |      1.0      |   Yes    |
+        |        Max         |      4.0      |   Yes    |
+        |        Mean        |      2.5      |   Yes    |
+        |        Sum         |      10.0     |   Yes    |
+        |      Variance      |      1.25     |   Yes    |
+        | Standard Deviation | 1.11803398875 |   Yes    |
+        |  # Missing Values  |       0       |   Yes    |
+        |  # unique values   |       4       |    No    |
+        +--------------------+---------------+----------+
+        Most frequent items:
+        +-------+-----+-----+-----+-----+
+        | value | 1.0 | 2.0 | 3.0 | 4.0 |
+        +-------+-----+-----+-----+-----+
+        | count |  1  |  1  |  1  |  1  |
+        +-------+-----+-----+-----+-----+
         Quantiles:
-                Min     1%      5%      25%     50%     75%     95%     99%     Max
-                1.0     1.0     1.0     2.0     3.0     4.0     4.0     4.0     4.0
+        +-----+-----+-----+-----+-----+-----+-----+-----+------+
+        |  0% |  1% |  5% | 25% | 50% | 75% | 95% | 99% | 100% |
+        +-----+-----+-----+-----+-----+-----+-----+-----+------+
+        | 1.0 | 1.0 | 1.0 | 2.0 | 3.0 | 4.0 | 4.0 | 4.0 | 4.0  |
+        +-----+-----+-----+-----+-----+-----+-----+-----+------+
 
         """
         _mt._get_metric_tracker().track('sketch.dict_value_summary')
@@ -592,26 +612,33 @@ class Sketch(object):
 
         Examples
         --------
-        >>> sa = SArray([[1,2,3], [4,5]])
+        >>> sa = graphlab.SArray([[1,2,3], [4,5]])
         >>> sa.sketch_summary().element_summary()
-        Exact Values
-        ------------
-        Length: 5
-        # Missing Values: 0
-        Min: 1.0
-        Max: 5.0
-        Mean: 3.0
-        Variance: 2.0
-        Standard Deviation: 1.41421356237
-        <BLANKLINE>
-        Approximate Values
-        ------------------
-        # Unique Values: 5
-        Most frequent items (item:count) :
-                1.0:1   2.0:1   3.0:1   4.0:1   5.0:1
+        +--------------------+---------------+----------+
+        |        item        |     value     | is exact |
+        +--------------------+---------------+----------+
+        |       Length       |       5       |   Yes    |
+        |        Min         |      1.0      |   Yes    |
+        |        Max         |      5.0      |   Yes    |
+        |        Mean        |      3.0      |   Yes    |
+        |        Sum         |      15.0     |   Yes    |
+        |      Variance      |      2.0      |   Yes    |
+        | Standard Deviation | 1.41421356237 |   Yes    |
+        |  # Missing Values  |       0       |   Yes    |
+        |  # unique values   |       5       |    No    |
+        +--------------------+---------------+----------+
+        Most frequent items:
+        +-------+-----+-----+-----+-----+-----+
+        | value | 1.0 | 2.0 | 3.0 | 4.0 | 5.0 |
+        +-------+-----+-----+-----+-----+-----+
+        | count |  1  |  1  |  1  |  1  |  1  |
+        +-------+-----+-----+-----+-----+-----+
         Quantiles:
-                Min     1%      5%      25%     50%     75%     95%     99%     Max
-                1.0     1.0     1.0     2.0     3.0     4.0     5.0     5.0     5.0
+        +-----+-----+-----+-----+-----+-----+-----+-----+------+
+        |  0% |  1% |  5% | 25% | 50% | 75% | 95% | 99% | 100% |
+        +-----+-----+-----+-----+-----+-----+-----+-----+------+
+        | 1.0 | 1.0 | 1.0 | 2.0 | 3.0 | 4.0 | 5.0 | 5.0 | 5.0  |
+        +-----+-----+-----+-----+-----+-----+-----+-----+------+
         """
         _mt._get_metric_tracker().track('sketch.element_summary')
         with cython_context():
@@ -619,21 +646,19 @@ class Sketch(object):
 
     def element_sub_sketch(self, keys = None):
         """
-        Returns the sub sketch for the given set of keys. Sub sketch is the sketch
-        for a given key (for dictionary type) or index (for array type) in an SArray
-        of either dictionary or array
+        Returns the sketch summary for the given set of keys. This is only
+        applicable for sketch summary created from SArray of sarray or dict type.
+        For dict SArray, the keys are the keys in dict value.
+        For array Sarray, the keys are indexes into the array value.
 
-        The sub sketch is only available for sketch created from SArray of dictionary
-        or array type.
-
-        The keys must be passed into original sketch_summary() call in order to be
-        able to be retrieved later
+        The keys must be passed into original sketch_summary() call in order to
+        be able to be retrieved later
 
         Parameters
         -----------
-        keys: list of str | str | list of int | int
-            The list of dictionary keys or array index to get sub sketch from. If
-            not given, then retrieve all sub sketches that are available
+        keys : list of str | str | list of int | int
+            The list of dictionary keys or array index to get sub sketch from.
+            if not given, then retrieve all sub sketches that are available
 
         Returns
         -------
@@ -642,28 +667,35 @@ class Sketch(object):
 
         Examples
         --------
-        >>> sa = SArray([{'a':1, 'b':2}, {'a':4, 'd':1}])
+        >>> sa = graphlab.SArray([{'a':1, 'b':2}, {'a':4, 'd':1}])
         >>> s = sa.sketch_summary(sub_sketch_keys=['a','b'])
         >>> s.element_sub_sketch(['a'])
-        {'a': Exact Values
-        ------------
-        Length: 2
-        # Missing Values: 0
-        Min: 1.0
-        Max: 4.0
-        Mean: 2.5
-        Variance: 2.25
-        Standard Deviation: 1.5
-        <BLANKLINE>
-        Approximate Values
-        ------------------
-        # Unique Values: 2
-        Most frequent items (item:count) :
-                1.0:1   4.0:1
-        Quantiles:
-                Min     1%      5%      25%     50%     75%     95%     99%     Max
-                1.0     1.0     1.0     1.0     4.0     4.0     4.0     4.0     4.0
-        }
+        {'a':
+         +--------------------+-------+----------+
+         |        item        | value | is exact |
+         +--------------------+-------+----------+
+         |       Length       |   2   |   Yes    |
+         |        Min         |  1.0  |   Yes    |
+         |        Max         |  4.0  |   Yes    |
+         |        Mean        |  2.5  |   Yes    |
+         |        Sum         |  5.0  |   Yes    |
+         |      Variance      |  2.25 |   Yes    |
+         | Standard Deviation |  1.5  |   Yes    |
+         |  # Missing Values  |   0   |   Yes    |
+         |  # unique values   |   2   |    No    |
+         +--------------------+-------+----------+
+         Most frequent items:
+         +-------+-----+-----+
+         | value | 1.0 | 4.0 |
+         +-------+-----+-----+
+         | count |  1  |  1  |
+         +-------+-----+-----+
+         Quantiles:
+         +-----+-----+-----+-----+-----+-----+-----+-----+------+
+         |  0% |  1% |  5% | 25% | 50% | 75% | 95% | 99% | 100% |
+         +-----+-----+-----+-----+-----+-----+-----+-----+------+
+         | 1.0 | 1.0 | 1.0 | 1.0 | 4.0 | 4.0 | 4.0 | 4.0 | 4.0  |
+         +-----+-----+-----+-----+-----+-----+-----+-----+------+}
         """
         single_val = False
         if keys == None:
@@ -680,6 +712,11 @@ class Sketch(object):
         with cython_context():
             ret_sketches = self.__proxy__.element_sub_sketch(keys)
             ret = {}
+
+            # check return key matches input key
+            for key in keys:
+              if key not in ret_sketches:
+                raise KeyError("Cannot retrieve element sub sketch for key '" + str(key) + "'. Element sub sketch can only be retrieved when the sketch_summary object was created using the 'sub_sketch_keys' option.")
             for key in ret_sketches:
                 ret[key] = Sketch(_proxy = ret_sketches[key])
 
