@@ -14,16 +14,15 @@ from inspect import isfunction as _isfunction
 import _timer as _timer
 import graphlab as _gl
 
-
 __LOGGER__ = logging.getLogger(__name__)
 
 
 class Task(object):
     """
-    A Task represents a unit-of work, capturing a block of code to run.
+    A Task encapsulates a block of code.
 
-    A task is a modular, composable, reusable component that encapsulates a
-    unit-of-work. It allows loose coupling between other tasks, and is the
+    A Task is a modular, composable, reusable component that encapsulates a
+    unit-of-work. It allows loose coupling between other Tasks, and is the
     robust building block for building GraphLab Data Pipelines.
 
     Tasks allow for code to logically reference inputs, outputs, and parameters
@@ -33,16 +32,36 @@ class Task(object):
     having a single responsibility in the pipeline, with the tasks loosely
     coupled. All inputs, outputs, and parameters can be bound at runtime.
 
-    An example Task definition, for generating random names and building SFrame
-    of them:
+    Parameters
+    ----------
+    name : str
+        Name of the Task, this is how the Task will be retrieved in subsequent
+        retrievals.
+
+    description : str, optional
+        Description for this Task, optional.
+
+    Returns
+    -------
+    out : :class:`~graphlab.deploy.Task`
+        A new Task.
+
+    Notes
+    -----
+    - Task names must be unique.
+
+    Examples
+    --------
+    An example Task definition, for generating random names and building an
+    :class:`~graphlab.SFrame` from them:
 
     >>> def name_task(task):
     >>>    import names
-    >>>    people = [ names.get_full_name() for i in range(task.params['number']) ]
+    >>>    people = [names.get_full_name() for i in range(task.params['number'])]
     >>>    sf = graphlab.SFrame({'names':people})
     >>>    task.outputs['names'] = sf
     >>>    print sf
-    >>> 
+    ... 
     >>> names = graphlab.deploy.Task('names')
     >>> names.set_required_packages(['names == 0.3.0'])
     >>> names.set_code(name_task)
@@ -52,14 +71,7 @@ class Task(object):
 
     def __init__(self, name, description=None):
         """
-        Create a task, specifying its name and session.
-
-        Parameters
-        ----------
-        name : string (unique)
-            Name of the task, this is how the task will be retrieved in subsequent retrievals
-        description : string (optional)
-            Description for this Task, optional.
+        Create a new Task specifying its name and optionally a description.
         """
         if name is None or name == "":
             raise TypeError("Name is required when creating a Task.")
@@ -87,7 +99,18 @@ class Task(object):
         self._session.register(self)
 
     def set_name(self, name):
-        """Sets the name of the task, which must be unique."""
+        """
+        Set the name of the Task, which must be unique.
+
+        Parameters
+        ----------
+        name : str
+            Name of the Task.
+
+        Returns
+        -------
+        self : Task
+        """
         if not isinstance(name, str):
             raise TypeError('Name must be a string')
 
@@ -96,7 +119,18 @@ class Task(object):
         return self
 
     def set_description(self, description):
-        """Set the description for this task."""
+        """
+        Set the description for this Task.
+
+        Parameters
+        ----------
+        description : str
+            A description for the Task.
+
+        Returns
+        -------
+        self : Task
+        """
         if not isinstance(description, str):
             raise TypeError('Description must be a string')
 
@@ -106,19 +140,19 @@ class Task(object):
 
     def clone(self, name):
         """
-        Clones current object to new task, with the specified name.
+        Make a copy of the current Task with a new name. The new task will have
+        different name, but will share all other task metadata like inputs,
+        outputs, description, and code.
 
         Parameters
         ----------
-        name : string
-            Name of the cloned task
-        session_aware : boolean
-            Should the session be aware of cloned task. False should only be used
-            when the task is being cloned to be run.
+        name : str
+            Name of the cloned Task.
 
         Returns
         -------
-        Newly cloned task instance.
+        out
+            A newly cloned Task instance.
         """
         return self._clone(name, session_aware=True)
 
@@ -136,53 +170,67 @@ class Task(object):
 
     def set_inputs(self, names, delete=False):
         """
-        Set input(s) for this task.
+        Set input(s) for this Task.
 
-        Inputs are required to be GraphLab Data structures (SFrame, SGraph,
-        Model, SArray), and must come from other Tasks. That means each 
-        input must be defined as a dependency to another Task's output.
+        Inputs are required to be GraphLab Data structures
+        (:class:`~graphlab.SFrame`, :class:`~graphlab.SGraph`,
+        :class:`~graphlab.SArray`, Model), and must come from other Tasks. That
+        means each input must be defined as a dependency to another Task's
+        output.
 
         Parameters
         ----------
-        names : list(str) | dict(str, (str|Task, str) | (str|Task) )
-            If a list of strings is provided then each entry is considered a
-            name for an input slot, and the value for that slot is None.
+        names : list [str] | dict [str, (str|Task, str) | (str|Task)]
+            Input Tasks can be referenced either by name or by reference and
+            the name for the slot in which that data can be found in the input
+            Task can be made explicit or left implicit ('output'). If a dict is
+            provided, then each key is considered a name for an input in this
+            Task, and each value is considered the definition of the input.
+            When a list is provided,  then each entry is considered a name for
+            an input in this Task, and the value for that slot is set to None.
 
-            Example of valid list input definitions:
-
-            >>> t.set_inputs(['one', 'two', 'three'])
-            
-            If a dict is provided, then each key is considered a name for an
-            input, and each value is considered the definition of the input.
-
-            Values for the dict can be either of the following:
-            
-                - a pair representing a Task, and its corresponding output that
-                  we wish to pass along as input
-                - or simply a Task, in which case the output name is assumed to
-                  be 'output'
-
-            Tasks can be referred to either by name (a string) or directly by
-            reference.
-
-            Example of different valid dict input definitions:
-
-            >>> t.set_inputs({
-            >>>  # Task named "prev", ouput named "out"
-            >>>  'a' : ('prev', 'out'),
-            >>>  # Task named "another", implied output named "output"
-            >>>  'b' : 'another', 
-            >>>  # Task t, implied output named "output"
-            >>>  'c' : t, 
-            >>>  # Task t, output named "dw"
-            >>>  'd' : (t, 'dw')}) 
-
-        delete : Boolean (optional)
-            If delete is set to True then the names parameters are removed.
+        delete : bool, optional
+            If delete is set to True then the named parameters are removed.
 
         Returns
         -------
         self : Task
+
+        See Also
+        --------
+        set_outputs
+
+        Examples
+        --------
+        To define only input names for a task, use a list of strings:
+
+        >>> t1 = graphlab.deploy.Task('set_inputs_ex1')
+        >>> t1.set_inputs(['one', 'two', 'three'])
+
+        Values for the dict can be either of the following:
+
+        - a pair representing a Task, and its corresponding output that we wish
+          to pass along as input
+
+        - or simply a Task, in which case the output name is assumed to be
+          'output'
+        
+        Tasks can be referred to either by name (a string) or directly by
+        reference. Example of different valid dict input definitions:
+
+        >>> t2 = graphlab.deploy.Task('set_inputs_ex2')
+        >>> t2.set_outputs(['out', 'output', 'dw'])
+        
+        >>> t3 = graphlab.deploy.Task('set_inputs_ex3')
+        >>> t3.set_inputs({
+        ...     # Task named "set_inputs_ex2", output named "out"
+        ...     'a' : ('set_inputs_ex2', 'out'),
+        ...     # Task named "set_inputs_ex2", implied output named "output"
+        ...     'b' : 'set_inputs_ex2', 
+        ...     # Task t2, implied output named "output"
+        ...     'c' : t2, 
+        ...     # Task t2, output named "dw"
+        ...     'd' : (t2, 'dw')}) 
         """
         if names is None:
             raise TypeError('names parameter is required for set_inputs')
@@ -200,12 +248,12 @@ class Task(object):
 
     def _set_one_input(self, name='input', from_task=None, delete=False):
         """
-        Set/Update an input for this task.
+        Set/Update an input for this Task.
 
         Parameters
         ----------
 
-        name : string
+        name : str
             Name for this input. This will be how the code refers to this input at runtime.
             Default is 'input'.
 
@@ -219,10 +267,10 @@ class Task(object):
 
             >>> task.set_input(name='in', from=('dep', 'out'))
             
-            then an input named 'in' will be defined on this task, which 
+            then an input named 'in' will be defined on this Task, which 
             has a dependency on the output named 'out' from Task named 'dep'.
 
-            If input specified using an object:
+            If input is specified using an object:
             
             >>> t1 = graphlab.deploy.Task('t1').set_output('out')
             >>> t2.set_input(name='in', from=(t1, 'out'))
@@ -230,7 +278,7 @@ class Task(object):
             then an input for t2 would be defined as being the 'out' output 
             of t1.
 
-        delete : Boolean (optional)
+        delete : bool, optional
             If delete is set to True then the name input is removed.
         """
         if delete is True and name in self._data['inputs']:
@@ -278,20 +326,20 @@ class Task(object):
 
     def _set_input(self, name, task=None, output_name=None):
         """
-        Set/Update an input for this task, by name.
+        Set/Update an input for this Task, by name.
 
         To remove an input, simply provide its name with no other parameters.
     
         Parameters
         ----------
-        name : string
-            Name of the task. This must be unique for this session.
+        name : str
+            Name of the Task. This must be unique for this session.
 
-        task : string
-            Name of the Task task to use as an input.
+        task : str
+            Name of the Task to use as an input.
 
-        output_name : string
-            Name of output from input task. If task or pipeline is set, this is required.
+        output_name : str
+            Name of output from input Task. If Task or pipeline is set, this is required.
 
         Returns
         -------
@@ -312,28 +360,62 @@ class Task(object):
 
     def set_code(self, code, function_name='run', directory=None, pattern="*.py"):
         """
-        Sets the code block to run when task is executed.
+        Set the code block to run when Task is executed.
 
         The code to be run needs to be a function that takes one argument. When
         this function is called, the argument will be an object with three
         dictionary attributes (inputs, outputs, params). The inputs dictionary will
         have instantiated data sources by name. The outputs dictionary needs to
         be assigned by name to the results to save. The params dictionary
-        allows capturing any additional keys/values with the task.
+        allows capturing any additional keys/values with the Task.
+        
+        Parameters
+        ----------
+        code : function | string 
+            Function to be called when this Task is executed. This can
+            either be a defined function or it can be a full file path. As a
+            file path this method will inspect the file and confirm it has a
+            function named according to the parameter function_name, which must
+            take only one argument. This function will then also take, by
+            default, the working directory of that file and run find_packages()
+            on it, collecting all the source files indicated. If code is set to
+            None, then this Task's code is removed.
 
+        function_name : str, optional
+            Name of the function to call. Default value is 'run'.
+
+        directory : os.path, optional
+            Optional parameter for the directory from which to grab the pattern
+            of files.
+        
+        pattern : str, optional
+            Optional parameter for a "glob" pattern (ex. *.py) to specify the
+            subset of files from which code should be captured. By default, all
+            .py files from the same directory as the file containing the
+            specified function are captured. This pattern is matched using
+            ``fnmatch``, so rules from fnmatch documentation apply.
+
+        Returns
+        -------
+        self : Task
+
+        Examples
+        --------
         Using a defined function:
 
         >>> def func(task):
         >>>     input = task.inputs['input']
         >>>     task.outputs['output'] = input.apply(lambda x : x * 2)
+        
+        >>> t1 = graphlab.deploy.Task("set_code_ex1")
         >>> t1.set_code(func)
         
         Another example, with different named tasks:
 
-        >>> task = graphlab.deploy.Task('example2')
-        >>> task.set_inputs(['one', 'two', 'three'])
-        >>> task.set_outputs(['result_one', 'result_two'])
-        >>>
+        >>> t2 = graphlab.deploy.Task('set_code_ex2')
+        >>> t2.set_inputs(['one', 'two', 'three'])
+        >>> t2.set_outputs(['result_one', 'result_two'])
+
         >>> def func(task):
         >>>     one = task.inputs['one']
         >>>     two = task.inputs['two']
@@ -342,44 +424,18 @@ class Task(object):
         >>>     task.outputs['result_one'] = first_result
         >>>     task.outputs['result_two'] = second_result
         
-        Using a file with a 'run' function:
+        The example below illustrates how to specify code in a file:
 
-        >>> t.set_code('/full/path/to/munge.py')
+        >>> t3 = graphlab.deploy.Task('set_code_ex3')
+        >>> t3.set_code('/full/path/to/munge.py')
         
         Using a file with a custom fuction name, and custom directory and
         pattern to include:
-        
-        >>> s.set_code('/full/path/to/transform.py', function_name='to_csv', 
-        >>>            directory='src', pattern='*.py')
-        
-        Parameters
-        ----------
-        code : function | string 
-            Code function to be called when this task is executed. This can
-            either be a defined function or it can be a full file path. As a
-            file path this method will inspect the file and confirm it has a
-            function named 'run' which takes one argument. This function will
-            then also take, by default, the working directory of that file and
-            run find_packages() on it, collecting all the source files
-            indicated. If None provided here then code is removed.
 
-        function_name : string (optional)
-            Name of the function to call, if not using 'run'. 
+        >>> t4 = graphlab.deploy.Task('set_code_ex4')
+        >>> t4.set_code('/full/path/to/transform.py', function_name='to_csv', 
+        >>>             directory='src', pattern='*.py')
 
-        directory : os.path (optional)
-            Optional parameter for the directory from which to grab the pattern
-            of files.
-        
-        pattern : string (optional)
-            Optional parameter for a "glob" pattern (ex. *.py) to specify the
-            subset of files from which code should be captured. Default will be
-            to capture the entire directory from where the file is defined, and
-            only the .py files. This pattern is matched using ``fnmatch``, so
-            rules from fnmatch documentation applies.
-
-        Returns
-        -------
-        task : Task, iteslf
         """
         if code is None:
             keys = ['code', 'codestr', 'funcname', 'filename']
@@ -447,28 +503,31 @@ class Task(object):
 
     def set_required_packages(self, packages=None, delete=False):
         """
-        Sets the required packages for this task. List of packages (in same
-        format as ``distutils.requires``) required for running this Task.
-
-        When running this task in a remote setting/environment, care is
-        taken to make sure these required packages are installed prior to
-        execution.
-
-        This method is not sophisticated or analyzing this set to look for duplicates.
-        Meaning, if you specify numpy==1.1.0 and numpy==1.3.0 the API will not complain.
-
-        >>> ex. ['numpy==1.8.0', 'pandas==0.13.1']
+        Set the required packages for running this Task. When running this
+        Task in a remote setting/environment, these packages are installed
+        prior to execution.
 
         Parameters
         ----------
-        packages : string iterable (list/set) 
+        packages : list | set [str]
             List of package requirements (same as disutils.requires) format for
-            packages required for running this task. If None is passed in
-            for packages then current requirement packages removed.
+            packages required for running this Task. If packages is set to None,
+            then this Task's current required packages are removed.
 
         Returns
         -------
         self : Task
+
+        Notes
+        -----
+        - No validation is performed on the specified packages list. So for
+          example, if a required package is specified twice with two different
+          versions, this API will not complain.
+
+        Examples
+        --------
+        >>> t = graphlab.deploy.Task('set_required_packages_ex1')
+        >>> t.set_required_packages(['numpy==1.8.0', 'pandas==0.13.1'])
         """
         if delete is True:
             self._data['packages'].clear()
@@ -480,8 +539,9 @@ class Task(object):
 
     def set_outputs(self, names, delete=False):
         """
-        Set/update output(s) for this task. Outputs are GraphLab Data
-        Structures (SGraph, SFrame, SArray, Model).
+        Set/update output(s) for this Task. Outputs are GraphLab Data
+        Structures (:class:`~graphlab.SFrame`, :class:`~graphlab.SGraph`,
+        :class:`~graphlab.SArray`, Model).
     
         Parameters
         ----------
@@ -489,26 +549,34 @@ class Task(object):
             If a list of strings is provided then each entry is considered a
             name for an output slot, and the value for that slot is None.
 
-            Example of valid list output definitions:
-
-            >>> t.set_outputs(['one', 'two', 'three'])
-            
-            If a dict is provided, then each key is considered a name for an
-            output, and each value is the path for that output.
-
-            Examples of setting dict output definitions:
-
-            >>> t.set_outputs({'one'   : '/tmp/path',
-            >>>                'two'   : 's3://file/path.csv',
-            >>>                'three' : None,
-            >>>                'four'  : 'hdfs://host/path'}) 
-
-        delete : Boolean (optional)
+        delete : bool, optional
             If delete is set to True then the names parameters are removed.
 
         Returns
         -------
         self : Task
+
+        Examples
+        --------
+        A few examples of valid list output definitions:
+
+        >>> t = graphlab.deploy.Task("set_outputs_ex_1")
+        >>> t.set_outputs(['one', 'two', 'three'])
+
+        If a dict is provided, then each key is considered a name for an
+        output, and each value is the path for that output.
+
+        Examples of setting dict output definitions:
+
+        >>> t = graphlab.deploy.Task("set_outputs_ex_2")
+        >>> t.set_outputs({'one'   : '/tmp/path',
+        >>>                'two'   : 's3://file/path.csv',
+        >>>                'three' : None,
+        >>>                'four'  : 'hdfs://host/path'}) 
+
+        See Also
+        --------
+        set_inputs
         """
         if names is None:
             raise TypeError('names is a required parameter for set_outputs and cannot be None')
@@ -526,14 +594,15 @@ class Task(object):
 
     def _set_one_output(self, name='output', path=None, delete=False):
         """
-        Set/update a GraphLab data structure output (SFrame or Model). Outputs
+        Set/update a GraphLab data structure output (:class:`~graphlab.SFrame`,
+        :class:`~graphlab.SGraph`, :class:`~graphlab.SArray`, Model). Outputs
         are for GraphLab data structures.
 
         Parameters
         ----------
-        name : string
+        name : str
             Named output parameter, default to 'output'
-        path : string (optional)
+        path : str, optional
             Path to save output. If None specified then named output is
             removed. Paths can include local, S3, or HDFS.
 
@@ -550,7 +619,7 @@ class Task(object):
             
             >>> t1.set_output('out', path='hdfs:///tmp/save-on-hdfs-here')
 
-        delete : Boolean (optional)
+        delete : bool, optional
             Delete the named output. If set to True then the output by that name is deleted.
 
         Returns
@@ -573,20 +642,29 @@ class Task(object):
 
     def save(self):
         """
-        Saves changes made to this task to disk.
+        Save changes made to this Task to disk.
+
+        When a Task is instantiated, it lives in memory and will not be
+        persisted to disk unless save is called explicitly. Persisted Tasks can
+        be reloaded after server restarts, whereas unsaved Tasks are lost.
+
+        See Also
+        --------
+        graphlab.deploy.tasks.load
         """
         self._session.save(self, typename=type(self).__name__)
         __LOGGER__.info("Task saved successfully.")
 
     def set_params(self, names, delete=False):
         """
-        Add/sets parameter(s) for this task.
+        Add/set parameter(s) for this Task.
 
         Parameters
         ----------
         params : dict
             Dictionary with key/values (strings), merged into current params.
-        delete : boolean
+
+        delete : bool, optional
             To delete a key, set to True and pass params with dictionary. To
             delete all keys, pass None for params and delete=True.
 
@@ -608,15 +686,16 @@ class Task(object):
 
     def _set_params(self, params=None, delete=False):
         """
-        Adds/sets params with the task.
+        Add/set params with the Task.
 
         Parameters
         ----------
         params : dict
             Dictionary with key/values (strings), merged into current params.
-        delete : boolean
-            To delete a key, set to True and pass params with dictionary. To delete all keys,
-            pass None for params and delete=True.
+
+        delete : bool
+            To delete a key, set to True and pass params with dictionary. To
+            delete all keys, pass None for params and delete=True.
 
         Returns
         -------
@@ -646,103 +725,105 @@ class Task(object):
 
     def get_name(self):
         """
-        Returns the name of this task.
+        Return the name of this Task.
         """
         return str(self.name)
 
     def get_description(self):
         """
-        Returns the description of this task.
+        Return the description of this Task.
         """
         return str(self._data['description'])
 
     def get_inputs(self):
         """
-        Returns a dictionary of inputs, by name
+        Return a dictionary of inputs, by name.
         """
         return dict.copy(self._data['inputs'])
 
     def get_code(self):
         """
-        Returns a human-readable string indicating the code that is set for
-        this Task
+        Return a human-readable string indicating the code that is set for
+        this Task.
         """
         return self.__code_str__()
 
     def _get_runnable_code(self):
         """
-        Returns machine-callable version of the code stored in Task. Not
+        Return machine-callable version of the code stored in Task. Not
         intended for interactive usage.
         """
         return self._data['code']
 
     def get_required_packages(self):
         """
-        Returns the set of packages (in distutils.requires format) required for
-        this task.
+        Return the set of packages (in distutils.requires format) required for
+        this Task.
+
+        References
+        ----------
+        - `Python distutils <https://docs.python.org/2/library/distutils.html>`_
         """
         return set.copy(self._data['packages'])
 
     def get_outputs(self):
         """
-        Returns output dictionary for the named outputs for this task.
+        Return output dictionary for the named outputs for this Task.
         """
         return dict.copy(self._data['outputs'])
 
     def get_params(self):
         """
-        Returns params set on task.
+        Return params set on Task.
         """
         return dict.copy(self._data['params'])
 
     def run(self, binding={}):
         """
-        Run this task standalone for development/debugging purposes.
-
-        This method is the same as calling:
-        
-        >>> graphlab.deploy.job.create(this_task, bindings=binding, 
-        >>>                            environment=LocalEnvironment('dummy'))
-        
-        This is a convenient way to debug a Task in isolation.
+        Run this Task standalone for development/debugging purposes.
 
         Parameters
         ----------
         binding : dict
             Dictionary of bindings to provide wiring for inputs/outputs/params.
-            This allows binding input/output/params for this task.
-
-            For more documentation see the graphlab.deploy.job.create
-            documentation.
-
-            This parameter supports everything job.create's bindings parameter
+            This allows binding input/output/params for this Task. This
+            parameter supports everything job.create's bindings parameter
             supports, but also allows specifying a binding with a specific
             object.
 
-            For example, you have a Task t1 that produces an output named
-            'results' and a Task t2 that has an input named 'in' that is
-            defined from the 'results' output from t1. This could be defined as
-            follows:
-
-            >>> t2.set_input('in', from=(t1, 'output'))
-            
-            Now, if you want to run t2 in isolation using this run method,
-            pass in an instance of an SFrame to override the definition of
-            'in':
-           
-            >>> sf = gl.SFrame() # SFrame to use that represents 'output' from
-            >>> t2.run(bindings={'in': sf})
-            
-            Using this binding technique enables iteratively building a Task
-            with dependencies without requiring to create Jobs while
-            developing.
-
         Returns
         -------
         task : RuntimeTask
             Instance of the RuntimeTask, but after it has been run. Useful for
             verifying that code ran as expected. This instance has been cloned
             and used for execution.
+
+        See Also
+        --------
+        job.create
+
+        Notes
+        -----
+        - This method is the same as calling :func:`graphlab.deploy.job.create`.
+          It provides a convenient way to debug a Task in isolation.
+
+        Examples
+        --------
+        You have a Task t1 that produces an output named 'results' and a Task t2
+        that has an input named 'in' that is defined from the 'results' output
+        from t1. This could be defined as follows:
+
+        >>> t2.set_input('in', from=(t1, 'output'))
+
+        Now, if you want to run t2 in isolation using this run method,
+        pass in an instance of an :class:`~graphlab.SFrame` to override the
+        definition of 'in':
+
+        >>> sf = graphlab.SFrame() # SFrame to use that represents 'output' from
+        >>> t2.run(bindings={'in': sf})
+
+        Using this binding technique enables iteratively building a Task with
+        dependencies without requiring to create Jobs while developing.
         """
         metrics = {}
         clone = self._clone(self.name, session_aware=False)
@@ -822,7 +903,7 @@ class Task(object):
                 isinstance(obj, _gl.SArray))
 
     def __load_data_structure__(self, filepath):
-        """Returns data structure if can be loaded, otherwise returns None and logs warning"""
+        """Return data structure if can be loaded, otherwise returns None and logs warning"""
         # try to load different supported types, since don't know what type just try all and swallow exceptions
         obj = None
         try:
@@ -929,6 +1010,17 @@ class Task(object):
 
         return out
 
+    def __to_dict__(self):
+        task_json = {}
+        task_json["name"] = self.get_name()
+        task_json["description"] = self.get_description()
+        task_json["inputs"] = self.__inputs_str__()
+        task_json["code"] = self.__code_str__()
+        task_json["outputs"] = self.get_outputs()
+        task_json["params"] = self.get_params()
+        task_json["required_packages"] = list(self.get_required_packages())
+        return task_json
+
     def _set_dirty_bit(self):
         if self._modified_since_last_saved is not None and not self._modified_since_last_saved:
             self._modified_since_last_saved = True
@@ -936,10 +1028,8 @@ class Task(object):
 
 class Pipeline(object):
     """
-    Creates a Pipeline. A Pipeline references a sequence
-    of steps being executed, and intends to capture the
-    order of operations between steps to orchestrate a Data product's
-    execution.
+    A Pipeline references a sequence of steps being executed, and captures the
+    order of operations between steps to manage a data product's execution.
     """
 
     def __init__(self, name, *args, **kwargs):
@@ -953,12 +1043,12 @@ class Pipeline(object):
 
     def clone(self, name):
         """
-        Clones current object to new pipeline, with the specified name.
+        Clone current object to new pipeline, with the specified name.
 
         Parameters
         ----------
-        name : string
-            Name of the cloned pipline
+        name : str
+            Name of the cloned pipeline
 
         Returns
         -------
@@ -989,7 +1079,7 @@ class Pipeline(object):
 
     def get_required_packages(self):
         """
-        Retrieve the set of required packages for this job to execute.
+        Retrieve the set of required packages for this Pipeline to execute.
         """
         result = set()
         for i in self.steps:
@@ -998,9 +1088,11 @@ class Pipeline(object):
         return result
 
     def set_tasks(self, steps):
-        '''
-        A list of steps. Each step is a list of tasks that can be executed in parallel.
-        '''
+        """
+        Set A list of steps to run for current pipeline.
+
+        Each step is a list of Tasks that can be executed in parallel.
+        """
         for task in steps:
             # TODO: Verify that steps are Tasks/Pipelines, and can function
             pass
@@ -1069,7 +1161,7 @@ class _OutputTypeDict(_collections.MutableMapping, dict):
 
     def _verify_output_slot(self, key):
         if key not in self._task.get_outputs():
-            raise NameError("Invalid output name used in output assignment. Name '%s' has not been specified in this task." % key)
+            raise NameError("Invalid output name used in output assignment. Name '%s' has not been specified in this Task." % key)
 
     def _verify_output_type(self, value):
         if not Task._is_valid_data_structure(value):
@@ -1077,8 +1169,9 @@ class _OutputTypeDict(_collections.MutableMapping, dict):
 
 class RuntimeTask(object):
     """
-    Task representation at runtime passed into code for a Task. When a Task is run, the code that is executed 
-    receives a RuntimeTask instance for that task.
+    A runtime representation for a Task. When a Task is run, the code that is
+    executed receives a RuntimeTask instance for that Task, with all of its
+    parameters bound.
     """
     def __init__(self, task):
         if task is None:
@@ -1092,33 +1185,33 @@ class RuntimeTask(object):
 
     def get_outputs(self):
         """
-        Return the output names defined in the task being run.
+        Return the output names defined in the Task being run.
 
         Returns
         -------
-        List of output names defined for this task.
+        List of output names defined for this Task.
         """
         result = sorted(list(self.__task.get_outputs().keys()))
         return result
 
     def get_inputs(self):
         """
-        Returns the input names defined in the task being run.
+        Return the input names defined in the Task being run.
 
         Returns
         -------
-        List of input names defined for this task.
+        List of input names defined for this Task.
         """
         result = sorted(list(self.__task.get_inputs().keys()))
         return result
 
     def get_params(self):
         """
-        Returns the parameter names defined in the task being run.
+        Return the parameter names defined in the Task being run.
 
         Returns
         -------
-        List of parameter names defined for this task.
+        List of parameter names defined for this Task.
         """
         result = sorted(list(self.__task.get_params().keys()))
         return result
