@@ -14,7 +14,7 @@ import logging
 import traceback
 import json
 
-from _context import ENVIRONMENT
+from environment import _ENVIRONMENT as ENVIRONMENT
 import _timer as _timer
 
 from boto import connect_s3 as _connect_s3
@@ -55,7 +55,7 @@ class ExecutionEnvironment(object):
             Session to this run is bound to.
         tasks : List of tasks
             Cloned tasks to execute, in sequence
-        name : string
+        name : str
             Name of this execution, should be the name of the Job returned
         environment : Environment
             Configuration for this execution. The type must match the ExecutionEnvironment.
@@ -70,21 +70,22 @@ class ExecutionEnvironment(object):
 
 class LocalExecutionEnvironment(ExecutionEnvironment):
     """
-    Execute on local machine, synchronously.
+    An ExecutionEnvironment for local, synchronous execution.
     """
 
     def __init__(self):
         super(LocalExecutionEnvironment, self).__init__(ENVIRONMENT.Local)
 
-    def run_job(self, job, session):
+    def run_job(self, job, session, status_path=None):
         job._status = 'Running'
         try:
-
-            job = self._run(job, session)
+            job._start_time = str(_datetime.datetime.now().isoformat())
+            job = self._run(job, session, status_path=status_path)
             job._status = 'Completed'
-        
+            job._end_time = str(_datetime.datetime.now().isoformat())
         except Exception as e:
             job._status = 'Failed'
+            job._end_time = str(_datetime.datetime.now().isoformat())
             raise e
 
         if session is not None:
@@ -92,10 +93,10 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
 
         return job
 
-    def run_task(self, task, metrics, location=None):
+    def run_task(self, task, metrics, location=None, status_path=None):
         # special-case Job inside Job
         if isinstance(task, _job.Job):
-            run = self.run_job(task, None)
+            run = self.run_job(task, None, status_path=status_path)
             metrics.update(run.metrics)
             return
 
@@ -107,6 +108,10 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
 
         starttime = str(_datetime.datetime.now())
         __LOGGER__.info("Task started: %s" % task.get_name())
+        # update task status, for both local and async if possible
+        task.status = 'Running'
+        if status_path is not None:
+            self.__write_task_status(task, status_path)
 
         # path for execution
         if location is None:
@@ -145,6 +150,10 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
                 metrics['task.%s.error' % task.name] = True
                 metrics['task.%s.error_msg' % task.name] = e
                 metrics['task.%s.traceback' % task.name] = traceback.format_exc(1)
+                # update task status, for both local and async if possible, when failed
+                task.status = 'Failed'
+                if status_path is not None:
+                    self.__write_task_status(task, status_path)
                 raise RuntimeError("Unable to complete task successfully, Exception raised, trace: %s" % 
                         traceback.format_exc(0), e)
 
@@ -159,6 +168,10 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
             metrics['task.%s.last_run' % task.name] = starttime
             metrics['task.%s.runtime' % task.name] = t.secs
             metrics['task.%s.metrics' % task.name] = runtime_task.metrics
+            # update task status when task is completed
+            task.status = 'Completed'
+            if status_path is not None:
+                self.__write_task_status(task, status_path)
 
         finally:
             if path in _sys.path:
@@ -167,7 +180,14 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
 
         return runtime_task
 
-    def _run(self, job, session=None):
+    def __write_task_status(self, task, status_path):
+        if status_path is None:
+            return
+        with open(status_path, 'a+') as f:
+            f.writelines(str(task.name + ":" + task.status + "\n"))
+            f.close()
+
+    def _run(self, job, session=None, status_path=None):
         if not job.metrics:
             job.metrics = {}
         job.metrics.clear()
@@ -183,7 +203,9 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
         for cur_step in job._sequence:
             for task in cur_step:
                 task.env = self
-                self.run_task(task, job.metrics, location=exec_dir)
+                self.run_task(task, job.metrics, location=exec_dir, status_path=status_path)
+                if not isinstance(task, _job.Job): # update task status in job if task is not job
+                    job._task_status[task.name] = task.status
                 if session is not None:
                     session.save(job)
                 self._completed[task.name] = task
@@ -216,7 +238,10 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
 
 
 class Ec2ExecutionEnvironment(ExecutionEnvironment):
-
+    """
+    An ExecutionEnvironment for deploying jobs in EC2.
+    """
+    
     def __init__(self):
         super(Ec2ExecutionEnvironment, self).__init__(ENVIRONMENT.EC2)
 
@@ -330,7 +355,10 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
 
 
 class HadoopExecutionEnvironment(ExecutionEnvironment):
-
+    """
+    An ExecutionEnvironment for Hadoop jobs.
+    """
+    
     def __init__(self):
         super(HadoopExecutionEnvironment, self).__init__(ENVIRONMENT.Hadoop)
         self.hadoop_jar = _os.path.join(_os.path.dirname(_os.path.realpath(__file__)), "graphlabHadoopYarn-1.0.jar")
@@ -443,7 +471,10 @@ class HadoopExecutionEnvironment(ExecutionEnvironment):
 
 
 class LocalAsynchronousEnvironment(ExecutionEnvironment):
-
+    """
+    An ExecutionEnvironment for local jobs that should be run asynchronously.
+    """
+    
     def __init__(self):
         super(LocalAsynchronousEnvironment, self).__init__(ENVIRONMENT.LocalAsynchronous)
 
