@@ -1,4 +1,4 @@
-from graphlab.cython import cloudpickle as _cloudpickle
+import graphlab_util.cloudpickle as _cloudpickle
 import graphlab as _gl
 import _artifact
 
@@ -31,23 +31,29 @@ class Job(object):
     """
     Jobs represent executions of Tasks. Job objects should not be instantiated
     directly, and are intended to be created using
-    :py:func:`~graphlab.deploy.job.create` API.
+    :func:`graphlab.deploy.job.create` API.
+
+    Parameters
+    ----------
+    name : str
+        Name of this Job, must be unique.
+
+    tasks : list[string] | list[:py:class:`~graphlab.deploy.Task`]
+        Collection of Tasks to be executed.
+
+    environment : Environment, optional
+        Environment used for this execution. See
+        :py:class:`~graphlab.deploy.environment.LocalAsync` for an example
+        environment.
+    
     """
 
     def __init__(self, name, tasks=[], environment=None, _sequence=None):
-        """
+        """__init__(name, tasks=[], environment=None)
         Construct a Job, with a unique name. Job objects should not be
         instantiated directly, and are intended to be created using
-        :py:func:`~graphlab.deploy.job.create` API.
+        :func:`graphlab.deploy.job.create` API.
 
-        Parameters
-        ----------
-        name : string
-            Name of this Job
-        tasks : List or iterable of str | Task
-            Collection of tasks to be executed.
-        environment : Environment
-            Environment used for this execution. Must be a class that derives from Environment
         """
         if not isinstance(name, str):
             raise TypeError("Name is required to be a string.")
@@ -57,6 +63,7 @@ class Job(object):
         self.tasks = tasks
         self._artifacts = tasks
         self.metrics = {}
+        self._task_status = {}
         self._status = None
         self._modified_since_last_saved = None
         self._typename = Job.__name__
@@ -89,6 +96,46 @@ class Job(object):
         """
         return self._status
 
+    def get_metrics(self):
+        """
+        Returns metrics information about execution
+
+        Returns
+        -------
+        Dictionary representation of metrics.
+        """
+        return self.metrics
+
+    def _get_start_time(self):
+        """
+        Returns the start time of this execution
+
+        Returns
+        -------
+        String representation of the start time
+        """
+        return self._start_time
+
+    def _get_end_time(self):
+        """
+        Returns the end time of this execution
+
+        Returns
+        -------
+        String representation of the start time
+        """
+        return self._end_time
+   
+    def _get_task_status(self):
+        """
+        Returns the status of the tasks in the job
+
+        Returns
+        -------
+        A dictionary representation of the tasks status
+        """
+        return self._task_status
+    
     def cancel(self):
         """
         Cancels this execution.
@@ -138,17 +185,17 @@ class Job(object):
 
                 sequence[-1].append(cur_task)
 
-        # TODO: this is broken, deserialized job doens't have a name, uses filepath instead
+        # TODO: this is broken, deserialized job doesn't have a name, uses filepath instead
         return Job(file_path, _sequence=sequence)
 
     def _serialize(self, file_path):
         """
         Serializes the Job to the provided file_path.
 
-        Special-case sequential job definitions (jobs where all the tasks are to be sequentially executed) and 
+        Special-case sequential job definitions (jobs where all the tasks are to be sequentially executed) and
         there are no parallel executions.
 
-        In those cases, instead of serializing the tasks into the appropriate folder, just pickle the entire job 
+        In those cases, instead of serializing the tasks into the appropriate folder, just pickle the entire job
         and write it that instead.
 
         This way when executing if a Job is found run_job will be called, vs run_task.
@@ -157,7 +204,7 @@ class Job(object):
         base_path = _tempfile.mkdtemp()
 
         # TODO: serialize job attributes (like .name) or else won't be in deserialized object
-    
+
         # identify if only one task per step, and special-case that
         if all([len(step) == 1 for step in self._sequence]):
             # one task per step, so handle with serial execution
@@ -166,11 +213,11 @@ class Job(object):
             job_idx = 0
             step_dir = _os.path.join(base_path, str(step_idx))
             _os.mkdir(step_dir)
-            
+
             job_file_path = _os.path.join(step_dir, str(job_idx))
             with open(job_file_path, 'w') as job_file:
                 _cloudpickle.dump(self, job_file)
-            
+
             relative_path = _os.path.join('steps', str(step_idx), str(job_idx))
             aggregate_file.write(job_file_path, relative_path)
 
@@ -184,7 +231,7 @@ class Job(object):
                     task_file_path = _os.path.join(step_dir, str(task_idx))
                     with open(task_file_path, 'w') as task_file:
                         _cloudpickle.dump(cur_task, task_file)
-                    
+
                     relative_path = _os.path.join('steps', str(step_idx), str(task_idx))
                     aggregate_file.write(task_file_path, relative_path)
 
@@ -229,7 +276,7 @@ class Job(object):
         output.write(environment)
         output.write('\n')
         return output.getvalue()
-    
+
     def __getstate__(self):
         # session attribute gets added when loaded from disk
         # make sure to remove it when pickling
@@ -252,6 +299,7 @@ class LocalAsynchronousJob(Job):
         self.log_file_path = log_file_path
         self.exec_dir = _os.path.dirname(log_file_path)
         self._terminated = False
+        self._finalized = False
 
         # remove the metrics attribute so we add it only once the job is completed
         del self.metrics
@@ -275,13 +323,14 @@ class LocalAsynchronousJob(Job):
         if _psutil.pid_exists(self._pid):
             _psutil.Process(pid=self._pid).kill()
             self._terminated = True
+            self._status = 'Canceled'
+            self.__finalize_save()
 
     def get_status(self):
         if hasattr(self, '_status') and self._status is not None:
             return self._status
 
         if(self._terminated):
-            self._status = 'Canceled'
             return self._status
 
         # if pid still exists then return 'Running'
@@ -300,28 +349,88 @@ class LocalAsynchronousJob(Job):
                 return status
         else:
             return self.__get_status()
+    
+    def get_start_time(self):
+        if '__starttime__' not in self._task_status:
+            self.__get_task_status()
+        if '__starttime__' in self._task_status:
+            return self._task_status["__starttime__"]
+        return None
+
+    def get_end_time(self):
+        if '__endtime__' in self._task_status:
+            return self._task_status["__endtime__"]
+        return None
+    
+    def get_metrics(self):
+        # return metrics only when the job is finished and metrics attr is present
+        if hasattr(self, 'metrics'):
+            return getattr(self, 'metrics')
+        return {}
+
+    def __get_task_status(self):
+        task_list = {}
+        try:
+            with open(_os.path.join(self.exec_dir, 'status'), 'r') as f:
+                for line in f:
+                    line_split = line.split(":");
+                    if (len(line_split) == 2):
+                        task_list[line_split[0]] = line_split[1].rstrip('\n')
+                    else:
+                        if '__starttime__' in task_list:
+                            task_list["__endtime__"] = line.rstrip('\n')
+                        else:
+                            task_list["__starttime__"] = line.rstrip('\n')
+        except IOError:
+            pass
+
+        self._task_status = task_list
+        return task_list
+
+    def get_task_status(self):
+        # pull from task status file until the job finishes
+        if self._finalized is False:
+            self.__get_task_status()
+        return self._task_status
 
     def __get_metrics(self):
         """ Go to executed job and pull the metrics out
         """
-        with open(_os.path.join(self.exec_dir, 'metrics'), 'r') as f:
-            metrics = f.read()
+        try:
+            with open(_os.path.join(self.exec_dir, 'metrics'), 'r') as f:
+                metrics = f.read()
+        except IOError:
+            self.metrics = {}
+            return
 
         import ast
         self.metrics = ast.literal_eval(metrics)
 
     def __get_status(self):
         # get exit code from dir
-        with(open(_os.path.join(self.exec_dir, 'exitcode'), 'r')) as f:
-            code = f.read()
+        try:
+            with(open(_os.path.join(self.exec_dir, 'exitcode'), 'r')) as f:
+                code = f.read()
+        except IOError:
+            self._status = 'Unknown' # if we cannot query exitcode, something is wrong
+            self.__finalize_save()
+            return self._status
 
         if code == '0':
             self._status = 'Completed'
+            self.__finalize_save()
             return self._status
         else:
             self._status = 'Failed'
+            self.__finalize_save()
             return self._status
 
+    def __finalize_save(self):
+        # when the job finishes, query the metrics and task status one last time, and save
+        self._finalized = True
+        self.__get_metrics()
+        self.__get_task_status()
+        self.save()
 
 class Ec2Job(Job):
     def __init__(self, name, tasks, environment, ec2_dns_name, ec2_instance_id, s3_bucket,
@@ -341,7 +450,7 @@ class Ec2Job(Job):
         if self._final_status:
             return self._final_status
 
-        # Check s3 for the status key. 
+        # Check s3 for the status key.
         credentials = {}   # If this is empty, boto will look in environment variables.
         if(hasattr(self.environment, 'aws_access_key') and hasattr(self.environment, 'aws_secret_key')):
             credentials['aws_access_key_id'] = self.environment.aws_access_key
