@@ -8,9 +8,9 @@ import time as _time
 import collections
 import logging
 
-import graphlab.cython.cloudpickle as _cloudpickle
+import graphlab_util.cloudpickle as _cloudpickle
+import environment as _environment
 import _artifact as _artifact
-import _context as _ctx
 import _executionenvironment as _env
 import _job as _job
 
@@ -22,16 +22,19 @@ __LOGGER__ = logging.getLogger(__name__)
 
 class Session(object):
     """
-    Manages Tasks, Environments, and Jobs. A session object is an abstraction that provides functionality
-    manage the lifecycle of: Tasks, Environments, and Jobs. Session is not intended to be interacted with directly, 
-    instead use graphlab.deploy.tasks, graphlab.deploy.jobs, graphlab.deploy.environments.
+    Manages Tasks, Environments, and Jobs.
+
+    Sessions provide functionality for managing the lifecycle of: Tasks, 
+    Environments, and Jobs. Session is not intended to be interacted with
+    directly, instead use graphlab.deploy.tasks, graphlab.deploy.jobs,
+    graphlab.deploy.environments.
     """
 
     def __init__(self, location):
         """
-        Create a new session, specifying a directory to use for the session. This directory will be used for
-        saving and loading all Tasks, Pipelines, Contexts, and Jobs. If the directory does not exist it will be
-        created.
+        Creates a new session, specifying a directory to use for the session.
+        This directory will be used for saving and loading all Tasks, Pipelines,
+        Contexts, and Jobs. If the directory does not exist it will be created.
         """
 
         if location is None:
@@ -60,7 +63,7 @@ class Session(object):
         Parameters
         ----------
         artifact : Task | Environment | Job
-            Task or Environment instance to be managed by this sesssion. If another Task or Environment has 
+            Task or Environment instance to be managed by this sesssion. If another Task or Environment has
             the same name, then this new object will overwrite the previous object.
 
         Returns
@@ -72,7 +75,7 @@ class Session(object):
 
         if self._cache == {} or self._objects == {}:
             self.list(refresh=True, typename=None)
-        
+
         if artifact.name is None or artifact.name == "":
             __LOGGER__.warning("Name is required for objects managed by a session.")
             return False
@@ -85,13 +88,14 @@ class Session(object):
 
     def list(self, refresh=False, typename=None):
         """
-        List all items managed by this session. Items include Tasks, Environments, and Jobs.
+        Lists all items managed by this session. Items include Tasks,
+        Environments, and Jobs.
 
         Parameters
         ----------
         refresh : boolean
             Refresh the listing, meaning re-query location to get current artifacts.
-        typename : str (optional)
+        typename : str, optional
             Optionally limit the output a specific type (Task, Environment, Job)
 
         Returns
@@ -124,16 +128,16 @@ class Session(object):
             if typename == 'Job':
                 required_type = _job.Job
             elif typename == 'Environment':
-                required_type = _ctx.Environment
+                required_type = _environment.Environment
             elif typename == 'Task':
                 required_type = _artifact.Task
             else:
                 raise Exception("Unknown typename: %s" % typename)
-            
+
             # filter out by typename
-            # filtered_values = [ (n, t, status, in_memory) 
+            # filtered_values = [ (n, t, status, in_memory)
             # for (n, t, status, in_memory) in self._cache.values() if True] # issubclass(t, required_type)]
-            filtered_values = [ (n, t, status) 
+            filtered_values = [ (n, t, status)
                     for (n, t, status) in self._cache.values() if issubclass(t, required_type)]
         else:
             filtered_values = self._cache.values()
@@ -167,11 +171,11 @@ class Session(object):
 
         Parameters
         ----------
-        artifact : object | string
+        artifact : object | str
             Artifact to delete, passed as an object or by name as a string
-        pattern : string
+        pattern : str
             Spec to delete artifacts ex. "*-Local-*". This uses fnmatch.fnmatch
-        typename : str (optional)
+        typename : str, optional
             Optionally limit the output a specific type (Task, Environment, Job)
         """
         loaddir = self.location
@@ -183,11 +187,11 @@ class Session(object):
                     # skip file if not type specified
                     if typename is not None:
                         (root, ext) = _os.path.splitext(file)
-                        
+
                         if typename == 'Job':
                             required_type = _job.Job
                         elif typename == 'Environment':
-                            required_type = _ctx.Environment
+                            required_type = _environment.Environment
                         elif typename == 'Task':
                             required_type = _artifact.Task
                         else:
@@ -218,11 +222,11 @@ class Session(object):
                 raise TypeError("Invalid parameters to delete, see documentation.")
 
     def __remove_artifact(self, name, typename):
-        
+
         if typename == 'Job':
             required_type = _job.Job
         elif typename == 'Environment':
-            required_type = _ctx.Environment
+            required_type = _environment.Environment
         elif typename == 'Task':
             required_type = _artifact.Task
         else:
@@ -235,7 +239,7 @@ class Session(object):
         if _os.path.exists(path) and _os.path.isfile(path):
             found = True
             _os.remove(path)
-        
+
         # remove from both caches
         if name in self._cache:
             found = True
@@ -249,10 +253,10 @@ class Session(object):
 
     def _get_name_from_index(self, index, typename):
         output_sf = self.list(refresh=True, typename=typename)
-        
+
         if index >= output_sf.num_rows():
             return None
-        
+
         return output_sf['Name'][index]
 
     def _load(self, name, typename=None):
@@ -261,9 +265,9 @@ class Session(object):
 
         Parameters
         ----------
-        name : string
+        name : str
             Name of item to load, full name required (ex. s0)
-        typename : str (optional)
+        typename : str, optional
             Optionally limit the output a specific type (Task, Environment, Job)
 
         Returns
@@ -281,7 +285,7 @@ class Session(object):
             obj = self._objects[name]
             if isinstance(obj, _job.Job) and typename == 'Job':
                 return obj
-            elif isinstance(obj, _ctx.Environment) and typename == 'Environment':
+            elif isinstance(obj, _environment.Environment) and typename == 'Environment':
                 return obj
             elif isinstance(obj, _artifact.Task) and typename == 'Task':
                 return obj
@@ -296,7 +300,7 @@ class Session(object):
         ----------
         obj : object
             Object to save to this session
-        typename : str (optional)
+        typename : str, optional
             Specify the type of this object (Task, Environment, Job)
         """
         if isinstance(obj, str):
@@ -312,7 +316,7 @@ class Session(object):
         if typename is None:
             if isinstance(obj, _job.Job):
                 typename = 'Job'
-            elif isinstance(obj, _ctx.Environment):
+            elif isinstance(obj, _environment.Environment):
                 typename = 'Environment'
             elif isinstance(obj, _artifact.Task):
                 typename = 'Task'
@@ -341,7 +345,7 @@ class Session(object):
         ----------
         name : str
             Name for artifact, confirm if unique or not
-        typename : str (optional)
+        typename : str, optional
             Specify the type of this object (Task, Environment, Job)
 
         Returns
@@ -368,8 +372,8 @@ class Session(object):
         """
         Helper method that returns True when type is supported by Session.
         """
-        return any([isinstance(obj, _artifact.Task), isinstance(obj, _artifact.Pipeline), 
-            isinstance(obj, _ctx.Environment), isinstance(obj, _job.Job)])
+        return any([isinstance(obj, _artifact.Task), isinstance(obj, _artifact.Pipeline),
+            isinstance(obj, _environment.Environment), isinstance(obj, _job.Job)])
 
     def __repr__(self):
         return self.__str__(typename=None)
@@ -382,7 +386,7 @@ class Session(object):
             ret = "\nSession (location: %s)\n\n" % (_os.path.abspath(self.location))
         else:
             ret = "\n%s(s): \n" % typename
-        
+
         output_sf = self.list(refresh=True, typename=typename)
         if output_sf.num_rows() == 0:
             ret += '\nNothing to display.\n'
@@ -437,7 +441,7 @@ class Session(object):
 
 def open(location=None):
     """
-    Open and return an Artifact session.
+    Opens and returns an Artifact session.
 
     Parameters
     ----------
@@ -448,34 +452,25 @@ def open(location=None):
     if location is None:
         location = _os.path.expanduser(_os.path.join('~', '.graphlab', 'artifacts'))
 
+    if 'GRAPHLAB_ARTIFACTS_TMP_SESSION_DIR' in _os.environ:
+        location = _os.environ['GRAPHLAB_ARTIFACTS_TMP_SESSION_DIR']
+        __LOGGER__.info('Using tmp session dir: %s' % location)
+
     return Session(location)
 
-class ScopedSession(object):
-    """
-    Allow one class to implement listing, loading, and deleting for Tasks, Jobs, Environments.
-    
-    This class is a helper over the Session object, restricting it to just one type (Task, Environment, etc).
-    Used by graphlab.deploy.tasks, graphlab.deploy.environments to provide access to the session scoped by type.
 
-    To load an item (in this example a Task):
-    >>> gl.deploy.tasks # returns a listing of Task objects
-    >>> task = gl.deploy.tasks['taskname'] # load Task and return it
-    
-    To delete an item (in this example a Job):
-    >>> gl.deploy.jobs # returns a listing of Job objects
-    >>> del gl.deploy.jobs['delete-me-named-job']
-    
-    """
-    def __init__(self, session, type):
+class ScopedSession(object):
+    def __init__(self, session, type, docstring):
         self._session = session
         self._typename = type.__name__
+        self.__doc__ = docstring
 
     def __repr__(self):
         return self._session.__str__(typename=self._typename)
 
     def list(self, refresh=True):
         """
-        List all items of this type (jobs, tasks, environments).
+        Lists all items of this type (jobs, tasks, environments).
 
         Parameters
         ----------
@@ -490,13 +485,13 @@ class ScopedSession(object):
 
     def load(self, item):
         """
-        Load an item by name.
+        Loads an item by name.
 
         Parameters
         ----------
-        name : string
+        name : str
             Name of item to load, full name required (ex. s0)
-        
+
         Returns
         -------
         Instance of the item.
@@ -505,15 +500,19 @@ class ScopedSession(object):
 
     def delete(self, item, pattern=None):
         """
-        Delete an item from disk. This will not ask permission and will delete
-        from the file system, so please be careful using this.
+        Deletes an item from disk.        
 
         Parameters
         ----------
-        item : object | string
+        item : object | str
             Item to delete, passed as an object or by name as a string (Job, Task, Environment)
-        pattern : string
+        pattern : str
             Spec to delete artifacts ex. "*-Local-*". This uses fnmatch.fnmatch
+
+        Notes
+        -----
+        - This will not ask permission and will delete from the file system, so
+        please be careful using this.
         """
         return self._session._delete(item, pattern=pattern, typename=self._typename)
 
