@@ -10,7 +10,9 @@ from graphlab.toolkits.supervised_learning import SupervisedLearningModel
 
 DEFAULT_SOLVER_OPTIONS = {
 'convergence_threshold': 1e-2,
-'max_iterations': 10}
+'max_iterations': 10,
+'lbfgs_memory_level': 10
+}
 
 DEFAULT_HYPER_PARAMETER_RANGE = {
     'penalty' : [0.001, 0.01, 0.1, 1.0, 10.0, 100.0]
@@ -66,16 +68,30 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
         smaller the penalty, the lower is the emphasis placed on misclassified
         examples, and the classifier would spend more time maximizing the
         margin for correctly classified examples. The default value is 1.0;
-        this parameter must be set to a value of atleast 1e-10.
+        this parameter must be set to a value of at least 1e-10.
 
 
     solver : string, optional
         Name of the solver to be used to solve the problem. See the
         references for more detail on each solver. Available solvers are:
 
-        - *auto (default)*: automatically chooses the best solver for the data
-          and model parameters.
-        - *sdca*: Stochastic-Dual Co-ordinate descent method. (See reference)
+        - *auto (default)*: automatically chooses the best solver (from the ones
+         listed below) for the data and model parameters.
+        - *lbfgs*: lLimited memory BFGS (``lbfgs``) is a robust solver for wide 
+        datasets(i.e datasets with many coefficients).  
+
+        The solvers are all automatically tuned and the default options should 
+        function well. See the solver options guide for setting additional 
+        parameters for each of the solvers.
+
+    feature_rescaling: bool, default = true
+
+        Feature rescaling is an important pre-processing step that ensures
+        that all features are on the same scale. An L2-norm rescaling is
+        performed to make sure that all features are of the same norm. Categorical
+        features are also rescaled by rescaling the dummy variables that 
+        are used to represent them. The coefficients are returned in original
+        scale of the problem.
 
     solver_options : dict, optional
         Solver options. The options and their default values are as follows:
@@ -87,6 +103,33 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
         +-----------------------+---------+-----------------------------------------+
         | max_iterations        |     10  | Max number of solver iterations         |
         +-----------------------+---------+-----------------------------------------+
+        | lbfgs_memory_level    |     10  | Memory used by lbfgs (lbfgs only)       |
+        +-----------------------+---------+-----------------------------------------+
+
+        convergence_threshold: 
+
+        Convergence is tested using variation in the training objective. The 
+        variation in the training objective is calculated using the difference 
+        between the objective values between two steps. Consider reducing this 
+        below the default value (0.01) for a more accurately trained model. 
+        Beware of overfitting (i.e a model that works well only on the training 
+        data) if this parameter is set to a very low value.
+
+        max_iterations: 
+        
+        The maximum number of allowed passes through the data. More passes over 
+        the data can result in a more accurately trained model. Consider 
+        increasing this (the default value is 10) if the training accuracy is 
+        low and the *Grad-Norm* in the display is large.
+
+        lbfgs_memory_level:
+        
+        The L-BFGS algorithm keeps track of gradient information from the 
+        previous ``lbfgs_memory_level`` iterations. The storage requirement for 
+        each of these gradients is the ``num_coefficients`` in the problem. 
+        Increasing the ``lbfgs_memory_level ``can help improve the quality of 
+        the model trained. Setting this to more than ``max_iterations`` has the 
+        same effect as setting it to ``max_iterations``.
 
     verbose : bool, optional
         If True, print progress updates.
@@ -96,6 +139,10 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
     out : SVMModel
         A trained model of type
         :class:`~graphlab.svm.SVMModel`.
+    
+    See Also
+    --------
+    SVMModel
 
     Notes
     -----
@@ -116,82 +163,35 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
     - A constant term of '1' is automatically added for the model intercept to
       model the bias term.
 
-    Examples
-    --------
-    *Training*
+    - Note that the hinge loss is approximated by the scaled logistic loss
+      function. (See user guide for details)
 
-    Given an :class:`~graphlab.SFrame` ``sf``, a list of feature columns
-    [``feature_1`` ... ``feature_K``], and a target column ``target`` with 0 and
-    1 values, create a
-    :class:`~graphlab.svm.SVMModel` as follows:
-
-    >>> m = svm.create(sf, target='target')
-
-    By default all columns of ``sf`` except the target are used as features, but
-    specific feature columns can be specified manually.
-
-    >>> m = svm.create(sf, 'target', ['feature_1', 'feature_2'])
-
-    The L2 regularization weight is set to 0.01. For stronger regularization
-    raise this weight.
-
-    >>> m = svm.create(sf, 'target', penalty=0.5)
-
-    *Model Querying*
-
-    Model attributes can be retrieved with either bracket syntax or the
-    :func:`~graphlab.svm.SVMModel.get` method. The set of queryable fields is
-    described in the ``get`` documentation, and can be obtained
-    programmatically with the :func:`~graphlab.svm.SVMModel.list_fields`
-    method. The model coefficients are stored in an SFrame.
-
-    >>> m.list_fields()
-    >>> coef = m['coefficients']  # an SFrame
-    >>> coef = m.get('coefficients')  # equivalent to previous line
-
-    The :func:`~graphlab.svm.SVMModel.summary` and
-    :func:`~graphlab.svm.SVMModel.training_stats` methods provide shortcuts to
-    many of the model attributes.
-
-    >>> print m.summary()
-    >>> m.training_stats()
-
-    *Prediction and Evaluation*
-
-    With the trained model we can make predictions for new data in SFrame
-    ``sf_new`` and compare these predictions to actual target values, if these
-    values are known. For both of these methods, the new SFrame must include
-    columns with the same names as the training features; the ``evaluate``
-    method requires ``sf_new`` to have the same ``target`` column name as well.
-
-    Predictions can be generated as class labels (0 or 1), or margins (i.e. the
-    distance of the observations from the hyperplane separating the classes). By
-    default, the predict method returns classes.
-
-    >>> pred_margin = m.predict(sf_new, output_type='margin')
-    >>> pred_class = m.predict(sf_new, output_type='class')
-
-    *Save and Load*
-
-    The model can be saved to disk and loaded at a later time.
-
-    >>> m.save("mymodel")
-    >>> m2 = graphlab.load_model("mymodel")
-
-    For more detail, see the documentation for individual methods of the
-    :class:`~graphlab.svm.SVMModel`.
 
     References
     ----------
     - `Wikipedia - Support Vector Machines
       <http://en.wikipedia.org/wiki/svm>`_
 
-    - Hsieh, Cho-Jui, et al. 'A dual coordinate descent method for large-scale
-      linear SVM.' Proceedings of the 25th international conference on Machine
-      learning. ACM, 2008.
+    - Zhang et al. - Modified Logistic Regression: An Approximation to
+      SVM and its Applications in Large-Scale Text Categorization (ICML 2003)
+
+
+    Examples
+    --------
+
+    Given an :class:`~graphlab.SFrame` ``sf``, a list of feature columns
+    [``feature_1`` ... ``feature_K``], and a target column ``target`` with 0 and
+    1 values, create a
+    :class:`~graphlab.svm.SVMModel` as follows:
+
+    >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+    >>> data['is_expensive'] = data['price'] > 30000
+    >>> model = graphlab.svm.create(data, 'is_expensive')
     """
 
-    print "Preparing the data..."
+    if verbose:
+      print "Preparing the data..."
+
     model_name = "supervised_learning_svm"
     _mt._get_metric_tracker().track(
         'toolkit.classification.svm.create')
@@ -259,7 +259,6 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
 
     model_proxy = ret['model']
     model = SVMModel(model_proxy)
-    model.summary()
     return model
 
 
@@ -276,37 +275,6 @@ class SVMModel(SupervisedLearningModel):
     are represented as points in space, mapped so that the examples from the
     two classes being classified are divided by linear separator. 
     
-
-    **Examples**
-
-
-    .. sourcecode:: python
-
-        # Load the data (From an S3 bucket)
-        >>> import graphlab as gl
-        >>> data =  gl.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
-
-        # Make sure the target is binary 0/1
-        >>> data['expensive'] = data['price'] > 30000
-        
-        # Make a logistic regression model
-        >>> model = gl.svm.create(data, target='expensive', features=['bath', 'bedroom', 'size'])
-
-        # Here, all columns (other than the target) are used as features.
-        >>> model = gl.svm.create(data, target='expensive')
-
-        # Extract the coefficients
-        >>> coefficients = model['coefficients']     # an SFrame
-        
-        # Make predictions (as margins, probability, or class)
-        >>> predictions = model.predict(data)    # Predicts 0/1
-        >>> predictions = model.predict(data, output_type='margin')       
-
-        # Evaluate the model 
-        >>> results = m.evaluate(data)               # a dictionary
-    
-    **Matematical Details**
-    
     Given a set of features :math:`x_i`, and a label :math:`y_i \in \{0,1\}`,
     SVM minimizes the loss function:
 
@@ -322,48 +290,38 @@ class SVMModel(SupervisedLearningModel):
            \min_{\\theta} \sum_{i = 1}^{n} f_i(\\theta) + \lambda ||\\theta||^{2}_{2} 
 
     where :math:`\lambda` is the ``penalty`` parameter.
-
-    **Model creation options guide**
-
-
-    solver: string, default = 'auto'
-
-        The optimization used to train the model is a carefully engineered
-        collection of methods that are automatically picked based on the input
-        data. The ``scda`` performs a dual-coordinate descent and solves the
-        SVM in the dual space. It works best when the model has many
-        coefficients.
     
 
-    feature_rescaling: string, default = true
+    Examples
+    --------
 
-        Feature rescaling is an important pre-processing step that ensures
-        that all features are on the same scale. An L2-norm rescaling is
-        performed to make sure that all features are of the same norm. Categorical
-        features are also rescaled by rescaling the dummy variables that 
-        are used to represent them. The coefficients are returned in original
-        scale of the problem.
+    .. sourcecode:: python
 
+        # Load the data (From an S3 bucket)
+        >>> import graphlab as gl
+        >>> data =  gl.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
 
-    **Solver Options Guide**
+        # Make sure the target is binary 0/1
+        >>> data['is_expensive'] = data['price'] > 30000
+        
+        # Make a logistic regression model
+        >>> model = gl.svm.create(data, target='is_expensive'
+                                        , features=['bath', 'bedroom', 'size'])
 
-    Most solvers are all automatically tuned and the default options should
-    function well. Changing the default settings may help in some cases.
+        # Extract the coefficients
+        >>> coefficients = model['coefficients']     # an SFrame
+        
+        # Make predictions (as margins, or class)
+        >>> predictions = model.predict(data)    # Predicts 0/1
+        >>> predictions = model.predict(data, output_type='margin')       
+
+        # Evaluate the model 
+        >>> results = model.evaluate(data)               # a dictionary
+
+    See Also
+    --------
+    create
     
-    max_iterations : integer, default = 10.
-
-        The maximum number of passes through the data allowed. Increasing his 
-        can result in more accurate training time. If the training accuracy 
-        is low and the ``Grad-Norm`` in the display is large, then the model 
-        has not trained completely.
-
-    convergence_threshold : double, default = 1e-2.
-
-        Convergence is tested using variation in the training objective.  The
-        variation in the training objective is calculated by dividing the
-        difference between the objective values between two step. 
-    
-
     """
     def __init__(self, model_proxy):
         '''__init__(self)'''
@@ -431,10 +389,6 @@ class SVMModel(SupervisedLearningModel):
         detailed below, and can be obtained programmatically with the
         :func:`~graphlab.svm.SVMModel.list_fields` method.
 
-        Each of these fields can be queried in one of two ways:
-
-        >>> out = m['field']
-        >>> out = m.get('field')  # equivalent to previous line
 
         +-----------------------+----------------------------------------------+
         |      Field            | Description                                  |
@@ -443,9 +397,11 @@ class SVMModel(SupervisedLearningModel):
         +-----------------------+----------------------------------------------+
         | convergence_threshold | Desired solver accuracy                      |
         +-----------------------+----------------------------------------------+
+        | feature_rescaling     | Bool indicating l2-rescaling of features     |
+        +-----------------------+---------+------------------------------------+
         | features              | Feature column names                         |
         +-----------------------+----------------------------------------------+
-        | penalty               | Misclassification penalty term               |
+        | lbfgs_memory_level    | Number of updates to store (lbfgs only)      |
         +-----------------------+----------------------------------------------+
         | max_iterations        | Maximum number of solver iterations          |
         +-----------------------+----------------------------------------------+
@@ -455,6 +411,8 @@ class SVMModel(SupervisedLearningModel):
         +-----------------------+----------------------------------------------+
         | num_features          | Number of dataset columns used for training  |
         +-----------------------+----------------------------------------------+
+        | penalty               | Misclassification penalty term               |
+        +-----------------------+----------------------------------------------+
         | solver                | Type of solver                               |
         +-----------------------+----------------------------------------------+
         | solver_status         | Solver status after training                 |
@@ -467,6 +425,8 @@ class SVMModel(SupervisedLearningModel):
         +-----------------------+----------------------------------------------+
         | train_time            | Training time (excludes preprocessing)       |
         +-----------------------+----------------------------------------------+
+        | trained               | Indicates whether model is trained           |
+        +-----------------------+----------------------------------------------+
 
         Parameters
         ----------
@@ -477,6 +437,24 @@ class SVMModel(SupervisedLearningModel):
         -------
         out
             Value of the requested fields.
+        
+        See Also
+        --------
+        list_fields
+
+        Examples
+        --------
+
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.svm.create(data, 
+                                  target='is_expensive',
+                                  features=['bath', 'bedroom', 'size'])
+        >>> print model['num_features']
+        3
+        >>> print model.get('num_features')       # equivalent to previous line
+        3
         """
 
         _mt._get_metric_tracker().track(
@@ -496,12 +474,17 @@ class SVMModel(SupervisedLearningModel):
 
     def summary(self):
         """
-        Display a summary of the model, including coefficients (if applicable),
-        training options, and training statistics. Use the
-        :func:`~graphlab.svm.SVMModel.get` method to retrieve these values
-        programmatically, or to see more detail about the queryable fields.
+        Display a summary of the SVMModel.
 
-        >>> m.summary()
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.svm.create(data,
+                                  target='is_expensive',
+                                  features=['bath', 'bedroom', 'size'])
+        >>> model.summary()
         """
 
         _mt._get_metric_tracker().track(
@@ -541,19 +524,19 @@ class SVMModel(SupervisedLearningModel):
         out : dict
             Dictionary with model default options.
 
-        >>> options =  m.get_default_options()
+        See Also
+        --------
+        get_current_options, list_fields, get
 
-        +-----------------------+----------------------------------------------+
-        |      Field            | Description                                  |
-        +=======================+==============================================+
-        | convergence_threshold |  0.01                                        |
-        +-----------------------+----------------------------------------------+
-        | penalty               |  0.01                                        |
-        +-----------------------+----------------------------------------------+
-        | max_iterations        |  10                                          |
-        +-----------------------+----------------------------------------------+
-        | solver                |  'auto'                                      |
-        +-----------------------+----------------------------------------------+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.svm.create(data,
+                                  target='is_expensive',
+                                  features=['bath', 'bedroom', 'size'])
+        >>> default_options = model.get_default_options()
 
         """
         _mt._get_metric_tracker().track(
@@ -569,7 +552,19 @@ class SVMModel(SupervisedLearningModel):
         out : dict
             Dictionary with options used to define and train the model.
 
-        >>> options =  m.get_current_options()
+        See Also
+        --------
+        get_default_options, list_fields, get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.svm.create(data,
+                                  target='is_expensive',
+                                  features=['bath', 'bedroom', 'size'])
+        >>> current_options = model.get_current_options()
         """
         _mt._get_metric_tracker().track('toolkit.classification.svm.get_current_options')
         return super(SVMModel, self).get_current_options()
@@ -613,8 +608,22 @@ class SVMModel(SupervisedLearningModel):
         out : SArray
             An SArray with model predictions.
 
-        >>> predictions = m.predict(test_data, output_type='class') # SArray
-        >>> predictions = m.predict(test_data, output_type='margin') # SArray
+        See Also
+        ----------
+        create, evaluate
+
+        Examples
+        ----------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.svm.create(data,
+                                  target='is_expensive',
+                                  features=['bath', 'bedroom', 'size'])
+
+        >>> class_predictions = model.predict(data)
+        >>> margin_predictions = model.predict(data, output_type='margin')
+
         """
 
         if output_type not in ['class', 'margin']:
@@ -668,15 +677,21 @@ class SVMModel(SupervisedLearningModel):
             Dictionary of evaluation results. The dictionary keys are *accuracy*
             and *confusion_table*.
 
-        References
+        See Also
         ----------
-        - `Wikipedia - confusion matrix
-          <http://en.wikipedia.org/wiki/Confusion_matrix>`_
+        create, predict
 
+        Examples
+        ----------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
 
-        >>> results = m.evaluate(test_data)
-        >>> accuracy = results['accuracy']               # Classification accuracy
-        >>> confusion_table = results['confusion_table'] # A dictionary
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.svm.create(data,
+                                  target='is_expensive',
+                                  features=['bath', 'bedroom', 'size'])
+
+        >>> results = model.evaluate(data)
+        >>> print results['accuracy']
         """
 
         _mt._get_metric_tracker().track(
@@ -702,16 +717,6 @@ class SVMModel(SupervisedLearningModel):
         return ret
 
 
-    def training_stats(self):
-        """
-        Return a dictionary containing statistics collected during model
-        training. These statistics are also available with the ``get`` method,
-        and are described in more detail in the documentation for that method.
-        """
-        _mt._get_metric_tracker().track(
-            'toolkit.classification.svm.training_stats')
-        return super(SVMModel, self).training_stats()
-
     def list_fields(self):
         """
         List the fields stored in the model, including data, model, and training
@@ -722,6 +727,21 @@ class SVMModel(SupervisedLearningModel):
         -------
         out : list
             List of fields queryable with the ``get`` method.
+        
+        See Also
+        --------
+        get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.svm.create(data,
+                                  target='is_expensive',
+                                  features=['bath', 'bedroom', 'size'])
+
+        >>> model.list_fields()
         """
 
         _mt._get_metric_tracker().track(
