@@ -1,7 +1,6 @@
 from graphlab.data_structures.sgraph import SGraph as _SGraph
-import graphlab.aggregate as _Aggregate
-import graphlab.toolkits.main as main
-from graphlab.toolkits.model import Model as _Model
+import graphlab.toolkits.main as _main
+from graphlab.toolkits.graph_analytics.model_base import GraphAnalyticsModel as _ModelBase
 import copy
 
 _HAS_IPYTHON = True
@@ -11,93 +10,84 @@ except:
     _HAS_IPYTHON = False
 
 
-class ShortestPathModel(_Model):
+def get_default_options():
     """
-    A model object containing the distance for each vertex in the graph to the
-    source vertex.
+    Get the default options for :func:`graphlab.shortest_path.create`.
 
-    An instance of this model can be created using :func:`graphlab.shortest_path.create`.
-    Do NOT construct the model directly.
+    Returns
+    -------
+    out : dict
+
+    Examples
+    --------
+    >>> graphlab.shortest_path.get_default_options()
     """
-    def __init__(self, input_params, model):
+    return _main.run('sssp_default_options', {})
+
+
+class ShortestPathModel(_ModelBase):
+    """
+    Model object containing the distance for each vertex in the graph to a
+    single source vertex, which is specified during
+    :func:`graphlab.shortest_path.create`.
+
+    The model also allows querying for one of the shortest paths from the source
+    vertex to any other vertex in the graph.
+
+    Below is a list of queryable fields for this model:
+
+    +----------------+------------------------------------------------------------+
+    | Field          | Description                                                |
+    +================+============================================================+
+    | graph          | A new SGraph with the triangle count as a vertex property. |
+    +----------------+------------------------------------------------------------+
+    | num_triangles  | Total number of triangles in the graph.                    |
+    +----------------+------------------------------------------------------------+
+    | triangle_count | An SFrame with the triangle count for each vertex.         |
+    +----------------+------------------------------------------------------------+
+    | runtime        | Total runtime of the toolkit                               |
+    +----------------+------------------------------------------------------------+
+
+    An instance of this model can be created using
+    :func:`graphlab.shortest_path.create`.  Do NOT construct the model
+    directly.
+
+    See Also
+    --------
+    create
+    """
+    def __init__(self, model):
         '''__init__(self)'''
         self.__proxy__ = model
-        self._input = input_params
-        self._fields = ['runtime', 'distance', 'graph']
         self._path_query_table = None
 
-    def list_fields(self):
-        """
-        List of fields stored in the model. Each of these fields can be queried
-        using the ``get`` function.
-
-        Returns
-        -------
-        out : list
-            A list of fields that can be queried using the ``get`` method.
-        """
-
-        return self._fields
-
-    def get(self, field):
-        """
-        Get the value of a given field. The list of all queryable fields is
-        detailed below, and can be obtained programmatically using the
-        :func:`~graphlab.graph_analytics.ShortestPathModel.list_fields`
-        method.
-
-        Each of these fields can be queried in one of two ways:
-
-        >>> out = m['field']
-        >>> out = m.get('field')  # equivalent to previous line
-
-        +-----------+----------------------------------------------------------+
-        |   Field   | Description                                              |
-        +===========+==========================================================+
-        | distance  | SFrame with each vertex's distance to the queried vertex |
-        +-----------+----------------------------------------------------------+
-        | graph     | Input graph with the distance as a vertex property       |
-        +-----------+----------------------------------------------------------+
-
-        Parameters
-        ----------
-        field : string
-            Name of the field to be retrieved.
-
-        Returns
-        -------
-        out : [various]
-            The current value of the requested field.
-
-       """
-
-        if (field == 'distance'):
-            return _SGraph(_proxy=self.__proxy__.get('__graph__')).select_fields(['distance']).get_vertices()
-        elif (field == 'graph'):
-            return _SGraph(_proxy=self.__proxy__.get('__graph__'))
-        else:
-            return self.__proxy__.get(field)
-
-    def summary(self):
-        """Return a summary of the model including hyperparameters, and runtime information."""
-
-        ret = {'hyperparams': self._input}
-        for key in ['runtime', 'distance']:
-            ret[key] = self.get(key)
+    def _result_fields(self):
+        ret = super(ShortestPathModel, self)._result_fields()
+        ret['distance'] = "SFrame with each vertex's distance. See m['distance']"
         return ret
 
+    def _setting_fields(self):
+        ret = super(ShortestPathModel, self)._setting_fields()
+        for k in ['source_vid', 'weight_field', 'max_distance']:
+            ret[k] = self[k]
+        return ret
+
+    def _method_fields(self):
+        return {'get_path': 'Get shortest path, e.g. m.get_path(vid=target_vid)'}
+
     def get_path(self, vid, show=False, highlight=None, **kwargs):
         """
-        Return one of the shortest paths between two vertices in a graph. The
-        source vertex is specified by the original call to shortest path, and
-        the destination vertex is specified in this function. Optionally, plots
-        the path with networkx.
+        Get the shortest path.
+        Return one of the shortest paths between the source vertex defined
+        in the model and the query vertex.
+        The source vertex is specified by the original call to shortest path.
+        Optionally, plots the path with networkx.
 
         Parameters
         ----------
         vid : string
-            ID of the destination vertex. The source vertex ID is specified when the
-            shortest path result is first computed.
+            ID of the destination vertex. The source vertex ID is specified
+            when the shortest path result is first computed.
 
         show : boolean
             Indicates whether the path should be plotted. Default is False.
@@ -106,19 +96,31 @@ class ShortestPathModel(_Model):
             If the path is plotted, identifies the vertices (by vertex ID) that
             should be highlighted by plotting in a different color.
 
+        kwargs :
+            Additional parameters passed into the :func:`graphlab.SGraph.show`
+            when `show` is True.
+
         Returns
         -------
         path : list
-            Containing pairs of (vertex_id, distance) on the path between the source and destination vertices.
+            List of pairs of (vertex_id, distance) in the path.
+
+        Examples
+        --------
+        >>> m.get_path(vid=0, show=True)
+
+        See Also
+        --------
+        SGraph.show
         """
         if self._path_query_table is None:
             self._path_query_table = self._generate_path_sframe()
 
-        source_vid = self._input['source_vid']
+        source_vid = self['source_vid']
         path = []
         path_query_table = self._path_query_table
         if not vid in path_query_table['vid']:
-            raise ValueError('Vertex id ' + str(vid) + ' not found')
+            raise ValueError('Destination vertex id ' + str(vid) + ' not found')
 
         record = path_query_table[path_query_table['vid'] == vid][0]
         dist = record['distance']
@@ -151,8 +153,8 @@ class ShortestPathModel(_Model):
             if highlight is not None:
 
                 if not isinstance(highlight, list):
-                    raise TypeError, "Input 'highlight' must be a list."
-                    
+                    raise TypeError("Input 'highlight' must be a list.")
+
                 path_names = set([x[0] for x in path])
                 path_highlight = list(set.intersection(path_names, set(highlight)))
 
@@ -162,16 +164,13 @@ class ShortestPathModel(_Model):
 
         return path
 
-    def _get_wrapper(self):
-        return lambda m: ShortestPathModel(self._input, m)
-
     def _generate_path_sframe(self):
         """
         Generates an sframe with columns: vid, parent_row_id, and distance.
         Used for speed up the path query.
         """
-        source_vid = self._input['source_vid']
-        weight_field = self._input['edge_attr']
+        source_vid = self['source_vid']
+        weight_field = self['weight_field']
 
         query_table = copy.copy(self.get('distance'))
         query_table.add_row_number('row_id')
@@ -179,7 +178,7 @@ class ShortestPathModel(_Model):
         g = self.get('graph').add_vertices(query_table)
         # The sequence id which a vertex is visited, initialized with 0 meaning not visited.
         g.vertices['__parent__'] = -1
-        weight_field = self._input['edge_attr']
+        weight_field = self['weight_field']
         if (weight_field == ""):
             weight_field = '__unit_weight__'
             g.edges[weight_field] = 1
@@ -198,13 +197,13 @@ class ShortestPathModel(_Model):
         return query_table
 
 
-def create(graph, source_vid, weight_field="", verbose=True, plot=False):
+def create(graph, source_vid, weight_field="", max_distance=1e30, verbose=True, plot=False):
     """
     Compute the single source shortest path distance from the source vertex to
     all vertices in the graph. Note that because SGraph is directed, shortest
     paths are also directed. To find undirected shortes paths add edges to the
-    SGraph in both directions. Return a model object with distance each of vertex in the
-    graph.
+    SGraph in both directions. Return a model object with distance each of
+    vertex in the graph.
 
     Parameters
     ----------
@@ -226,27 +225,39 @@ def create(graph, source_vid, weight_field="", verbose=True, plot=False):
 
     Returns
     -------
-    out : Model
-        A Model object that contains the shortest path distance for each vertex
-        to the source vertex.
+    out : ShortestPathModel
+
+    References
+    ----------
+    - `Wikipedia - ShortestPath <http://en.wikipedia.org/wiki/Shortest_path_problem>`_
 
     Examples
     --------
     If given an :class:`~graphlab.SGraph` ``g``, we can create
     a :class:`~graphlab.shortest_path.ShortestPathModel` as follows:
 
-    >>> sp = shortest_path.create(g, source_vid=123)
+    >>> g = graphlab.load_graph('http://snap.stanford.edu/data/email-Enron.txt.gz', format='snap')
+    >>> sp = graphlab.shortest_path.create(g, source_vid=1)
 
-    We can obtain the shortest path from the source vertex to each vertex in
-    the graph ``g`` as follows:
+    We can obtain the shortest path distance from the source vertex to each
+    vertex in the graph ``g`` as follows:
 
     >>> sp_sframe = sp['distance']   # SFrame
 
+    To get the actual path from the source vertex to any destination vertex:
+
+    >>> path = sp.get_path(vid=10)
+
+
     We can obtain an auxiliary graph with additional information corresponding
     to the shortest path from the source vertex to each vertex in the graph
     ``g`` as follows:
 
     >>> sp_graph = sp.get['graph']       # SGraph
+
+    See Also
+    --------
+    ShortestPathModel
     """
     if not isinstance(graph, _SGraph):
         raise TypeError('graph input must be a SGraph object.')
@@ -254,8 +265,7 @@ def create(graph, source_vid, weight_field="", verbose=True, plot=False):
     if plot is True:
         print "The plot functionality for shortest path is not yet implemented."
 
-    opts = {'source_vid': source_vid, 'edge_attr': weight_field,
-            'graph': graph.__proxy__}
-    params = main.run('sssp', opts, verbose, plot)
-    return ShortestPathModel({'source_vid': source_vid,
-                             'edge_attr': weight_field}, params['model'])
+    opts = {'source_vid': source_vid, 'weight_field': weight_field,
+            'max_distance': max_distance, 'graph': graph.__proxy__}
+    params = _main.run('sssp', opts, verbose, plot)
+    return ShortestPathModel(params['model'])
