@@ -29,9 +29,6 @@ def run(toolkit_name, options, verbose=True, show_progress=False):
         A map containing the required input for the toolkit function,
         for example: {'graph': g, 'reset_prob': 0.15}.
 
-    data : pandas.DataFrame
-        The target dataframe.
-
     verbose : bool
         If true, enable progress log from server.
 
