@@ -17,6 +17,11 @@ def load_model(location):
     location : string
         Location of the model to load. Can be a local path or a remote URL.
         Because models are saved as directories, there is no file extension.
+
+    Examples
+    ----------
+    >>> model.save('my_model_file')
+    >>> loaded_model = gl.load_model('my_model_file')
     """
     _mt._get_metric_tracker().track('toolkit.model.load_model')
 
@@ -32,8 +37,7 @@ class Model(object):
     the list of the queriable fields can be obtained via `list_fields`.
 
     Model object also supports dictionary type access, e.g. m['pagerank']
-    is equivalent to m.get('pagerank'), and m.keys() is equivalent to
-    m.list_fields().
+    is equivalent to m.get('pagerank').
     """
 
     """The UnityModel Proxy Object"""
@@ -45,21 +49,30 @@ class Model(object):
 
         Returns
         -------
-        out : string
+        out : str
             The name of the model object.
+
+        Examples
+        --------
+        >>> model_name = m.name()
         """
         return self.__class__.__name__
 
     def list_fields(self):
         """
-        Return the a list of queriable key of the model
+        List of fields stored in the model. Each of these fields can be queried
+        using the ``get(field)`` function or ``m[field]``.
 
         Returns
         -------
         out : list[str]
-            available fields in the model
+            A list of fields that can be queried using the ``get`` method.
+
+        Examples
+        --------
+        >>> fields = m.list_fields()
         """
-        _mt._get_metric_tracker(self.___class__.__module__ + '.list_fields')
+        _mt._get_metric_tracker().track(self.__class__.__module__ + '.list_fields')
         return self.__proxy__.list_fields()
 
     def get(self, field):
@@ -84,20 +97,25 @@ class Model(object):
         if field in self.list_fields():
             return self.__proxy__.get(field)
         else:
-            raise KeyError('Key \"%s\" not in model. Available keys are %s.' % (field, ', '.join(self.keys())))
+            raise KeyError('Field \"%s\" not in model. Available fields are %s.' % (field, ', '.join(self.list_fields())))
 
     def summary(self):
-        """Print a summary of the model including a description of training
+        """
+        Print a summary of the model.
+        The summary includes a description of training
         data, options, hyper-parameters, and statistics measured during model
-        creation."""
-        raise NotImplementedError
+        creation.
+
+        Examples
+        --------
+        >>> m.summary()
+        """
+        _mt._get_metric_tracker().track(self.__class__.__module__ + '.summary')
+        print self.__repr__()
 
     def __getitem__(self, key):
         return self.get(key)
 
-    def keys(self):
-        return self.list_fields()
-
     def _get_wrapper(self):
         """Return a lambda function: UnityModel -> M, for constructing model
         class M from a UnityModel proxy."""
@@ -106,22 +124,28 @@ class Model(object):
     def save(self, location):
         """
         Save the model. The model is saved as a directory which can then be
-        loaded using the ``load_model`` method.
+        loaded using the :py:func:`~graphlab.load_model` method.
 
         Parameters
         ----------
         location : string
             Target destination for the model. Can be a local path or remote URL.
+
+        See Also
+        ----------
+        graphlab.load_model
+
+        Examples
+        ----------
+        >>> model.save('my_model_file')
+        >>> loaded_model = gl.load_model('my_model_file')
+
         """
         _mt._get_metric_tracker().track('toolkit.model.save')
         return glconnect.get_unity().save_model(self, make_internal_url(location))
 
     def __repr__(self):
-        ret = "Class: " + self.name() + "\n"
-        ret += "Summary:\n"
-        summary = self.summary()
-        for k in summary:
-            ret += "\t %s: %s\n" % (k, summary[k])
-        ret += "Available fields for get():\n\t" + str(self.list_fields()) + "\n"
-        ret += "\n"
-        return ret
+        raise NotImplementedError
+
+    def __str__(self):
+        return self.__repr__()
