@@ -14,14 +14,14 @@ def create(dataset, label, features=None, distance='auto', method='auto',
     """
     Create a nearest neighbor model, which can be searched efficiently and
     quickly for the nearest neighbors of a query observation. If the `method`
-    argument is specified as 'auto', the type of model is chosen automatically
-    based on the type of data in 'dataset'.
+    argument is specified as `auto`, the type of model is chosen automatically
+    based on the type of data in `dataset`.
 
     Parameters
     ----------
     dataset : SFrame
         Reference data. This SFrame must have a column with labels for each row.
-        
+
     label : string
         Name of the SFrame column with row labels.
 
@@ -30,7 +30,7 @@ def create(dataset, label, features=None, distance='auto', method='auto',
         observations and the query points. 'None' (the default) indicates that
         all columns except the label should be used as features. Each column can
         be one of the following types:
-        
+
         - *Numeric*: values of numeric type integer or float.
 
         - *Array*: list of numeric (integer or float) values. Each list element
@@ -41,47 +41,33 @@ def create(dataset, label, features=None, distance='auto', method='auto',
 
         Columns of type *list* are not supported. Convert them to array columns
         if all entries in the list are of numeric types.
-          
+
     distance : {'auto', 'euclidean', 'manhattan', 'jaccard', 'cosine'}, optional
+
         Function that measures the distances between two observations. Please
-        see the references for detailed descriptions of the distances. Note that
-        for sparse vectors, euclidean, manhattan, and cosine distance assume
-        missing keys have value 0.0. For the distances below, suppose :math:`u`
-        and :math:`v` are observations with :math:`d` variables each.
+        see the notes and references for detailed descriptions of the distances.
+        Note that for sparse vectors, euclidean, manhattan, and cosine distance
+        assume missing keys have value 0.0.
 
         - *auto* (default): the model chooses a reasonable distance based on the
           data types in 'dataset'.
 
-        - *euclidean*:
+        - *euclidean*
 
-            .. math:: D(u, v)
-                = \\sqrt{\sum_i^d (u_i - v_i)^2}
-
-        - *manhattan*:
-
-            .. math:: D(u, v)
-                = \\sum_i^d |u_i - v_i|
+        - *manhattan*
 
         - *jaccard*: works only with variables in a dictionary feature, where
           the keys are treated as a set and the values are ignored. Suppose
           :math:`S` and :math:`T` are the sets of keys from two observations'
           dictionaries.
 
-            .. math:: D(S, T)
-                = 1 - \\frac{|S \cap T|}{|S \cup T|}
-
         - *cosine*: works only with the 'brute-force' method because it is not a
           true metric. Please see `Wikipedia
           <http://en.wikipedia.org/wiki/Cosine_similarity>`_ for more detail.
 
-            .. math:: D(u, v)
-                = 1 - \\frac{\sum_i^d u_i v_i}
-                {\sqrt{\sum_i^d u_i^2}
-                \sqrt{\sum_i^d v_i^2}}
-
     method : {'auto', 'ball-tree', 'brute-force'}, optional
         Method for computing nearest neighbors. The options are:
-        
+
         - *auto* (default): the method is chosen automatically, based on the
           type of data and the distance. If the distance is 'manhattan' or
           'euclidean' and the features are numeric or vectors of numeric values,
@@ -121,6 +107,10 @@ def create(dataset, label, features=None, distance='auto', method='auto',
         A structure for efficiently computing the nearest neighbors in 'dataset'
         of new query points.
 
+    See Also
+    --------
+    NearestNeighborsModel.query
+
     Notes
     -----
     - If the features should be weighted equally in the distance calculations
@@ -128,54 +118,24 @@ def create(dataset, label, features=None, distance='auto', method='auto',
       features. One way to do this is to subtract the mean of each column and
       divide by the standard deviation.
 
-    Examples
-    --------
-    *Training*
+    - Distance definitions. Suppose :math:`u` and :math:`v` are observations
+      with :math:`d` variables each.
 
-    Given an :class:`~graphlab.SFrame` ``sf`` with a list of columns
-    [``feature_1`` ... ``feature_K``] denoting features and a label column
-    ``row_label``, create a
-    :class:`~graphlab.nearest_neighbors.NearestNeighborsModel`:
+        - `euclidean`
+            
+            .. math:: D(u, v) = \\sqrt{\sum_i^d (u_i - v_i)^2}
 
-    >>> import graphlab as gl
-    >>> model = gl.nearest_neighbors.create(sf, 'row_label')
+        - `manhattan`
+            .. math:: D(u, v) = \\sum_i^d |u_i - v_i|
 
-    By default, all columns in the training data except the label are used as
-    features. It's also possible to select a subset of columns in the SFrame:
+        - `jaccard`
+            .. math:: D(S, T) = 1 - \\frac{|S \cap T|}{|S \cup T|}
 
-    >>> model = gl.nearest_neighbors.create(sf, 'label', ['feature_1', 'feature_2'])
-
-    The distance function and model type are chosen automatically based on the
-    type of data in 'sf', unless otherwise specified. For datasets with a large
-    number of rows and up to about 100 variables, the ball tree method often
-    leads to much faster queries.
-
-    >>> model = gl.nearest_neighbors.create(sf, 'label',
-                                            ['feature_1', 'feature_2'],
-                                            distance='manhattan',
-                                            method='ball-tree')
-
-    If the ball tree is used, it's important to choose an approriate value for
-    the 'leaf_size' parameter, which controls how many observations are stored
-    in each leaf of the ball tree. By default, this is set so that the tree is
-    no more than 12 levels deep, but larger or smaller values may lead to
-    quicker queries depending on the shape and dimension of the data.
-
-    >>> model = gl.nearest_neighbors.create(sf, 'label',
-                                            ['feature_1', 'feature_2'],
-                                            distance='manhattan',
-                                            method='ball-tree',
-                                            leaf_size=2000)
-
-    *Querying*
-
-    Queries should be passed in the form of an SFrame that includes the same
-    columns used to train the model (other columns may be present, but are
-    ignored). The same SFrame may be used as queries to get all-point nearest
-    neighbors.
-
-    >>> knn = model.query(sf, 'row_label', features=['feature_1', 'feature_2'], k=5)
+        - `cosine`
+            .. math::
 
+                D(u, v) = 1 - \\frac{\sum_i^d u_i v_i}
+                {\sqrt{\sum_i^d u_i^2}\sqrt{\sum_i^d v_i^2}}
 
     References
     ----------
@@ -195,12 +155,29 @@ def create(dataset, label, features=None, distance='auto', method='auto',
 
     - `Wikipedia - Cosine distance
       <http://en.wikipedia.org/wiki/Cosine_similarity>`_
+
+    Examples
+    --------
+    Construct a nearest neighbors model with automatically determined method and
+    distance:
+
+    >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                              'feature1': [0.98, 0.62, 0.11],
+                              'feature2': [0.69, 0.58, 0.36]})
+    >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+
+    For datasets with a large number of rows and up to about 100 variables, the
+    ball tree method often leads to much faster queries.
+
+    >>> model = graphlab.nearest_neighbors.create(sf, 'label',
+                                                  distance='manhattan',
+                                                  method='ball-tree')
     """
 
     ## Validate input
-    if not (isinstance(dataset, _SFrame) or 
-                    (_HAS_PANDAS and isinstance(_pandas.DataFrame))):
-        raise TypeError("Input 'dataset' modulest be an SFrame.")
+    if not (isinstance(dataset, _SFrame) or
+                    (_HAS_PANDAS and isinstance(dataset, _pandas.DataFrame))):
+        raise TypeError("Input 'dataset' must be an SFrame or pandas.DataFrame")
 
     if type(dataset) != _SFrame:
         dataset = _SFrame(dataset)
@@ -374,13 +351,32 @@ class NearestNeighborsModel(Model):
     def summary(self):
         """
         Display a summary of the NearestNeighborsModel.
+
+        Examples
+        --------
+        >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                  'feature1': [0.98, 0.62, 0.11],
+                                  'feature2': [0.69, 0.58, 0.36]})
+        >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+        >>> model.summary()
+                                    Model summary
+        --------------------------------------------------------
+        Class                   : NearestNeighborsModel
+        Distance                : euclidean
+        Method                  : ball tree
+        Examples                : 3
+        Features                : 2
+        Variables               : 2
+        Training time (sec)     : 0.0232
+        Tree depth              : 1
+        Leaf size               : 1000
         """
 
         _mt._get_metric_tracker().track('toolkit.nearest_neighbors.summary')
 
         print ""
         print "                    Model summary                       "
-        print "--------------------------------------------------------"        
+        print "--------------------------------------------------------"
         print self.__repr__()
 
     def list_fields(self):
@@ -392,6 +388,28 @@ class NearestNeighborsModel(Model):
         -------
         out : list
             List of fields queryable with the ``get`` method.
+
+        See Also
+        --------
+        get
+
+        Examples
+        --------
+         >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                  'feature1': [0.98, 0.62, 0.11],
+                                  'feature2': [0.69, 0.58, 0.36]})
+        >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+        >>> model.list_fields()
+        ['distance',
+         'features',
+         'label',
+         'leaf_size',
+         'method',
+         'num_examples',
+         'num_features',
+         'num_variables',
+         'train_time',
+         'tree_depth']
         """
 
         _mt._get_metric_tracker().track('toolkit.nearest_neighbors.list_fields')
@@ -412,18 +430,26 @@ class NearestNeighborsModel(Model):
         +-----------------------+----------------------------------------------+
         |      Field            | Description                                  |
         +=======================+==============================================+
-        | train_time           | Time to create the reference structure       |
+        | distance              | Measure of dissimilarity between two points  |
         +-----------------------+----------------------------------------------+
         | features              | Feature column names                         |
         +-----------------------+----------------------------------------------+
         | label                 | Label column names                           |
         +-----------------------+----------------------------------------------+
+        | leaf_size             | Max size of leaf nodes (ball tree only)      |
+        +-----------------------+----------------------------------------------+
+        | method                | Method of organizing reference data          |
+        +-----------------------+----------------------------------------------+
         | num_examples          | Number of reference data observations        |
         +-----------------------+----------------------------------------------+
         | num_features          | Number of features for distance computation  |
         +-----------------------+----------------------------------------------+
         | num_variables         | Number of variables for distance computation |
         +-----------------------+----------------------------------------------+
+        | train_time            | Time to create the reference structure       |
+        +-----------------------+----------------------------------------------+
+        | tree_depth            | Number of levels in the tree (ball tree only)|
+        +-----------------------+----------------------------------------------+
 
         Parameters
         ----------
@@ -434,6 +460,22 @@ class NearestNeighborsModel(Model):
         -------
         out
             Value of the requested field.
+
+        See Also
+        --------
+        list_fields
+
+        Examples
+        --------
+        >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                  'feature1': [0.98, 0.62, 0.11],
+                                  'feature2': [0.69, 0.58, 0.36]})
+        >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+        >>> model.get('num_features')
+        2
+
+        >>> model['train_time']
+        0.023223
         """
 
         _mt._get_metric_tracker().track('toolkit.nearest_neighbors.get')
@@ -455,6 +497,19 @@ class NearestNeighborsModel(Model):
         -------
         out : dict
             Dictionary with the default options.
+
+        See Also
+        --------
+        get_current_options, list_fields, get
+
+        Examples
+        --------
+        >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                  'feature1': [0.98, 0.62, 0.11],
+                                  'feature2': [0.69, 0.58, 0.36]})
+        >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+        >>> model.get_default_options()
+        {'distance': 'auto', 'leaf_size': 0}
         """
 
         _mt._get_metric_tracker().track(
@@ -475,6 +530,19 @@ class NearestNeighborsModel(Model):
         out : dict
             Dictionary of options used to train the current instance of the
             NearestNeighborsModel.
+
+        See Also
+        --------
+        get_default_options, list_fields, get
+
+        Examples
+        --------
+        >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                  'feature1': [0.98, 0.62, 0.11],
+                                  'feature2': [0.69, 0.58, 0.36]})
+        >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+        >>> model.get_current_options()
+        {'distance': 'euclidean', 'leaf_size': 1000}
         """
 
         _mt._get_metric_tracker().track(
@@ -496,6 +564,26 @@ class NearestNeighborsModel(Model):
         out : dict
             Dictionary of statistics compiled during creation of the
             NearestNeighborsModel.
+
+        See Also
+        --------
+        summary
+
+        Examples
+        --------
+        >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                  'feature1': [0.98, 0.62, 0.11],
+                                  'feature2': [0.69, 0.58, 0.36]})
+        >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+        >>> model.training_stats()
+        {'features': 'feature1, feature2',
+         'label': 'label',
+         'leaf_size': 1000,
+         'num_examples': 3,
+         'num_features': 2,
+         'num_variables': 2,
+         'train_time': 0.023223,
+         'tree_depth': 1}
         """
 
         _mt._get_metric_tracker().track(
@@ -545,6 +633,33 @@ class NearestNeighborsModel(Model):
             observation, the third is the distance between the query and
             reference observations, and the fourth is the rank of the reference
             observation among the query's k-nearest neighbors.
+
+        Examples
+        --------
+        First construct a toy SFrame and create a nearest neighbors model:
+
+        >>> sf = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                  'feature1': [0.98, 0.62, 0.11],
+                                  'feature2': [0.69, 0.58, 0.36]})
+        >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+
+        A new SFrame contains query observations with same schema as the
+        reference SFrame. This SFrame is passed to the ``query`` method.
+
+        >>> queries = graphlab.SFrame({'label': [str(x) for x in range(3)],
+                                      'feature1': [0.05, 0.61, 0.99],
+                                      'feature2': [0.06, 0.97, 0.86]})
+        >>> model.query(queries, 'label', k=2)
+        +-------------+-----------------+----------------+------+
+        | query_label | reference_label |    distance    | rank |
+        +-------------+-----------------+----------------+------+
+        |      0      |        2        | 0.305941170816 |  1   |
+        |      0      |        1        | 0.771556867638 |  2   |
+        |      1      |        1        | 0.390128184063 |  1   |
+        |      1      |        0        | 0.464004310325 |  2   |
+        |      2      |        0        | 0.170293863659 |  1   |
+        |      2      |        1        | 0.464004310325 |  2   |
+        +-------------+-----------------+----------------+------+
         """
 
         _mt._get_metric_tracker().track(
@@ -574,7 +689,7 @@ class NearestNeighborsModel(Model):
                              "the query SFrame 'dataset'.")
 
         if not isinstance(k, (int, float)) or k <= 0:
-            raise ValueError("Input 'k' must be a single number greater than 0.") 
+            raise ValueError("Input 'k' must be a single number greater than 0.")
 
 
         ## Extract the features and labels
