@@ -10,79 +10,53 @@ from graphlab.data_structures.sarray import SArray as _SArray
 from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
 import logging
 
-
-def create(dataset,
-           user_column='user_id', item_column='item_id', target_column=None,
-           user_data=None, item_data=None,
-           similarity_type='jaccard',
-           only_top_k=0, threshold=0.0,
-           threshold_degree=None, threshold_window=None,
-           verbose=True, plot=False, **method_options):
-    """
-    *Deprecated*. Please use
-
-    ``graphlab.recommender.create(..., method="item_similarity")``
-
-    instead.
-    """
-
-    _mt._get_metric_tracker().track('toolkit.recsys.item_similarity.create')
-
-    logging.warning("item_similarity.create will be deprecated soon. Please switch to recommender.create")
-
-    if not (isinstance(dataset, _SFrame) or (_HAS_PANDAS and isinstance(dataset, _pandas.DataFrame))):
-        raise TypeError('dataset input must be a pandas.DataFrame or SFrame')
-
-    if type(dataset) != _SFrame:
-        dataset = _SFrame(dataset)
-
-    if plot is True:
-        print "The plot functionality for item similarity is not yet implemented."
-        plot = False
-
-    method_options = {'similarity_type': similarity_type,
-                      'only_top_k': only_top_k,
-                      'threshold': threshold}
-
-    # Add threshold options if provided by the user.
-    if threshold_degree is not None:
-        method_options.update({'threshold_lower_degree': threshold_degree})
-
-    if threshold_window is not None:
-        method_options.update({'threshold_window_degree': threshold_window})
-
-
-    m = _graphlab.recommender.create(dataset,
-            user_column=user_column, item_column=item_column, target_column=target_column,
-            user_data=user_data, item_data=item_data,
-            verbose=verbose,
-            method='item_similarity',
-            **method_options)
-
-    return m
-
-
 class ItemSimilarityModel(RecommenderModel):
     """
     A model that ranks an item according to its similarity to other items
-    observed for the user in question. The similarity can be computed based on
-    the observed users for each item, e.g. the rating given to both items by
-    users who have rated both items.
+    observed for the user in question. 
 
-    An instance of this model can be created using
-    :func:`create(..., method='item_similarity') <graphlab.recommender.create>`.
-    Do NOT construct the model directly.
+    **Creating an ItemSimilarityModel**
 
+    This model can be created using
+    :func:`graphlab.recommender.create(..., method='item_similarity') <graphlab.recommender.create>`.
+    Do NOT instantiate this model class directly.
+
+    When creating a ItemSimilarityModel, the following options may be 
+    specified:
+
+    +---------------------------+---------+-------------------------------+
+    |        Options            | Default |   Description                 |
+    +===========================+=========+===============================+
+    | similarity_type           | jaccard | Similarity metric to use.     |
+    |                           |         | One of 'jaccard', 'cosine',   |
+    |                           |         | or 'pearson'.                 |
+    +---------------------------+---------+-------------------------------+
+    | threshold                 | 0.001   | Predictions ignore items      |
+    |                           |         | below this similarity value.  |
+    +---------------------------+---------+-------------------------------+
+    | only_top_k                | 100     | Number of similar items to    |
+    |                           |         | store for each items. Default |
+    |                           |         | value is the number of items. |
+    +---------------------------+---------+-------------------------------+
+    | nearest_items             | None    | See notes.                    |
+    +---------------------------+---------+-------------------------------+
+
+    **Side information**
+
+    Currently, ItemSimilarityModel does not support side features.
+
+    Notes
+    -----
     **Model Definition**
 
     A model with ``method=item_similarity`` first computes the similarity
-    between items using the observations of users who have rated those two
+    between items using the observations of users who have interacted with both
     items. Given a similarity between item :math:`i` and :math:`j`,
-    :math:`S(i,j)`, we score an item :math:`j` for user :math:`u` using a
+    :math:`S(i,j)`, it scores an item :math:`j` for user :math:`u` using a
     weighted average of the user's previous observations :math:`I_u`.
 
-    There are three choices of similarity metrics to use: ``jaccard``,
-    ``cosine`` and ``pearson``.
+    There are three choices of similarity metrics to use: 'jaccard',
+    'cosine' and 'pearson'.
 
     `Jaccard similarity
     <http://en.wikipedia.org/wiki/Jaccard_index>`_
@@ -96,10 +70,7 @@ class ItemSimilarityModel(RecommenderModel):
     where :math:`U_{i}` is the set of users who rated item :math:`i`.
     Jaccard is a good choice when one only has implicit feedbacks of items
     (e.g., people rated them or not), or when one does not care about how
-    many stars items received. A prediction for item :math:`j` is made via
-
-    .. math:: y_{uj}
-            = \\frac{\sum_{i \in I_u} \mbox{JS}(i,j)  }{|I_u|}
+    many stars items received.
 
     If one needs to compare the ratings of items, Cosine and Pearson similarity
     are recommended.
@@ -115,12 +86,9 @@ class ItemSimilarityModel(RecommenderModel):
     and :math:`U_{ij}` is the set of users who rated both items :math:`i` and
     :math:`j`. A problem with Cosine similarity is that it does not consider
     the differences in the mean and variance of the ratings made to
-    items :math:`i` and :math:`j`. Predictions are made via
+    items :math:`i` and :math:`j`.
 
-    .. math:: y_{uj}
-            = \\frac{\sum_{i \in I_u} \mbox{CS}(i,j) r_{ui} }{\sum_{i \in I_u} \mbox{CS}(i,j)}
-
-    A popular measure that compares ratings where the effects of means and
+    Another popular measure that compares ratings where the effects of means and
     variance have been removed is Pearson Correlation similarity:
 
     .. math:: \mbox{PS}(i,j)
@@ -129,13 +97,28 @@ class ItemSimilarityModel(RecommenderModel):
                 {\sqrt{\sum_{u\in U_{ij}} (r_{ui} - \\bar{r}_i)^2}
                  \sqrt{\sum_{u\in U_{ij}} (r_{uj} - \\bar{r}_j)^2}}
 
-    Predictions are made via
+    The predictions of items depend on whether `target` is specified.
+    When the `target` is absent, a prediction for item :math:`j` is made via
 
     .. math:: y_{uj}
-            = \\frac{\sum_{i \in I_u} \mbox{PS}(i,j) r_{ui} }{\sum_{i \in I_u} |\mbox{PS}(i,j)|}
+            = \\frac{\sum_{i \in I_u} \mbox{SIM}(i,j)  }{|I_u|}
+
+
+    Otherwise, predictions for ``jaccard`` and ``cosine`` similarities are made via
+
+    .. math:: y_{uj}
+            = \\frac{\sum_{i \in I_u} \mbox{SIM}(i,j) r_{ui} }{\sum_{i \in I_u} \mbox{SIM}(i,j)}
+
+    Predictions for ``pearson`` similarity are made via
+
+    .. math:: y_{uj}
+            = \\bar{r}_j + \\frac{\sum_{i \in I_u} \mbox{SIM}(i,j) (r_{ui} - \\bar{r}_i) }{\sum_{i \in I_u} \mbox{SIM}(i,j)}
+
 
     For more details of item similarity methods, please see, e.g.,
-    Chapter 4 of [1]_
+    Chapter 4 of [Ricci_et_al]_.
+
+    **Incorporating pre-defined similar items**
 
     For item similarity models, one may choose to provide user-specified
     nearest neighbors graph using the keyword argument `nearest_items`. This is
@@ -147,21 +130,25 @@ class ItemSimilarityModel(RecommenderModel):
     you first create an ItemSimilarityModel and use
     :class:`~graphlab.recommender.ItemSimilarityModel.get_similar_items`:
 
-    >>> m = gl.recommender.create(data, 'user', 'item',
-                                  method='item_similarity')
+    >>> sf = graphlab.SFrame({'user_id': ["0", "0", "0", "1", "1", "2", "2", "2"],
+    ...                       'item_id': ["a", "b", "c", "a", "b", "b", "c", "d"]})
+    >>> m = graphlab.recommender.create(sf, method='item_similarity')
     >>> nn = m.get_similar_items()
-    >>> m2 = gl.recommender.create(data, 'user', 'item',
-                                   method='item_similarity',
-                                   nearest_items=nn)
+    >>> m2 = graphlab.recommender.create(sf,
+                                         method='item_similarity',
+                                         nearest_items=nn)
 
     With the above code, the item similarities computed for model `m` can be
     used to create a new recommender object, `m2`. Note that we could have
     created `nn` from some other means, but now use `m2` to make
     recommendations via `m2.recommend()`.
 
-    .. [1] Ricci, Francesco, Lior Rokach, and Bracha Shapira.
-            Introduction to recommender systems handbook. Springer US, 2011.
-
+    References
+    ----------
+    .. [Ricci_et_al] Francesco Ricci, Lior Rokach, and Bracha Shapira.
+        `Introduction to recommender systems handbook
+        <http://www.ics.uci.edu/~smyth/courses/cs27
+        7/papers/intro-rec-sys-handbook.pdf>`_. Springer US, 2011.
     """
 
     def __init__(self, model_proxy):
@@ -181,23 +168,31 @@ class ItemSimilarityModel(RecommenderModel):
 
         Parameters
         ----------
-        items: SArray or list; optional
-            An :class:`~graphlab.SArray` (or list) of items for which to get similar items.
-            If 'None', then get_similar_items(...) generates most similar items for all items in the training set.
+        items : SArray or list; optional        
+            An :class:`~graphlab.SArray` or list of item ids for which to get
+            similar items. If 'None', then return the `k` most similar items for
+            all items in the training set.
 
-        k: int, optional
+        k : int, optional
             The number of similar items for each item.
 
         Returns
-        ------
+        -------
         out : SFrame
+            A SFrame with the top ranked similar items for each item. The
+            columns are `item_id`, 'similar_item', 'score' and 'rank', where
+            `item_id` matches the item column name specified at training time.
+            The 'rank' is between 1 and `k` and 'score' gives the similarity
+            score of that item. The value of the score depends on the method
+            used for computing item similarities.
 
-            A SFrame with the top ranked similar items for each item.
-            The columns are ``item_column``, *similar_item*, *sore* and *rank*,
-            where ``item_column`` matches the item column name specified at training time.
-            The *rank* is between 1 and ``k`` and *score* gives the similarity score of that item.
-            The value of the score depends on the method used for computing item similarities.
+        Examples
+        --------
 
+        >>> sf = graphlab.SFrame({'user_id': ["0", "0", "0", "1", "1", "2", "2", "2"],
+                                  'item_id': ["a", "b", "c", "a", "b", "b", "c", "d"]})
+        >>> m = graphlab.recommender.create(sf, method='item_similarity')
+        >>> nn = m.get_similar_items()
         """
 
         _mt._get_metric_tracker().track('toolkit.recsys.get_similar_items')
