@@ -43,19 +43,19 @@ class RecommenderModel(Model):
 
         Returns
         -------
-        The current value of the requested field.
-
-        Example
-        -------
-
-        m = gl.recommender.create(data, "user_id", "movie_id", "rating",
-                                  method = "matrix_factorization")
-
-        d = m.get("coefficients")
-
-        U1 = d['user_id']
-        U2 = d['movie_id']
+        out
+            The current value of the requested field.
 
+        Examples
+        --------
+        >>> data = graphlab.SFrame({'user_id': ["0", "0", "0", "1", "1", "2", "2", "2"],
+        ...                         'item_id': ["a", "b", "c", "a", "b", "b", "c", "d"],
+        ...                         'rating': [1, 3, 2, 5, 4, 1, 4, 3]})
+        >>> m = graphlab.recommender.create(data, "user_id", "item_id", "rating",
+        ...                                 method = "matrix_factorization")
+        >>> d = m.get("coefficients")
+        >>> U1 = d['user_id']
+        >>> U2 = d['movie_id']
         """
 
         opts = {'model': self.__proxy__, 'field': field}
@@ -86,7 +86,8 @@ class RecommenderModel(Model):
 
 
     def summary(self):
-        """Returns a summary of statistics about the model, the data
+        """
+        Returns a summary of statistics about the model, the data
         used to train the model, and the training performance.
 
         Returns
@@ -137,7 +138,7 @@ class RecommenderModel(Model):
         if is_trained:
 
             ret += "\nSchema\n"
-            for k in ["user_column", "item_column", "target_column"]:
+            for k in ["user_id", "item_id", "target"]:
                 if k in options.keys():
                     if options[k] not in ['', 'not_specified'] and options[k] is not None:
                         ret += "  %-14s %s\n" % \
@@ -201,9 +202,9 @@ class RecommenderModel(Model):
 
         # Remove any options that should not be shown under "Settings"
         to_ignore = ['random_seed',
-                     'user_column',
-                     'item_column',
-                     'target_column']
+                     'user_id',
+                     'item_id',
+                     'target']
         for k in to_ignore:
             if k in options:
                 del options[k]
@@ -261,8 +262,8 @@ class RecommenderModel(Model):
         response = _graphlab.toolkits.main.run('recsys_set_current_options', opts)
         return response
 
-    def score(self, dataset,
-              new_observation_data=None, new_user_data=None, new_item_data=None):
+    def predict(self, dataset,
+                new_observation_data=None, new_user_data=None, new_item_data=None):
         """
         Return a score prediction for the user ids and item ids in the provided
         data set.
@@ -272,14 +273,14 @@ class RecommenderModel(Model):
         dataset : SFrame
             Dataset in the same form used for training.
 
-        new_observation_data: SFrame, optional
+        new_observation_data : SFrame, optional
             ``new_observation_data`` gives additional observation data
             to the model, which may be used by the models to improve
             score accuracy.  Must be in the same format as the
             observation data passed to ``create``.  How this data is
             used varies by model.
 
-        new_user_data: SFrame, optional
+        new_user_data : SFrame, optional
             ``new_user_data`` may give additional user data to the
             model.  If present, scoring is done with reference to this
             new information.  If there is any overlap with the side
@@ -287,7 +288,7 @@ class RecommenderModel(Model):
             data is preferred.  Must be in the same format as the user
             data passed to ``create``.
 
-        new_item_data: SFrame, optional
+        new_item_data : SFrame, optional
             ``new_item_data`` may give additional item data to the
             model.  If present, scoring is done with reference to this
             new information.  If there is any overlap with the side
@@ -300,8 +301,12 @@ class RecommenderModel(Model):
         out : SArray
             An SArray with predicted scores for each given observation
             predicted by the model.
+
+        See Also
+        --------
+        recommend, evaluate
         """
-        _mt._get_metric_tracker().track('toolkit.recsys.score')
+        _mt._get_metric_tracker().track('toolkit.recsys.predict')
 
         if _HAS_PANDAS and type(dataset) == _pandas.DataFrame:
             dataset = _SFrame(dataset)
@@ -352,15 +357,17 @@ class RecommenderModel(Model):
 
         Parameters
         ----------
-        users:  SArray or list; optional
+        users :  SArray or list; optional
             An :class:`~graphlab.SArray` (or list) of users for which
             to make recommendations.  If 'None', then recommend(...)
             generates predictions for all users in the training set.
+            The type of the SArray or list must be the same as the type 
+            of the user_id column in the training data.
 
-        k: int, optional
+        k : int, optional
             The number of recommendations to generate for each user.
 
-        items: SArray, optional
+        items : SArray, optional
             To restrict the set of items under consideration, the user
             may pass in an SArray of items from which the recommended
             items will be chosen.  This allows one to choose only
@@ -368,14 +375,14 @@ class RecommenderModel(Model):
             category.  The default is that all items are under
             consideration.
 
-        new_observation_data: SFrame, optional
+        new_observation_data : SFrame, optional
             ``new_observation_data`` gives additional observation data
             to the model, which may be used by the models to improve
             score and recommendation accuracy.  Must be in the same
             format as the observation data passed to ``create``.  How
             this data is used varies by model.
 
-        new_user_data: SFrame, optional
+        new_user_data : SFrame, optional
             ``new_user_data`` may give additional user data to the
             model.  If present, scoring is done with reference to this
             new information.  If there is any overlap with the side
@@ -383,7 +390,7 @@ class RecommenderModel(Model):
             data is preferred.  Must be in the same format as the user
             data passed to ``create``.
 
-        new_item_data: SFrame, optional
+        new_item_data : SFrame, optional
             ``new_item_data`` may give additional item data to the
             model.  If present, scoring is done with reference to this
             new information.  If there is any overlap with the side
@@ -391,7 +398,7 @@ class RecommenderModel(Model):
             data is preferred.  Must be in the same format as the item
             data passed to ``create``.
 
-        exclude: SFrame, optional
+        exclude : SFrame, optional
             An :class:`~graphlab.SFrame` of user / item pairs.  The
             column names must be equal to the user and item columns of
             the main data, and it provides the model with user/item
@@ -399,28 +406,31 @@ class RecommenderModel(Model):
             user-item-pairs are always excluded from the predictions,
             even if exclude_known is False.
 
-        exclude_known: bool
+        exclude_known : bool, optional
             By default, all user-item interactions previously seen in
             the training data, or in any new data provided using
             new_observation_data=..., are excluded from the
             recommendations.  Passing in ``exclude_known = False``
             overrides this behavior.
 
-        verbose: bool
+        verbose : bool, optional
             If True, print the progress of generating recommendation.
 
         Returns
         -------
         out : SFrame
             A SFrame with the top ranked items for each user. The
-            columns are: ``user_column``, ``item_column``, *score*,
-            and *rank*, where ``user_column`` and ``item_column``
+            columns are: ``user_id``, ``item_id``, *score*,
+            and *rank*, where ``user_id`` and ``item_id``
             match the user and item column names specified at training
             time.  The rank column is between 1 and ``k`` and gives
             the relative score of that item.  The value of score
             depends on the method used for recommendations.
 
-
+        See Also
+        --------
+        predict
+        evaluate
         """
         _mt._get_metric_tracker().track('toolkit.recsys.recommend')
         assert type(k) == int
@@ -498,9 +508,9 @@ class RecommenderModel(Model):
         response = _graphlab.toolkits.main.run("recsys_get_stats", opts)
         return response
 
-    def _evaluate_precision_recall(self, dataset, cutoffs=[5, 10, 15], skip_set=None, exclude_known=True, verbose=True):
+    def evaluate_precision_recall(self, dataset, cutoffs=[5, 10, 15], skip_set=None, exclude_known=True, verbose=True):
         """
-        Compute a model's precision and recall for a particular dataset.
+        Compute a model's precision and recall scores for a particular dataset.
 
         Parameters
         ----------
@@ -509,62 +519,78 @@ class RecommenderModel(Model):
             This will be compared to the model's recommendations, which exclude
             the (user, item) pairs seen at training time.
 
-        cutoffs : list
+        cutoffs : list, optional
             A list of cutoff values for which one wants to evaluate precision
-            and recall, i.e. the value of k in precision@k.
+            and recall, i.e. the value of k in "precision at k".
 
-        skip_set: SFrame
-            Pass to recommend() as exclude
+        skip_set : SFrame, optional
+            Passed to :meth:`recommend` as ``exclude``.
 
-        exclude_known : bool
-            Pass to recommend() as exclude_known
-            If True, exclude training item from recommendation.
+        exclude_known : bool, optional
+            Passed to :meth:`recommend` as ``exclude_known``. If True, exclude
+            training item from recommendation.
+
+        verbose : bool, optional
+            Enables verbose output. Default is verbose.
 
         Returns
         -------
         out : SFrame
             Contains the precision and recall at each cutoff value and each
             user in ``dataset``.
+
+        See Also
+        --------
+        precision_recall_by_user
         """
 
-        users = dataset[self.get('user_column')].unique()
+        _mt._get_metric_tracker().track('toolkit.recsys.evaluate_precision_recall')
 
-        dataset = dataset[[self.get('user_column'), self.get('item_column')]]
+        users = dataset[self.get('user_id')].unique()
+
+        dataset = dataset[[self.get('user_id'), self.get('item_id')]]
 
         recs = self.recommend(users=users, k=max(cutoffs), exclude=skip_set, exclude_known=exclude_known, verbose=verbose)
         pr = _graphlab.recommender.util.precision_recall_by_user(dataset, recs, cutoffs)
         return pr
 
-    def _evaluate_rmse(self, dataset, target_column):
+    def evaluate_rmse(self, dataset, target):
         """
         Evaluate the prediction error for each user-item pair in the given data
         set.
 
         Parameters
         ----------
-        dataset : SFrame/pandas.DataFrame
-            A DataFrame in the same format as the one used during training.
+        dataset : SFrame
+            An SFrame in the same format as the one used during training.
 
-        target_column : str
-            The target column to evaluate rmse on.
+        target : str
+            The name of the target rating column in `dataset`.
 
         Returns
         -------
         out : dict
-            A dictionary with three items: *rmse_by_user* and *rmse_by_item*, which
-            are SFrames; and *rmse_overall*, which is a float.
+            A dictionary with three items: 'rmse_by_user' and 'rmse_by_item',
+            which are SFrames containing the average rmse for each user and
+            item, respectively; and 'rmse_overall', which is a float.
+
+        See Also
+        --------
+        graphlab.evaluation.rmse
         """
 
+        _mt._get_metric_tracker().track('toolkit.recsys.evaluate_rmse')
+
         if _HAS_PANDAS and type(dataset) == _pandas.DataFrame:
             dataset = _SFrame(dataset)
 
-        assert target_column in dataset.column_names(), \
+        assert target in dataset.column_names(), \
                'Provided dataset must contain a target column with the same \
                 name as the target used during training.'
-        y = dataset[target_column]
-        yhat = self.score(dataset)
-        user_column = self.get('user_column')
-        item_column = self.get('item_column')
+        y = dataset[target]
+        yhat = self.predict(dataset)
+        user_column = self.get('user_id')
+        item_column = self.get('item_id')
         assert user_column in dataset.column_names() and \
                item_column in dataset.column_names(), \
             'Provided data set must have a column pertaining to user ids and \
@@ -588,81 +614,93 @@ class RecommenderModel(Model):
 
     def evaluate(self, dataset, metric='auto',
                  exclude_known_for_precision_recall=True,
-                 target_column=None,
+                 target=None,
                  verbose=False, **kwargs):
-        r"""Evaluate the model's ability to make predictions for another
-        dataset. If the model is trained to predict a particular target, the
+        r"""
+        Evaluate the model's ability to make rating predictions or 
+        recommendations. 
+
+        If the model is trained to predict a particular target, the
         default metric used for model comparison is root-mean-squared error
         (RMSE). Suppose :math:`y` and :math:`\widehat{y}` are vectors of length
-        :math:`N` with actual and predicted ratings. Then the RMSE is
+        :math:`N`, where :math:`y` contains the actual ratings and 
+        :math:`\widehat{y}` the predicted ratings. Then the RMSE is defined as
 
         .. math::
 
-            RMSE = \sqrt{\frac{1}{N} \sum_{i=1}^N (\widehat{y}_i - y_i)^2}
+            RMSE = \sqrt{\frac{1}{N} \sum_{i=1}^N (\widehat{y}_i - y_i)^2} .
 
-        If the model was not trained on a target column during, the default
-        metrics for model comparison are precision and recall.
+        If the model was not trained on a target column, the default metrics for
+        model comparison are precision and recall. Let
+        :math:`p_k` be a vector of the :math:`k` highest ranked recommendations
+        for a particular user, and let :math:`a` be the set of items for that 
+        user in the groundtruth `dataset`. The "precision at cutoff k" is 
+        defined as
+
+        .. math:: P(k) = \frac{ | a \cap p_k | }{k}
+
+        while "recall at cutoff k" is defined as
+
+        .. math:: R(k) = \frac{ | a \cap p_k | }{|a|}
 
         Parameters
         ----------
-        dataset : SFrame/pandas.DataFrame
-            An SFrame or DataFrame that is in the same format as provided for
-            training.
+        dataset : SFrame
+            An SFrame that is in the same format as provided for training.
 
-        metric : str, {'auto', 'rmse', 'precision_recall'}
-            Metric for the evaluation. The default automatically choose
-            'rmse' for models trained with a target_column, and 'precision_recall'
+        metric : str, {'auto', 'rmse', 'precision_recall'}, optional
+            Metric to use for evaluation. The default automatically chooses
+            'rmse' for models trained with a `target`, and 'precision_recall'
             otherwise.
 
-        exclude_known_for_precision_recall : bool, optional
-            A useful option for evaluating precision-recall.
-            Recommender models have the option to exclude items seen in the training
-            data from the final recommendation list.
-            Set this option to True when evaluating on test data,
-            and False when evaluating precision-recall on training data.
+        exclude_known_for_precision_recall : bool, optional            
+            A useful option for evaluating precision-recall. Recommender models
+            have the option to exclude items seen in the training data from the
+            final recommendation list. Set this option to True when evaluating
+            on test data, and False when evaluating precision-recall on training
+            data.
 
-        target_column : str, optional
-            The target column for evaluating rmse.
-            If the model is trained with a target column, the default
-            is to using the same column.
-            If the model is trained without a target column, this option
-            must provided by user.
+        target : str, optional        
+            The name of the target column for evaluating rmse. If the model is
+            trained with a target column, the default is to using the same
+            column. If the model is trained without a target column and `metric`
+            is set to 'rmse', this option must provided by user.
 
         verbose : bool, optional
             Enables verbose output. Default is verbose.
 
-        kwargs : dict
-            Parameters passed on to internal function
-            `_evaluate_precision_recall`.
+        **kwargs
+            When `metric` is set to 'precision_recall', these parameters
+            are passed on to :meth:`evaluate_precision_recall`.
 
         Returns
         -------
-        out : SFrame
-            Results from  model evaluation procedure.
+        out : SFrame or dict
+            Results from the model evaluation procedure. If the model is trained
+            on a target (i.e. RMSE is the evaluation criterion), a dictionary
+            with three items is returned: items *rmse_by_user* and
+            *rmse_by_item* are SFrames with per-user and per-item RMSE, while
+            *rmse_overall* is the overall RMSE (a float). If the model is
+            trained without a target (i.e. precision and recall are the
+            evaluation criteria) an :py:class:`~graphlab.SFrame` is returned
+            with both of these metrics for each user at several cutoff values.
 
-        Notes
-        -----
-            If the model is trained on a target (i.e. RMSE is the evaluation
-            criterion), a dictionary with three items is returned: items
-            *rmse_by_user* and *rmse_by_item* are SFrames with per-user and
-            per-item RMSE, while *rmse_overall* is the overall RMSE (a float).
-            If the model is trained without a target (i.e. precision and recall
-            are the evaluation criteria) and :py:class:`~graphlab.SFrame` is
-            returned with both of these metrics for each user at several cutoff
-            values.
+        See Also
+        --------
+        evaluate_precision_recall, evaluate_rmse, precision_recall_by_user
         """
 
         _mt._get_metric_tracker().track('toolkit.recsys.evaluate')
 
         if metric is 'auto':
-            if self.get('target_column') is None or self.get('target_column') == '':
+            if self.get('target') is None or self.get('target') == '':
                 metric = 'precision_recall'
             else:
                 metric = 'rmse'
 
         # If the model does not have a target column, compute prec-recall.
         if metric == 'precision_recall':
-            results = self._evaluate_precision_recall(dataset,
+            results = self.evaluate_precision_recall(dataset,
                                                       exclude_known=exclude_known_for_precision_recall,
                                                       verbose=verbose,
                                                       **kwargs)
@@ -671,12 +709,12 @@ class RecommenderModel(Model):
                 print results.groupby('cutoff', {'mean_precision': _graphlab.aggregate.AVG('precision'),
                                                  'mean_recall': _graphlab.aggregate.AVG('recall')}).topk('cutoff', reverse=True)
         elif metric == 'rmse':
-            if target_column is None:
-                target_column = self.get('target_column')
-            if target_column is None or target_column == "":
-                raise ValueError('Target column cannot be None. Please provide the target_column if the model \
-                    is not trained with target_column')
-            results = self._evaluate_rmse(dataset, target_column)
+            if target is None:
+                target = self.get('target')
+            if target is None or target == "":
+                raise ValueError('Target column cannot be None. Please provide the target if the model \
+                    is not trained with a target column')
+            results = self.evaluate_rmse(dataset, target)
             if verbose:
                 print "\nOverall RMSE: ", results['rmse_overall']
                 print "\nPer User RMSE (best)"
