@@ -8,6 +8,8 @@ from graphlab.data_structures.sframe import SArray as _SArray
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
 from graphlab import vowpal_wabbit as _vw
+from graphlab.toolkits.supervised_learning import _vw_solver_warning
+
 
 DEFAULT_SOLVER_OPTIONS = {
 'convergence_threshold': 1e-2,
@@ -96,10 +98,26 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
         - *fista*: accelerated gradient descent
         - *sgd*: stochastic gradient descent
 
+        The model is trained using a carefully engineered collection of methods 
+        that are automatically picked based on the input data. The ``newton`` 
+        method  works best for datasets with plenty of examples and few features
+        (long datasets). Limited memory BFGS (``lbfgs``) is a robust solver for 
+        wide datasets (i.e datasets with many coefficients).  ``fista`` is the 
+        default solver for L1-regularized linear regression. Gradient-descent 
+        (GD) is another well tuned method that can work really well on 
+        L1-regularized problems.  The solvers are all automatically tuned and 
+        the default options should function well. See the solver options guide 
+        for setting additional parameters for each of the solvers.
+
+
     feature_rescaling: boolean, optional
-        Feature rescaling is the process of scaling individual features to
-        have unit L2-norm. This process is particularly useful when features
-        vary widely in their ranges.  (Not applicable when solver = 'vw')
+        Feature rescaling is an important pre-processing step that ensures that
+        all features are on the same scale. An L2-norm rescaling is performed
+        to make sure that all features are of the same norm. Categorical
+        features are also rescaled by rescaling the dummy variables that are
+        used to represent them. The coefficients are returned in original scale
+        of the problem. This process is particularly useful when features
+        vary widely in their ranges. (Not applicable when solver = 'vw')
 
 
     solver_options : dict, optional
@@ -121,6 +139,53 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
         | step_size             |     1.0 | Initial solver step size (fista, sgd)    |
         +-----------------------+---------+------------------------------------------+
 
+        Most solvers are all automatically tuned and the default options should
+        function well. Changing the default settings may help in some cases.
+    
+
+        auto_tuning:
+
+        Toggles wethere the step size in Stochastic Gradient Descent (SGD) is
+        automatically in training
+
+        convergence_threshold: 
+
+        Convergence is tested using variation in the training objective. The 
+        variation in the training objective is calculated using the difference 
+        between the objective values between two steps. Consider reducing this 
+        below the default value (0.01) for a more accurately trained model. 
+        Beware of overfitting (i.e a model that works well only on the training 
+        data) if this parameter is set to a very low value.
+        
+        lbfgs_memory_level: 
+
+        The L-BFGS algorithm keeps track of gradient information from the 
+        previous ``lbfgs_memory_level`` iterations. The storage requirement for 
+        each of these gradients is the ``num_coefficients`` in the problem. 
+        Increasing the ``lbfgs_memory_level ``can help improve the quality of 
+        the model trained. Setting this to more than ``max_iterations`` has the 
+        same effect as setting it to ``max_iterations``.     
+
+        max_iterations:
+
+        The maximum number of allowed passes through the data. More passes over 
+        the data can result in a more accurately trained model. Consider 
+        increasing this (the default value is 10) if the training accuracy is 
+        low and the *Grad-Norm* in the display is large.
+
+        mini_batch_size: 
+
+        The number of examples that are considered in Stochastic Gradient 
+        Descent (SGD) before a parameter update is made. 
+
+        step_size:
+
+        The starting step size to use for the ``fista`` and ``gd`` solvers. The
+        default is set to 1.0, this is an aggressive setting. If the first
+        iteration takes a considerable amount of time, reducing this parameter
+        may speed up model training. 
+
+
     verbose : bool, optional (default True)
         If True, print progress updates.
 
@@ -130,6 +195,10 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
         A trained model of type
         :class:`~graphlab.linear_regression.LinearRegressionModel`.
 
+    See Also
+    --------
+    LinearRegressionModel
+
     Notes
     -----
     - Categorical variables are encoded by creating dummy variables. For a
@@ -149,78 +218,13 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
     - A constant term is automatically added for the model intercept. This term
       is not regularized.
 
-    - For solver='vw', the "step_size" option is the "learning_rate" parameter
+    - For ``solver``='vw', the ``step_size`` option is the ``learning_rate`` parameter
       in the  :class:`~graphlab.vowpal_wabbit.VWModel`.
+    
+    .. warning::
+      The solver 'vw' will not be supported in future releases. Use 
+      graphlab.vowpal_wabbit.create() for Vowpal Wabbit.
 
-    Examples
-    --------
-    *Training*
-
-    Given an :class:`~graphlab.SFrame` ``sf`` with a list of columns
-    [``feature_1`` ... ``feature_K``] denoting features and a target column
-    ``target``, we can create a
-    :class:`~graphlab.linear_regression.LinearRegressionModel` as follows:
-
-    >>> m = linear_regression.create(sf, 'target')
-
-    By default, all columns in the training data except the target are used as
-    features. We can also select only a subset of columns in the SFrame to train
-    the model:
-
-    >>> m = linear_regression.create(sf, 'target', ['feature_1', 'feature_2'])
-
-    For ridge regression, we can set the ``L2_penalty`` parameter higher (the
-    default is 0.01). For Lasso regression, we set the L1_penalty higher, and
-    for elastic net, we set both to be higher.
-
-    >>> m_ridge = linear_regression.create(sf, 'target', L2_penalty=0.1)
-    >>> m_lasso = linear_regression.create(sf, 'target', L2_penalty=0.,
-                                           L1_penalty=1.0)
-    >>> m_enet = linear_regression.create(sf, 'target', L2_penalty=0.5,
-                                          L1_penalty=0.5)
-
-    *Model Querying*
-
-    Model attributes can be retrieved with either bracket syntax or the
-    :func:`~graphlab.linear_regression.LinearRegressionModel.get` method. The
-    set of queryable fields is described in the ``get`` documentation, and
-    can be obtained programmatically with the
-    :func:`~graphlab.linear_regression.LinearRegressionModel.list_fields`
-    method. The model coefficients are stored in an SFrame.
-
-    >>> list_of_fields = m.list_fields()
-    >>> coef = m['coefficients']  # an SFrame
-    >>> coef = m.get('coefficients')  # equivalent to previous line
-    >>> rmse = m.get('train_rmse')
-
-    The :func:`~graphlab.linear_regression.LinearRegressionModel.summary`
-    and
-    :func:`~graphlab.linear_regression.LinearRegressionModel.training_stats`
-    methods provide shortcuts to many of the model attributes.
-
-    >>> m.summary()
-    >>> stats = m.training_stats()
-
-    *Prediction and Evaluation*
-
-    With the trained model we can make predictions for new data in SFrame
-    ``sf_new`` and compare these predictions to actual target values, if these
-    values are known. For both of these methods, the new SFrame must include
-    columns with the same names as the training features; the ``evaluate``
-    method requires ``sf_new`` to have the same ``target`` column name as well.
-
-    >>> predictions = m.predict(sf_new)
-    >>> results = m.evaluate(sf_new)
-
-    *Saving and Loading*
-
-    The model can be saved to disk for later use.
-
-    >>> m.save("mymodel")
-    >>> m = graphlab.load_model("mymodel")
-
-    For more, see the documentation for
-    :class:`~graphlab.linear_regression.LinearRegressionModel`.
 
     References
     ----------
@@ -257,9 +261,41 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
 
     - `Vowpal Wabbit on Github
       <https://github.com/JohnLangford/vowpal_wabbit/wiki>`_
+
+    Examples
+    --------
+
+    Given an :class:`~graphlab.SFrame` ``sf`` with a list of columns
+    [``feature_1`` ... ``feature_K``] denoting features and a target column
+    ``target``, we can create a
+    :class:`~graphlab.linear_regression.LinearRegressionModel` as follows:
+
+    >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+    >>> model = graphlab.linear_regression.create(data, target='price', features=['bath', 'bedroom', 'size'])
+    
+
+    For ridge regression, we can set the ``L2_penalty`` parameter higher (the
+    default is 0.01). For Lasso regression, we set the L1_penalty higher, and
+    for elastic net, we set both to be higher.
+
+    .. sourcecode:: python
+
+      # Ridge regression
+      >>> model_ridge = graphlab.linear_regression.create(data, 'price', L2_penalty=0.1)
+      
+      # Lasso
+      >>> model_lasso = graphlab.linear_regression.create(data, 'price', L2_penalty=0.,
+                                                                   L1_penalty=1.0)
+
+      # Elastic net regression
+      >>> model_enet  = graphlab.linear_regression.create(data, 'price', L2_penalty=0.5,
+                                                                 L1_penalty=0.5)
+
     """
 
-    print "Preparing the data..."
+    if verbose:
+      print "Preparing the data..."
     _mt._get_metric_tracker().track('toolkit.regression.linear_regression.create')
 
     if not (isinstance(dataset, _SFrame) or (_HAS_PANDAS and isinstance(dataset, _pandas.DataFrame))):
@@ -271,6 +307,8 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
     # Regression model names.
     model_name = "regression_linear_regression"
     solver = solver.lower()
+    if solver == 'vw' or solver == 'vowpal_wabbit':
+        _vw_solver_warning()
 
     # Make sure all keys in the dictionary are lower case.
     # Also keeps a separate copy of solver_options to prevent changes.
@@ -306,13 +344,11 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
                 'l2_penalty'  : L2_penalty})
 
     # Pre-training
-    print "Starting pre-training..."
     ret = _graphlab.toolkits.main.run("supervised_learning_train_init", opts)
     opts.update(ret)
 
     # Switching point for vw
     if(solver == 'vw' or solver == 'vowpal-wabbit'):
-
         # Make the SFrame into VW format
         required_columns = features + [target]
         sf = dataset.select_columns(required_columns)
@@ -346,14 +382,12 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
     else:
 
         # Print some output for ipython notebook users
-        print "Starting training..."
         ret = _graphlab.toolkits.main.run("supervised_learning_train", opts,
                 verbose)
 
         model_proxy = ret['model']
         model = LinearRegressionModel(model_proxy)
 
-    model.summary()
     return model
 
 
@@ -365,38 +399,9 @@ class LinearRegressionModel(SupervisedLearningModel):
     :func:`graphlab.linear_regression.create`. Do not construct the model
     directly.
 
-
-    **Examples**
-
-
-    .. sourcecode:: python
-
-        # Load the data (From an S3 bucket)
-        >>> import graphlab as gl
-        >>> data =  gl.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
-
-        # Make a linear regression model
-        >>> model = gl.linear_regression.create(data, target='expensive', features=['bath', 'bedroom', 'size'])
-
-        # Here, all columns (other than the target) are used as features.
-        >>> model = gl.linear_regression.create(data, target='expensive')
-
-        # Extract the coefficients
-        >>> coefficients = model['coefficients']     # an SFrame
-        
-        # Make predictions (as margins, probability, or class)
-        >>> predictions = model.predict(data)    # Predicts 0/1
-        >>> predictions = model.predict(data, output_type='probability')
-        >>> predictions = model.predict(data, output_type='margin')       
-
-        # Evaluate the model 
-        >>> results = m.evaluate(data)               # a dictionary
-    
-    **Matematical Details**
-    
-    Given a set of features :math:`x_i`, and a label :math:`y_i`,
-    linear regression interprets the probability that the label is in one class
-    as a linear function of a linear combination of the features.
+    Given a set of features :math:`x_i`, and a label :math:`y_i`, linear
+    regression interprets the probability that the label is in one class as
+    a linear function of a linear combination of the features.
 
         .. math::
           f_i(\\theta) =  \\theta^T x + \epsilon_i
@@ -404,8 +409,8 @@ class LinearRegressionModel(SupervisedLearningModel):
     where :math:`\epsilon_i` is noise.  An intercept term is added by appending
     a column of 1's to the features.  Regularization is often required to
     prevent overfitting by penalizing models with extreme parameter values. The
-    linear regression module supports L1 and L2 regularization, which are
-    added to the loss function.
+    linear regression module supports L1 and L2 regularization, which are added
+    to the loss function.
     
     The composite objective being optimized for is the following:
 
@@ -415,75 +420,32 @@ class LinearRegressionModel(SupervisedLearningModel):
     where :math:`\lambda_1` is the ``L1_penalty`` and :math:`\lambda_2` is the
     ``L2_penalty``.
 
-    **Model creation options guide**
 
+    Examples
+    --------
+    .. sourcecode:: python
 
-    solver: string, default = 'auto'
+        # Load the data (From an S3 bucket)
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
 
-        The optimization used to train the model is a carefully engineered
-        collection of methods that are automatically picked based on the input
-        data. The ``newton`` method  works best for datasets with plenty of
-        examples and few features (long datasets). Limited memory BFGS
-        (``lbfgs``) is a robust solver for wide datasets (i.e datasets with
-        many coefficients).  ``fista'' is the default solver for L1-regularized
-        linear regression. Gradient-descent (GD) is another well tuned methods
-        that can work really well on L1-regularized problems.  The solvers are
-        all automatically tuned and the default options should function well.
-        See the solver options guide for setting additional parameters for each
-        of the solvers.
+        # Make a linear regression model
+        >>> model = graphlab.linear_regression.create(data, target='price', features=['bath', 'bedroom', 'size'])
+
+        # Here, all columns (other than the target) are used as features.
+        >>> model = graphlab.linear_regression.create(data, target='price')
+
+        # Extract the coefficients
+        >>> coefficients = model['coefficients']   
+        
+        # Make predictions (as margins, probability, or class)
+        >>> predictions = model.predict(data)      
+
+        # Evaluate the model 
+        >>> results = model.evaluate(data)         
     
-
-    feature_rescaling: boolean, default = true
-
-        Feature rescaling is an important pre-processing step that ensures
-        that all features are on the same scale. An L2-norm rescaling is
-        performed to make sure that all features are of the same norm. Categorical
-        features are also rescaled by rescaling the dummy variables that 
-        are used to represent them. The coefficients are returned in original
-        scale of the problem.
-
-
-    **Solver Options Guide**
-
-    Most solvers are all automatically tuned and the default options should
-    function well. Changing the default settings may help in some cases.
-    
-
-    step_size : double, default = 1.0 (fista only)
-
-        The starting step size to use for the ``fista'' and ``gd'' solvers. The
-        default is set to 1.0, this is an aggressive setting. If the first
-        iteration is taking a considerable amount of time, then reduce this
-        step size to speed up model training. If the first iteration is very
-        fast, then consider increasing this step size to a more aggressive
-        value.
-
-    max_iterations : integer, default = 10.
-
-        The maximum number of passes through the data allowed. Increasing his 
-        can result in more accurate training time. If the training accuracy 
-        is low and the ``Grad-Norm`` in the display is large, then the model 
-        has not trained completely.
-
-    convergence_threshold : double, default = 1e-2.
-
-        Convergence is tested using variation in the training objective.  The
-        variation in the training objective is calculated by dividing the
-        difference between the objective values between two step. 
-    
-    lbfgs_memory_level: integer, default = 10.
-
-        The L-BFGS algorithm keeps track of gradient information from the
-        previous ``lbfgs_memory_level`` iterations. The storage requirement for
-        each of the gradients is the number of ``coefficients`` in the problem.
-        Increasing the ``lbfgs_memory_level`` can help improve the quality of
-        the model trained at the cost of more memory. For problems with fewer
-        coefficients (less than a few million), increasing the
-        ``lbfgs_memory_level`` is recommended for better quality of the
-        solution. Setting this to more than ``max_iterations`` has no more
-        benefits as setting it to the same as the ``max_iterations``.
-
-
+    See Also
+    --------
+    create
 
     """
     def __init__(self, model_proxy, use_vw=False):
@@ -575,12 +537,7 @@ class LinearRegressionModel(SupervisedLearningModel):
         detailed below, and can be obtained programmatically using the
         :func:`~graphlab.linear_regression.LinearRegressionModel.list_fields`
         method.
-
-        Each of these fields can be queried in one of two ways:
-
-        >>> out = m['field']
-        >>> out = m.get('field')  # equivalent to previous line
-
+        
         +-----------------------+----------------------------------------------+
         |      Field            | Description                                  |
         +=======================+==============================================+
@@ -590,6 +547,8 @@ class LinearRegressionModel(SupervisedLearningModel):
         +-----------------------+----------------------------------------------+
         | convergence_threshold | Desired solver accuracy                      |
         +-----------------------+----------------------------------------------+
+        | feature_rescaling     | Bool indicating l2-rescaling of features     |
+        +-----------------------+----------------------------------------------+
         | features              | Feature column names                         |
         +-----------------------+----------------------------------------------+
         | l1_penalty            | L1 regularization weight                     |
@@ -625,6 +584,7 @@ class LinearRegressionModel(SupervisedLearningModel):
         | train_time            | Training time (excludes preprocessing)       |
         +-----------------------+----------------------------------------------+
 
+
         Parameters
         ----------
         field : string
@@ -634,6 +594,24 @@ class LinearRegressionModel(SupervisedLearningModel):
         -------
         out : [various]
             The current value of the requested field.
+
+        See Also
+        --------
+        list_fields
+
+        Examples
+        --------
+
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> model = graphlab.linear_regression.create(data,
+                                             target='price',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> print model['num_features']
+        3
+        >>> print model.get('num_features')       # equivalent to previous line
+        3
+
         """
 
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.get')
@@ -645,7 +623,7 @@ class LinearRegressionModel(SupervisedLearningModel):
                 raise ValueError("Models trained with 'vw' as the "\
                                 "solver cannot provide coefficients.")
             vw_option_map = {'solver'     : 'solver',
-                             'target'     : 'target_column',
+                             'target'     : 'target',
                              'step_size'  : 'step_size',
                              'l1_penalty' : 'l1_penalty',
                              'l2_penalty' : 'l2_penalty',
@@ -676,11 +654,18 @@ class LinearRegressionModel(SupervisedLearningModel):
 
     def summary(self):
         """
-        Display a summary of the model, coefficients (if applicable),
-        training options, and training statistics.  Use the
-        :func:`~graphlab.linear_regression.LinearRegressionModel.get` method
-        to retrieve these values programmatically, or to see more detail about
-        the queryable fields.
+        Display a summary of the LinearRegressionModel.
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> model = graphlab.linear_regression.create(data,
+                                             target='price',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> model.summary()
+
+
         """
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.summary')
         solver = self.get('solver')
@@ -716,12 +701,25 @@ class LinearRegressionModel(SupervisedLearningModel):
 
     def get_default_options(self):
         """
-        A dictionary describing the default options for the model.
+        A dictionary describing the default options for the LinearRegressionModel.
 
         Returns
         -------
         out : dict
              A dictionary with default option (name, value) pairs.
+        
+        See Also
+        --------
+        get_current_options, list_fields, get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> model = graphlab.linear_regression.create(data,
+                                             target='price',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> default_options = model.get_default_options()
         """
 
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.get_default_options')
@@ -739,6 +737,19 @@ class LinearRegressionModel(SupervisedLearningModel):
         out : dict
              A dictionary with option (name, value) pairs requested during
              train time.
+        
+        See Also
+        --------
+        get_current_options, list_fields, get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> model = graphlab.linear_regression.create(data,
+                                             target='price',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> current_options = model.get_current_options()
         """
 
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.get_options')
@@ -778,6 +789,19 @@ class LinearRegressionModel(SupervisedLearningModel):
         -------
         out : SArray
             Predicted target value for each example (i.e. row) in the dataset.
+        
+        See Also
+        ----------
+        create, evaluate
+
+        Examples
+        ----------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> model = graphlab.linear_regression.create(data,
+                                             target='price',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> results = model.predict(data)
         """
 
         _mt._get_metric_tracker().track(
@@ -845,10 +869,18 @@ class LinearRegressionModel(SupervisedLearningModel):
         out : dict
             Results from  model evaluation procedure.
 
-        References
+        See Also
         ----------
-        - `Wikipedia - root-mean-square deviation
-          <http://en.wikipedia.org/wiki/Root-mean-square_deviation>`_
+        create, predict
+
+        Examples
+        ----------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> model = graphlab.linear_regression.create(data,
+                                             target='price',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> results = model.evaluate(data)
         """
 
         _mt._get_metric_tracker().track(
@@ -869,47 +901,6 @@ class LinearRegressionModel(SupervisedLearningModel):
         opts.update(response)
         return _graphlab.toolkits.main.run('supervised_learning_get_evaluate_stats', opts)
 
-
-    def training_stats(self):
-        """
-        Returns a dictionary with (name, value) pairs with statistics collected
-        during training time. This includes:
-
-        Returns
-        -------
-        out : dict
-            Statistics about model training, e.g. runtime.
-
-            The following description provides details on each of fields
-            +-----------------------+------------------------------------------------------+
-            |      Field Name       | Description                                          |
-            +=======================+======================================================+
-            | solver_status         | Solver status during training.                       |
-            +-----------------------+------------------------------------------------------+
-            | step_size             | Initial step size requested during training.         |
-            +-----------------------+------------------------------------------------------+
-            | train_iters           | Total number of iterations required for training.    |
-            +-----------------------+------------------------------------------------------+
-            | train_loss            | Residual sum-of-squares training loss.               |
-            +-----------------------+------------------------------------------------------+
-            | train_rmse            | Training root-mean-squared-error (RMSE).             |
-            +-----------------------+------------------------------------------------------+
-            | train_time            | Time required for training (excludes preprocessing). |
-            +-----------------------+------------------------------------------------------+
-        """
-
-        _mt._get_metric_tracker().track('toolkit.regression.linear_regression.training_stats')
-        solver = self.get('solver')
-        # Hard code the options exposed from the VW model proxy.
-        if solver == 'vw':
-            training_stats_fields = ['train_iters',
-                    'train_rmse',
-                    'train_time']
-            return dict(zip(training_stats_fields,
-                        map(self.get, training_stats_fields)))
-        else:
-            return super(LinearRegressionModel, self).training_stats()
-
     def list_fields(self):
         """
         List of fields stored in the model. Each of these fields can be queried
@@ -920,6 +911,19 @@ class LinearRegressionModel(SupervisedLearningModel):
         -------
         out : list
             A list of fields that can be queried using the ``get`` method.
+        
+        See Also
+        --------
+        get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> model = graphlab.linear_regression.create(data,
+                                             target='price',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> model.list_fields()
         """
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.list_fields')
         solver = self.get('solver')
@@ -942,4 +946,3 @@ class LinearRegressionModel(SupervisedLearningModel):
                     'model_name': self.__name__}
             response = _graphlab.toolkits.main.run('supervised_learning_list_keys', opts)
             return sorted(response.keys())
-
