@@ -10,6 +10,7 @@ from graphlab import vowpal_wabbit as _vw
 from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
 import math
 from graphlab.toolkits.main import ToolkitError
+from graphlab.toolkits.supervised_learning import _vw_solver_warning
 
 DEFAULT_SOLVER_OPTIONS = {
 'convergence_threshold': 1e-2,
@@ -98,9 +99,24 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
         iteratively re-weighted least squares algorithm. If the L1_penalty is
         greater than 0, use either the 'fista' or 'vw' solver.
 
+        The model is trained using a carefully engineered collection of methods 
+        that are automatically picked based on the input data. The ``newton`` 
+        method  works best for datasets with plenty of examples and few features
+        (long datasets). Limited memory BFGS (``lbfgs``) is a robust solver for 
+        wide datasets (i.e datasets with many coefficients).  ``fista`` is the 
+        default solver for L1-regularized linear regression. The solvers are all
+        automatically tuned and the default options should function well. See 
+        the solver options guide for setting additional parameters for each of 
+        the solvers.
+
     feature_rescaling: boolean, optional
-        Feature rescaling is the process of scaling individual features to
-        have unit L2-norm. This process is particularly useful when features
+
+        Feature rescaling is an important pre-processing step that ensures that
+        all features are on the same scale. An L2-norm rescaling is performed
+        to make sure that all features are of the same norm. Categorical
+        features are also rescaled by rescaling the dummy variables that are
+        used to represent them. The coefficients are returned in original scale
+        of the problem. This process is particularly useful when features
         vary widely in their ranges.  (Not applicable when solver = 'vw')
 
     solver_options : dict, optional
@@ -118,6 +134,43 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
         | step_size             |    1.0  | Initial solver step size (fista only)   |
         +-----------------------+---------+-----------------------------------------+
 
+        Most solvers are all automatically tuned and the default options should
+        function well. Changing the default settings may help in some cases.
+    
+
+        convergence_threshold: 
+
+        Convergence is tested using variation in the training objective. The 
+        variation in the training objective is calculated using the difference 
+        between the objective values between two steps. Consider reducing this 
+        below the default value (0.01) for a more accurately trained model. 
+        Beware of overfitting (i.e a model that works well only on the training 
+        data) if this parameter is set to a very low value.
+        
+        lbfgs_memory_level: 
+
+        The L-BFGS algorithm keeps track of gradient information from the 
+        previous ``lbfgs_memory_level`` iterations. The storage requirement for 
+        each of these gradients is the ``num_coefficients`` in the problem. 
+        Increasing the ``lbfgs_memory_level ``can help improve the quality of 
+        the model trained. Setting this to more than ``max_iterations`` has the 
+        same effect as setting it to ``max_iterations``.     
+
+        max_iterations:
+
+        The maximum number of allowed passes through the data. More passes over 
+        the data can result in a more accurately trained model. Consider 
+        increasing this (the default value is 10) if the training accuracy is 
+        low and the *Grad-Norm* in the display is large.
+
+        step_size:
+
+        The starting step size to use for the ``fista`` and ``gd`` solvers. The
+        default is set to 1.0, this is an aggressive setting. If the first
+        iteration takes a considerable amount of time, reducing this parameter
+        may speed up model training. 
+
+
     verbose : bool, optional
         If True, print progress updates.
 
@@ -126,6 +179,10 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
     out : LogisticRegressionModel
         A trained model of type
         :class:`~graphlab.logistic_regression.LogisticRegressionModel`.
+    
+    See Also
+    --------
+    LogisticRegressionModel
 
     Notes
     -----
@@ -149,76 +206,7 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
     
     - For solver='vw', the "step_size" option is the "learning_rate" parameter
       in the  :class:`~graphlab.vowpal_wabbit.VWModel`. 
-
-    Examples
-    --------
-    *Training*
-
-    Given an :class:`~graphlab.SFrame` ``sf``, a list of feature columns
-    [``feature_1`` ... ``feature_K``], and a target column ``target`` with 0 and
-    1 values, create a
-    :class:`~graphlab.logistic_regression.LogisticRegressionModel` as follows:
-
-    >>> m = logistic_regression.create(sf, 'target')
-
-    By default all columns of ``sf`` except the target are used as features, but
-    specific feature columns can be specified manually.
-
-    >>> m = logistic_regression.create(sf, 'target', ['feature_1', 'feature_2'])
-
-    The L2 and L1 regularization weights are set to 0.01 and 0.0 by default. For
-    stronger regularization raise one or both of these weights.
-
-    >>> m = logistic_regression.create(sf, 'target', L2_penalty=0.5,
-                                       L1_penalty=1.)
-
-    *Model Querying*
-
-    Model attributes can be retrieved with either bracket syntax or the
-    :func:`~graphlab.logistic_regression.LogisticRegressionModel.get` method. The
-    set of queryable fields is described in the ``get`` documentation, and
-    can be obtained programmatically with the
-    :func:`~graphlab.logistic_regression.LogisticRegressionModel.list_fields`
-    method. The model coefficients are stored in an SFrame.
-
-    >>> m.list_fields()
-    >>> coef = m['coefficients']  # an SFrame
-    >>> coef = m.get('coefficients')  # equivalent to previous line
-
-    The :func:`~graphlab.logistic_regression.LogisticRegressionModel.summary`
-    and
-    :func:`~graphlab.logistic_regression.LogisticRegressionModel.training_stats`
-    methods provide shortcuts to many of the model attributes.
-
-    >>> print m.summary()
-    >>> m.training_stats()
-
-    *Prediction and Evaluation*
-
-    With the trained model we can make predictions for new data in SFrame
-    ``sf_new`` and compare these predictions to actual target values, if these
-    values are known. For both of these methods, the new SFrame must include
-    columns with the same names as the training features; the ``evaluate``
-    method requires ``sf_new`` to have the same ``target`` column name as well.
-
-    Predictions can be generated as class labels (0 or 1), probabilities that
-    the target value is 1 (rather than 0), or margins (i.e. the distance of the
-    observations from the hyperplane separating the classes). By default, the
-    predict method returns probabilities.
-
-    >>> pred_probs = m.predict(sf_new)  # class probabilities P(Y=1)
-    >>> pred_margin = m.predict(sf_new, output_type='margin')
-    >>> pred_class = m.predict(sf_new, output_type='class')
-
-    *Save and Load*
-
-    The model can be saved to disk and loaded at a later time.
-
-    >>> m.save("mymodel")
-    >>> m2 = graphlab.load_model("mymodel")
-
-    For more detail, see the documentation for individual methods of the
-    :class:`~graphlab.logistic_regression.LogisticRegressionModel`.
+    
 
     References
     ----------
@@ -249,9 +237,41 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
 
     - `Vowpal Wabbit on Github
       <https://github.com/JohnLangford/vowpal_wabbit/wiki>`_
+
+
+    Examples
+    --------
+
+    Given an :class:`~graphlab.SFrame` ``sf``, a list of feature columns
+    [``feature_1`` ... ``feature_K``], and a target column ``target`` with 0 and
+    1 values, create a
+    :class:`~graphlab.logistic_regression.LogisticRegressionModel` as follows:
+
+    >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+    >>> data['is_expensive'] = data['price'] > 30000
+    >>> model = graphlab.logistic_regression.create(data, 'is_expensive')
+
+    By default all columns of ``data`` except the target are used as features, but
+    specific feature columns can be specified manually.
+
+    >>> model = graphlab.logistic_regression.create(data, 'is_expensive', ['bedroom', 'size'])
+
+
+    .. sourcecode:: python
+
+      # L2 normalization
+      >>> model_ridge = graphlab.logistic_regression.create(data, 'is_expensive', L2_penalty=0.1)
+      
+      # L1 normalization
+      >>> model_lasso = graphlab.logistic_regression.create(data, 'is_expensive', L2_penalty=0.,
+                                                                   L1_penalty=1.0)
+
+      # L1 and L2 normalization
+      >>> model_enet  = graphlab.logistic_regression.create(data, 'is_expensive', L2_penalty=0.5, L1_penalty=0.5)
     """
 
-    print "Preparing the data..."
+    if verbose:
+      print "Preparing the data..."
     model_name = "regression_logistic_regression"
     _mt._get_metric_tracker().track(
         'toolkit.regression.logistic_regression.create')
@@ -268,6 +288,8 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
     # Make the solver name and all solver options lower case. Also keeps a
     # separate copy of solver_options to prevent changes.
     solver = solver.lower()
+    if solver == 'vw' or solver == 'vowpal_wabbit':
+        _vw_solver_warning()
 
     if solver_options is not None:
         _solver_options = {k.lower(): v for k,v in solver_options.items()}
@@ -320,7 +342,6 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
 
 
     if solver == 'vw' or solver == 'vowpal_wabbit':
-
         # Make the SFrame into VW format
         required_columns = features + [target]
         sf = dataset.select_columns(required_columns)
@@ -363,7 +384,6 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
             model_proxy = ret['model']
             model = LogisticRegressionModel(model_proxy)
 
-    model.summary()
     return model
 
 
@@ -382,39 +402,9 @@ class LogisticRegressionModel(SupervisedLearningModel):
     Logistic regression predictions may take the form of the linear combination
     (also known as the margin), the probability that :math:`y = 1`, or a binary
     class value, which is obtained by thresholding the probability at 0.5. The
-    :class:`~graphlab.evaluation` toolkit gives you more flexible evaluation metrics.
+    :class:`~graphlab.evaluation` toolkit gives you more flexible evaluation
+    metrics.
     
-
-    **Examples**
-
-
-    .. sourcecode:: python
-
-        # Load the data (From an S3 bucket)
-        >>> import graphlab as gl
-        >>> data =  gl.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
-
-        # Make sure the target is binary 0/1
-        >>> data['expensive'] = data['price'] > 30000
-        
-        # Make a logistic regression model
-        >>> model = gl.logistic_regression.create(data, target='expensive', features=['bath', 'bedroom', 'size'])
-
-        # Here, all columns (other than the target) are used as features.
-        >>> model = gl.logistic_regression.create(data, target='expensive')
-
-        # Extract the coefficients
-        >>> coefficients = model['coefficients']     # an SFrame
-        
-        # Make predictions (as margins, probability, or class)
-        >>> predictions = model.predict(data)    # Predicts 0/1
-        >>> predictions = model.predict(data, output_type='probability')
-        >>> predictions = model.predict(data, output_type='margin')       
-
-        # Evaluate the model 
-        >>> results = m.evaluate(data)               # a dictionary
-    
-    **Matematical Details**
     
     Given a set of features :math:`x_i`, and a label :math:`y_i \in \{0,1\}`,
     logistic regression interprets the probability that the label is in one class
@@ -436,70 +426,33 @@ class LogisticRegressionModel(SupervisedLearningModel):
     where :math:`\lambda_1` is the ``L1_penalty`` and :math:`\lambda_2` is the
     ``L2_penalty``.
 
-    **Model creation options guide**
+    Examples
+    --------
+    .. sourcecode:: python
 
+        # Load the data (From an S3 bucket)
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
 
-    solver: string, default = 'auto'
+        # Make sure the target is binary 0/1
+        >>> data['is_expensive'] = data['price'] > 30000
+        
+        # Make a logistic regression model
+        >>> model = graphlab.logistic_regression.create(data, target='is_expensive', features=['bath', 'bedroom', 'size'])
 
-        The optimization used to train the model is a carefully engineered
-        collection of methods that are automatically picked based on the input
-        data. The ``newton`` method  works best for datasets with plenty
-        of examples and few features (long datasets). Limited memory BFGS
-        (``lbfgs``) is a robust solver for wide datasets (i.e datasets with many
-        coefficients).  ``fista'' is the default solver for L1-regularized logistic
-        regression.  The solvers are all automatically tuned and the default
-        options should function well. See the solver options guide for setting
-        additional parameters for each of the solvers.
+        # Extract the coefficients
+        >>> coefficients = model['coefficients']
+        
+        # Make predictions (as margins, probability, or class)
+        >>> predictions = model.predict(data)    
+        >>> predictions = model.predict(data, output_type='probability')
+        >>> predictions = model.predict(data, output_type='margin')       
+
+        # Evaluate the model 
+        >>> results = model.evaluate(data)
     
-
-    feature_rescaling: string, default = true
-
-        Feature rescaling is an important pre-processing step that ensures
-        that all features are on the same scale. An L2-norm rescaling is
-        performed to make sure that all features are of the same norm. Categorical
-        features are also rescaled by rescaling the dummy variables that 
-        are used to represent them. The coefficients are returned in original
-        scale of the problem.
-
-
-    **Solver Options Guide**
-
-    Most solvers are all automatically tuned and the default options should
-    function well. Changing the default settings may help in some cases.
-    
-
-    step_size : double, default = 1.0 (fista only)
-
-        The starting step size to use for the ``fista'' solver. The default is set to 
-        1.0, this is an aggressive setting. If the first iteration is taking 
-        a considerable amount of time, then reduce this step size to speed
-        up model training. If the first iteration is very fast, then consider
-        increasing this step size to a more aggressive value.
-
-    max_iterations : integer, default = 10.
-
-        The maximum number of passes through the data allowed. Increasing his 
-        can result in more accurate training time. If the training accuracy 
-        is low and the ``Grad-Norm`` in the display is large, then the model 
-        has not trained completely.
-
-    convergence_threshold : double, default = 1e-2.
-
-        Convergence is tested using variation in the training objective.  The
-        variation in the training objective is calculated by dividing the
-        difference between the objective values between two step. 
-    
-    lbfgs_memory_level: integer, default = 10.
-
-        The L-BFGS algorithm keeps track of gradient information from the
-        previous ``lbfgs_memory_level`` iterations. The storage requirement for
-        each of the gradients is the number of ``coefficients`` in the problem.
-        Increasing the ``lbfgs_memory_level`` can help improve the quality of the
-        model trained at the cost of more memory. For problems with fewer
-        coefficients (less than a few million), increasing the
-        ``lbfgs_memory_level`` is recommended for better quality of the
-        solution. Setting this to more than ``max_iterations`` has no more
-        benefits as setting it to the same as the ``max_iterations``.
+    See Also
+    --------
+    create
 
 
     """
@@ -588,11 +541,6 @@ class LogisticRegressionModel(SupervisedLearningModel):
         method. Note that some options are not available if the model is trained
         with the ``vw`` solver.
 
-        Each of these fields can be queried in one of two ways:
-
-        >>> out = m['field']
-        >>> out = m.get('field')  # equivalent to previous line
-
         +-----------------------+----------------------------------------------+
         |      Field            | Description                                  |
         +=======================+==============================================+
@@ -600,6 +548,8 @@ class LogisticRegressionModel(SupervisedLearningModel):
         +-----------------------+----------------------------------------------+
         | convergence_threshold | Desired solver accuracy                      |
         +-----------------------+----------------------------------------------+
+        | feature_rescaling     | Bool indicating l2-rescaling of features     |
+        +-----------------------+----------------------------------------------+
         | features              | Feature column names                         |
         +-----------------------+----------------------------------------------+
         | l1_penalty            | L1 regularization weight                     |
@@ -630,6 +580,8 @@ class LogisticRegressionModel(SupervisedLearningModel):
         +-----------------------+----------------------------------------------+
         | train_time            | Training time (excludes preprocessing)       |
         +-----------------------+----------------------------------------------+
+        | trained               | Indicates whether model is trained           |
+        +-----------------------+----------------------------------------------+
 
         Parameters
         ----------
@@ -640,6 +592,26 @@ class LogisticRegressionModel(SupervisedLearningModel):
         -------
         out
             Value of the requested fields.
+
+        See Also
+        --------
+        list_fields
+
+        Examples
+        --------
+
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.logistic_regression.create(data,
+                                             target='is_expensive',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> print model['num_features']
+        3
+        >>> print model.get('num_features')       # equivalent to previous line
+        3
+
+
         """
 
         _mt._get_metric_tracker().track(
@@ -653,7 +625,7 @@ class LogisticRegressionModel(SupervisedLearningModel):
                 raise ValueError("Models trained with 'vw' as the" + \
                                 "solver cannot provide coefficients.")
             vw_option_map = {'solver'         : 'solver',
-                             'target'         : 'target_column',
+                             'target'         : 'target',
                              'step_size'      : 'step_size',
                              'l1_penalty'     : 'l1_penalty',
                              'l2_penalty'     : 'l2_penalty',
@@ -682,11 +654,17 @@ class LogisticRegressionModel(SupervisedLearningModel):
 
     def summary(self):
         """
-        Display a summary of the model, including coefficients (if applicable),
-        training options, and training statistics. Use the
-        :func:`~graphlab.logistic_regression.LogisticRegressionModel.get` method
-        to retrieve these values programmatically, or to see more detail about
-        the queryable fields.
+        Display a summary of the LogisticRegressionModel.
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.logistic_regression.create(data,
+                                             target='is_expensive',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> model.summary()
         """
         _mt._get_metric_tracker().track(
             'toolkit.regression.logistic_regression.summary')
@@ -729,6 +707,20 @@ class LogisticRegressionModel(SupervisedLearningModel):
         -------
         out : dict
             Dictionary with model default options.
+        
+        See Also
+        --------
+        get_current_options, list_fields, get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.logistic_regression.create(data,
+                                             target='is_expensive',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> default_options = model.get_default_options()
         """
         _mt._get_metric_tracker().track('toolkit.regression.logistic_regression.get_default_options')
         if self.get('solver') == 'vw':
@@ -744,6 +736,20 @@ class LogisticRegressionModel(SupervisedLearningModel):
         -------
         out : dict
             Dictionary with options used to define and train the model.
+
+        See Also
+        --------
+        get_default_options, list_fields, get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.logistic_regression.create(data,
+                                             target='is_expensive',
+                                             features=['bath', 'bedroom', 'size'])
+        >>> current_options = model.get_current_options()
         """
         if self.get('solver') == 'vw':
             max_iterations = _vw.VWModel(self.__proxy__).get('max_iterations')
@@ -794,6 +800,24 @@ class LogisticRegressionModel(SupervisedLearningModel):
         -------
         out : SArray
             An SArray with model predictions.
+
+        See Also
+        ----------
+        create, evaluate
+
+        Examples
+        ----------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.logistic_regression.create(data,
+                                             target='is_expensive',
+                                             features=['bath', 'bedroom', 'size'])
+
+        >>> probability_predictions = model.predict(data)
+        >>> margin_predictions = model.predict(data, output_type='margin')
+        >>> class_predictions = model.predict(data, output_type='class')
+
         """
 
         _mt._get_metric_tracker().track(
@@ -880,10 +904,22 @@ class LogisticRegressionModel(SupervisedLearningModel):
             Dictionary of evaluation results. The dictionary keys are *accuracy*
             and *confusion_table*.
 
-        References
+        See Also
         ----------
-        - `Wikipedia - confusion matrix
-          <http://en.wikipedia.org/wiki/Confusion_matrix>`_
+        create, predict
+
+        Examples
+        ----------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.logistic_regression.create(data,
+                                             target='is_expensive',
+                                             features=['bath', 'bedroom', 'size'])
+
+        >>> results = model.evaluate(data)
+        >>> print results['accuracy']
+
         """
 
         _mt._get_metric_tracker().track(
@@ -935,24 +971,6 @@ class LogisticRegressionModel(SupervisedLearningModel):
                     'confusion_table': confusion_matrix}
 
 
-    def training_stats(self):
-        """
-        Return a dictionary containing statistics collected during model
-        training. These statistics are also available with the ``get`` method,
-        and are described in more detail in the documentation for that method.
-        """
-        _mt._get_metric_tracker().track(
-            'toolkit.regression.logistic_regression.training_stats')
-
-        solver = self.get('solver')
-        # Hard code the options exposed from the VW model proxy.
-        if solver == 'vw':
-            training_stats_fields = ['train_iters', 'train_time']
-            return dict(zip(training_stats_fields,
-                        map(self.get, training_stats_fields)))
-        else:
-            return super(LogisticRegressionModel, self).training_stats()
-
     def list_fields(self):
         """
         List the fields stored in the model, including data, model, and training
@@ -963,6 +981,21 @@ class LogisticRegressionModel(SupervisedLearningModel):
         -------
         out : list
             List of fields queryable with the ``get`` method.
+        
+        See Also
+        --------
+        get
+
+        Examples
+        --------
+        >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        >>> data['is_expensive'] = data['price'] > 30000
+        >>> model = graphlab.logistic_regression.create(data,
+                                             target='is_expensive',
+                                             features=['bath', 'bedroom', 'size'])
+
+        >>> model.list_fields()
         """
 
         _mt._get_metric_tracker().track(
@@ -984,3 +1017,4 @@ class LogisticRegressionModel(SupervisedLearningModel):
                     'model_name': self.__name__}
             response = _graphlab.toolkits.main.run('supervised_learning_list_keys', opts)
             return sorted(response.keys())
+
