@@ -9,9 +9,14 @@ details. Also, see :py:func:`~graphlab.SFrame.pack` and
 :py:func:`~graphlab.SFrame.unstack` for ways of creating SArrays
 containing dictionary types.
 
-We provide methods for learning topic models, which can be
-useful for modeling large document collections.
-See :py:func:`~graphlab.toolkits.text.topic_model.create` for more.
+We provide methods for learning topic models, which can be useful for modeling
+large document collections. See
+:py:func:`~graphlab.toolkits.text.topic_model.create` for more, as well as the
+`How-Tos <http://www.graphlab.com/learn/how-to>`_, data science `Gallery
+<http://www.graphlab.com/learn/gallery>`_, and `text analysis chapter of
+the User Guide
+<http://www.graphlab.com/learn/userguide.html#Modeling_data_Text_analysis>`_.
+
 """
 
 import topic_model
